<?php
/**
 * Diafan.CMS
 *
 * @author     diafan.ru
 * @version    4.3
 * @license http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2011 OOO "Диафан". (http://diafan.ru)
 */

if(! defined('DIAFAN'))
{
	include(dirname(dirname(dirname(__FILE__))).'/includes/404.php');
}

/**
 * Frame_admin
 * 
 * Удаление элемента
 */
class Frame_admin
{
	/**
	 * @var object основной объект системы
	 */
	public $diafan;

	/**
	 * @var string таблица в базе данных
	 */
	public $table;

	/**
	 * @var array настройки отображения
	 */
	public $config_module = array();

	/**
	 * Конструктор класса
	 * 
	 * @return void
	 */
	public function __construct($diafan)
	{
		$this->diafan = $diafan;	
	}

	/**
	 * Удаляет элемент
	 * 
	 * @return boolean
	 */
	public function del_module()
	{		
		//проверка прав пользователя на удаление
		if($this->diafan->admin_roles('del'))
		{
			$this->diafan->goto_diafan(URL);
			return FALSE;
		}
		
		//в файле модуля можно определить функцию del_module_название_модуля,
		//которая будет выполняться перед удалением элемента
		//и функцию del_module, которая будет выполняться вместо основной функции удаления
		
		$this->diafan->rewrite = preg_replace('/[^A-Za-z0-9]+/', '', $this->diafan->rewrite);
		$func = 'del_module_'.preg_replace('/[^A-Za-z0-9_-]+/', '', $this->diafan->rewrite);
		if(method_exists($this, $func))
		{
			if($this->diafan->del || $this->diafan->trash)
			{
				$this->$func();
			}
			elseif(! empty($_GET["ids"]))
			{
				foreach($_GET["ids"] as $id)
				{
					if($_GET["action"] == "trash")
					{
						$this->diafan->trash = intval($id);
					}
					else
					{
						$this->diafan->del = intval($id);
					}
					$this->$func();
				}
			}
		}

		if($this->diafan->trash)
		{
			$this->diafan->del = $this->diafan->trash;
		}
		$this->diafan->_cache->delete("", $this->diafan->module);

		if(! $this->diafan->trash)
		{
			$this->diafan->trash = $_GET["action"] == "trash" ? 1 : 0;
		}

		//тип данных - категория
		if($this->config_module["category"])
		{
			if($this->diafan->del)
			{
				$this->del_category($this->table, $this->diafan->del);
			}
			elseif(! empty($_GET["ids"]))
			{
				foreach($_GET["ids"] as $id)
				{
					$this->del_category($this->table, intval($id));
				}
			}
		}
		//тип данных - элемент
		else
		{
			if($this->diafan->del)
			{
				$this->del_element($this->table, $this->diafan->del);
			}
			elseif(! empty($_GET["ids"]))
			{
				foreach($_GET["ids"] as $id)
				{
					$this->del_element($this->table, intval($id));
				}
			}
		}
		$this->diafan->_cache->delete("", "rewrite");

		$this->diafan->goto_diafan(URL);
		return TRUE;
	}

	/**
	 * Удаляет категорию
	 * 
	 * @param string $table название таблицы
	 * @param integer $del_id номер удаляемой записи
	 * @param integer $trash_pid номер записи в корзине, с которой связано удаление
	 * @return boolean true
	 */
	private function del_category($table, $del_id, $trash_pid = 0)
	{
		$trash_id = $this->del_or_trash($table, $del_id, $trash_pid);

		$this->diafan->_cache->delete("", str_replace('_category', '', $table));

		$this->del_or_trash_where("rewrite", "cat_id='".$del_id."' AND module_name='".$table."'", $trash_id);

		if($this->diafan->configmodules("comments_cat"))
		{
			$this->del_or_trash_where("comments", "element_id='".$del_id."' AND module_name='".$table."'", $trash_id);
			$this->diafan->_cache->delete("","comments");
		}

		$this->del_or_trash_where("param","cat_id='".$del_id."' AND module_name='".str_replace('_category', '', $table)."'", $trash_id);
		
		if($this->config_module["parent"])
		{
			$result = DB::query("SELECT id FROM {".$table."} WHERE parent_id=%d", $del_id);

			while($row = DB::fetch_array($result))
			{
				$this->del_category($table, $row["id"], $trash_id);
			}
		}

		$table = str_replace("_category", '', $table);
		$result = DB::query("SELECT id FROM {".$table."} WHERE cat_id=%d", $del_id);
		
		while($row = DB::fetch_array($result))
		{
			$this->del_element($table, $row["id"], $trash_id);
		}
		return TRUE;
	}

	/**
	 * Удаляет элементы в категории
	 * 
	 * @param string $table название таблицы
	 * @param integer $del_id номер удаляемой записи
	 * @param integer $trash_pid номер записи в корзине, с которой связано удаление
	 * @return boolean true
	 */
	private function del_element($table, $del_id, $trash_pid = 0)
	{
		$trash_id = $this->del_or_trash($table, $del_id, $trash_pid);

		if($table == "site")
		{
			$this->del_or_trash_where("rewrite", "site_id='".$del_id."' AND module_name='".$table."'", $trash_id);
		}
		else
		{
			$this->del_or_trash_where("rewrite", "element_id='".$del_id."' AND module_name='".$table."'", $trash_id);
		}
		
		$this->diafan->_cache->delete("", $table);

		if (is_dir(ABSOLUTE_PATH.USERFILES."/".$table))
		{
			$this->del_photo($table, $del_id, $trash_id);
			$this->del_files($table, $del_id, $trash_id);
		} 

		if($this->diafan->configmodules("comments"))
		{
			$this->del_or_trash_where("comments", "element_id='".$del_id."' AND module_name='".$table."'", $trash_id);
			$this->diafan->_cache->delete("", "comments");
		}

		if($this->diafan->configmodules("rating"))
		{
			$this->del_or_trash_where("rating","element_id='".$del_id."' AND module_name='".$table."'", $trash_id);
			$this->diafan->_cache->delete("","rating");
		}

		if($this->diafan->configmodules("tags"))
		{
			$this->del_or_trash_where("tags", "element_id='".$del_id."' AND module_name='".$table."'", $trash_id);
			$this->diafan->_cache->delete("", "tags");
		}

		if($table == "tags_name")
		{
			$this->del_or_trash_where("tags", "tags_name_id=".$del_id, $trash_id);
		}

		if($table == "param")
		{
			$this->del_or_trash_where("param_element", "param_id=".$del_id, $trash_id);
			$this->del_or_trash_where("param_select",  "param_id=".$del_id, $trash_id);
		}

		if($table == "shop_order")
		{
			$this->del_or_trash_where("shop_goods", "order_id=".$del_id, $trash_id);
		}

		$this->del_or_trash_where("param_element","element_id=".$del_id." AND module_name='".$table."'", $trash_id);

		if($this->config_module["parent"] && ! $this->config_module["category"])
		{
			$result = DB::query("SELECT id FROM {".$table."} WHERE parent_id=%d", $del_id);

			while($row = DB::fetch_array($result))
			{
				$this->del_element($table, $row["id"], $trash_id);
			}
		}
		return TRUE;
	}

	/**
	 * Помечает строку на удаление или удаляет строку из базы данных
	 * 
	 * @param string $table название таблицы
	 * @param integer $del_id номер удаляемой записи
	 * @param integer $trash_id номер записи в корзине, с которой связано удаление
	 * @return integer|boolean true
	 */
	protected function del_or_trash($table, $del_id, $trash_id)
	{
		if($this->diafan->trash)
		{
			DB::query("UPDATE {".$table."} SET trash='1' WHERE id=%d", $del_id);
			DB::query("INSERT INTO {trash} (table_name,module_name,element_id,created,parent_id,user_id) VALUES ('%s','%s','%d','%d','%d','%d')",
				  $table,
				  $this->diafan->module,
				  $del_id,
				  time(),
				  $trash_id,
				  $this->diafan->_user->id
				);
			return DB::next_id("trash");
		}
		else
		{
			DB::query("DELETE FROM {trash} WHERE element_id=%d AND table_name='%s'", $del_id, $table);
			DB::query("DELETE FROM {".$table."} WHERE id=%d", $del_id);
		}
		return TRUE;
	}

	/**
	 * Помечает строки на удаление или удаляет строки из базы данных
	 * 
	 * @param string $table название таблицы
	 * @param string $del_where SQL-условие для удаление записей в базе данных
	 * @param integer $trash_id номер записи в корзине, с которой связано удаление
	 * @return boolean true
	 */
	protected function del_or_trash_where($table, $del_where, $trash_id)
	{
		if($this->diafan->trash)
		{	
			$result = DB::query("SELECT id FROM {".$table."} WHERE ".$del_where." AND trash='0'");
			while($row = DB::fetch_array($result))
			{
				DB::query("INSERT INTO {trash} (table_name,module_name,element_id,created,parent_id,user_id) VALUES ('%s','%s','%d','%d','%d','%d')"
					     ,$table
					     ,$table
					     ,$row["id"]
					     ,time()
					     ,$trash_id
					     ,$this->diafan->_user->id
					);
			}
			DB::query("UPDATE {".$table."} SET trash='1' WHERE ".$del_where);
			
		}
		else
		{
			$del_ids = DB::query_result("SELECT GROUP_CONCAT(id SEPARATOR ',') FROM {".$table."} WHERE ".$del_where);
			DB::query("DELETE FROM {trash} WHERE element_id in (%s) AND table=%h", $del_ids, $table);

			DB::query("DELETE FROM {".$table."} WHERE ".$del_where);
		}
		return TRUE;
	}

	/**
	 * Удаляет элементы из корзины
	 * 
	 * @param string $table название таблицы
	 * @param integer $id номер удаляемого элемента
	 * @param integer $trash_id номер записи в корзине, с которой связано удаление
	 * @return boolean true
	 */
	protected function del_from_trash($table, $id, $trash_id)
	{
		if($table == "images")
		{
			$row = DB::fetch_array(DB::query("SELECT name, module_name FROM {images} WHERE id='%d'", $id));
			unlink(ABSOLUTE_PATH.USERFILES."/".$row["module_name"].'/'.$row["name"].'_'.$id.'.jpg');
			unlink(ABSOLUTE_PATH.USERFILES."/".$row["module_name"].'/medium/'.$row["name"].'_'.$id.'.jpg');
			unlink(ABSOLUTE_PATH.USERFILES."/".$row["module_name"].'/small/'.$row["name"].'_'.$id.'.jpg');
		}
		if($table == "attachment")
		{
			$row = DB::fetch_array(DB::query("SELECT name,module_name FROM {attachment} WHERE id='%d'", $id));
			unlink(ABSOLUTE_PATH.USERFILES."/".$row["module_name"]."/files/".$id);
		}

		DB::query("DELETE FROM {trash} WHERE id=%d", $trash_id);
		DB::query("DELETE FROM {".$table."} WHERE id=%d", $id);

		$result = DB::query("SELECT element_id,table_name,id FROM {trash} WHERE parent_id=%d", $trash_id);

		while($row = DB::fetch_array($result))
		{
			$this->del_from_trash($row["table_name"], $row["element_id"], $row["id"]);
		}
		return TRUE;
	}

	/**
	 * Помечает изображения на удаление или удаляет изображения
	 * 
	 * @param string $module_name название модуля, к которому прикреплены изображения
	 * @param integer $element_id номер элемента, к которому прикреплены изображения
	 * @param integer $trash_id номер записи в корзине, с которой связано удаление
	 * @return boolean true
	 */
	protected function del_photo($module_name, $element_id, $trash_id = 0)
	{
		if(! $this->diafan->trash)
		{
			$result = DB::query("select id,name from {images} where module_name='%s' AND element_id='%d'", $module_name, $element_id);
			while ($row = DB::fetch_array($result))
			{
				unlink(ABSOLUTE_PATH.USERFILES."/".$module_name.'/'.$row["name"].'_'.$row["id"].'.jpg');
				unlink(ABSOLUTE_PATH.USERFILES."/".$module_name.'/medium/'.$row["name"].'_'.$row["id"].'.jpg');
				unlink(ABSOLUTE_PATH.USERFILES."/".$module_name.'/small/'.$row["name"].'_'.$row["id"].'.jpg');
			}
		}
		$this->del_or_trash_where("images", "element_id='".$element_id."' AND module_name='".$module_name."'", $trash_id);
		return TRUE;
	}

	/**
	 * Помечает файлы на удаление или удаляет файлы
	 *
	 * @param string $module_name название модуля, к которому прикреплены файлы
	 * @param integer $element_id номер элемента, к которому прикреплены файлы
	 * @param integer $trash_id номер записи в корзине, с которой связано удаление
	 * @return boolean true
	 */
	protected function del_files($module_name, $element_id, $trash_id = 0)
	{
		if(is_dir(ABSOLUTE_PATH.USERFILES."/".$module_name."/files"))
		{
			if(! $this->diafan->trash)
			{
				$result = DB::query("SELECT id, name FROM {attachment} WHERE module_name='%s' AND element_id='%d'", $module_name, $element_id);
				while ($row = DB::fetch_array($result))
				{
					if(file_exists(ABSOLUTE_PATH.USERFILES."/".$module_name."/files/".$row["id"]))
					{
						unlink(ABSOLUTE_PATH.USERFILES."/".$module_name."/files/".$row["id"]);
					}						
				}
			}
			$this->del_or_trash_where("attachment", "element_id='".$element_id."' AND module_name='".$module_name."'", $trash_id);
		}
		return TRUE;
	}
	
}