<?php
/**
 * Diafan.CMS
 *
 * @author     diafan.ru
 * @version    4.3
 * @license http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2011 OOO "Диафан". (http://diafan.ru)
 */

if(! defined('DIAFAN'))
{
	include(dirname(dirname(dirname(__FILE__))).'/includes/404.php');
}

/**
 * Frame_admin
 * 
 * Сохранение элемента
 */
class Frame_admin
{
	/**
	 * @var object основной объект системы
	 */
	public $diafan;

	/**
	 * @var string таблица в базе данных
	 */
	public $table;

	/**
	 * @var array настройки отображения
	 */
	public $config_module = array();

	/**
	 * @var array поля таблицы с типами
	 */
	public $variable_table = array();

	/**
	 * @var array переводимые поля
	 */
	public $language_names = array();

	/**
	 * @var array массив полей таблицы для SQL-запроса на обновление
	 */
	protected $query;

	/**
	 * @var array массив новых значений для SQL-запроса на обновление
	 */
	protected $value;

	/**
	 * @var array массив старых значений обновляемых полей
	 */
	protected $oldrow = array();

	/**
	 * @var integer номер страницы, к которому прикреплен текущий элемент
	 */
	protected $site_id;

	/**
	 * Конструктор класса
	 * 
	 * @return void
	 */
	public function __construct($diafan)
	{
		$this->diafan = $diafan;
	}

	/**
	 * Добавляет элемент в базу данных
	 * 
	 * @return boolean
	 */
	public function new_module()
	{
		// Проверяет права на добавление
		if($this->diafan->admin_roles('addnew'))
		{
			$this->diafan->goto_diafan(URL);
			return FALSE;
		}

		// Проверяет, пришел ли запрос из формы добавления/редактирования
		if(empty($_POST["save_post"]))
		{
			$this->diafan->goto_diafan(URL);
			return FALSE;
		}

		// В файле модуля можно определить функцию new_module_название_модуля,
		// которая будет выполняться перед добавлением элемента
		// и функцию new_module, которая будет выполняться вместо основной функции удаления

		$func = 'new_module_'.preg_replace('/[^A-Za-z]+/', '', $this->diafan->rewrite);
		if(method_exists($this, $func))
		{
			$this->$func();
		}
		DB::query(
			"INSERT INTO {".$this->table."} (".($this->config_module["parent"] ? "parent_id" : '').") "
			."VALUES (".($this->config_module["parent"] ? "'%d'" : '').")",
			($this->config_module["parent"] ? $this->diafan->enterid : '')
		);

		$this->diafan->save = DB::next_id($this->table);

		if(! $this->diafan->save)
		{
			echo '<script language="javascript" type="text/javascript">alert(\''
			     .sprintf(_ADMIN_ERROR_INSERT_INTO,DB_PREFIX.$this->table)
			     .'\');history.go(-1);</script>';
			return FALSE;
		}

		$this->save_module();
		return TRUE;
	}

	/**
	 * Сохраняет изменения
	 * 
	 * @return boolean
	 */
	public function save_module()
	{
		// Проверяет, пришел ли запрос из формы добавления/редактирования
		if(empty($_POST["save_post"]))
		{
			$this->diafan->goto_diafan(URL);
			return FALSE;
		}

		// Проверка прав на сохранение
		if($this->diafan->admin_roles('save') && ! $this->diafan->savenew)
		{
			$this->diafan->goto_diafan(URL);
			return FALSE;
		}

		// Массив $this->oldrow содержит прежние значения полей для сохраняемого элемента
		if(! $this->diafan->config)
		{
			$this->oldrow = DB::fetch_array(DB::query("SELECT * FROM {".$this->table."} WHERE id = %d"
			                                          .($this->config_module["trash"] ? " AND trash='0'" : ''),
			                                          $this->diafan->save));
			if(! $this->oldrow)
			{
				$this->diafan->goto_diafan(URL);
				return FALSE;
			}
		}

		// Подготовка значений полей для элемента в соответсвии с указанными типами
		if($this->config_module['tab_card'])
		{
			foreach($this->variable_table as $title => $variable_table)
			{
				$this->prepare_new_values($variable_table);
			}
		}
		else
		{
			$this->prepare_new_values($this->variable_table);
		}

		// Сохраняет конфигурацию модуля
		if($this->diafan->config)
		{
			for($q = 0; $q < count($this->value); $q++)
			{
				$this->value[$q] = str_replace("\n", '', $this->value[$q]);
			}
			DB::query("DELETE FROM {config} WHERE module_name='%s' and cat_id='%d'", $this->diafan->rewrite, $this->diafan->site);
			DB::query("INSERT INTO {config} (module_name,config,cat_id) VALUES ('%s', '".str_replace("'", '', implode("\n", $this->query))."', '%d')",
			          array_merge(
			              array($this->diafan->rewrite),
			              $this->value,
			              array($this->diafan->site)
			          )
			         );
			$this->diafan->err = 1;

			// Удаляет кэш конфигурации модулей
			$this->diafan->_cache->delete("configmodules");
		}

		// Сохраняет элемент
		else
		{
			DB::query("UPDATE {".$this->table."} SET ".implode(', ',$this->query)." WHERE id = %d",
			          array_merge($this->value, array($this->diafan->save)));
		}

		// Удаляет кэш модуля
		$this->diafan->_cache->delete("", $this->diafan->module);
		$this->diafan->_cache->delete("", $this->diafan->rewrite);

		$this->redirect();
		return TRUE;
	}

	/**
	 * Подготавливает новые значения для сохранения
	 * 
	 * @return boolean true
	 */
	private function prepare_new_values($variable_table)
	{
		foreach ($variable_table as $key => $type_value)
		{
			$name = $key.(in_array($key, $this->language_names) ? _LANG : '');
			$func = 'save_'.preg_replace('/[^A-Za-z_]+/', '', $key).'_module'.($this->diafan->config ? '_config' : '');

			if(method_exists($this, $func))
			{
				$this->$func();
			}

			elseif ($type_value == 'password') 
			{
				if(! empty($_POST[$key]))
				{
					$this->query[] = $name."='%s'";
					$this->value[] = code_password($_POST[$key]);
				}
			}

			elseif ($type_value == 'date' || $type_value == 'datetime')
			{
				$this->query[] = $name."='%d'";
				$this->value[] = $this->diafan->unixdate($_POST[$key]);
			}

			elseif ($type_value == 'hr' || $type_value == 'disabled')
			{
				continue;
			}

			elseif ($type_value == 'floattext')
			{
				$this->value[] = str_replace(',', '.', ! empty($_POST[$key]) ? $_POST[$key] : 0);
				$this->query[] = $name."='%f'";
			}

			else
			{
				$this->value[] = ! empty($_POST[$key]) ? $_POST[$key] : '';

				if ($type_value == 'text' ||  $type_value == 'select')
				{
					$this->query[] = $name."='%h'";
				}

				elseif ($type_value == 'checkbox' || $type_value == 'numtext')
				{
					$this->query[] = $name."='%d'";
				}

				else //editor,textarea,none,function,...
				{
					$this->query[] = $name."='%s'";
				}
			}

			// Если сохраняет конфигурацию модуля и переменная - переводимая
			// необходимо сохранить значения переменной для других языковых версий
			if($this->diafan->config && in_array($key, $this->language_names))
			{
				foreach ($this->diafan->language_all as $lk => $lval)
				{
					if($lval[0] != _LANG && $this->diafan->configmodules($key.$lval[0], $this->diafan->rewrite, $lval[0]))
					{
						$this->query[] = $key.$lval[0]."='%s'";
						$this->value[] = $this->diafan->configmodules($key.$lval[0], $this->diafan->rewrite, $lval[0]);
					}
				}
			}
		}
		if($this->table == "param")
		{
			$this->query[] = "module_name='%s'";
			$this->value[] = str_replace('param', '', $this->file);
		}
		return TRUE;
	}

	/**
	 * Производит перенаправление на страницу редактирования, на список и пр.
	 * 
	 * @return boolean true
	 */
	protected function redirect()
	{
		// если сохраняли Ajaxом, отдаем положительный результат
		if(! empty($_SERVER["HTTP_X_REQUESTED_WITH"]) && $_SERVER["HTTP_X_REQUESTED_WITH"] == 'XMLHttpRequest')
		{
			echo '{success:true}';
			exit;
		}

		// Для модуля Языки сайта
		if($this->table == "language")
		{
			if(_LANG == $this->diafan->save)
			{
				$this->diafan->goto_diafan(BASE_PATH.'language/');
			}
			else
			{
				$this->diafan->goto_diafan(URL.($this->diafan->err ? 'error'.$this->diafan->err.'/' : ''));
			}
		}

		// "Применить" ($_POST["noOut"]) - редирект на edit_module 
		if($_POST["noOut"])
		{
			$this->diafan->goto_diafan(
				URL
				.($this->diafan->err ? 'error'.$this->diafan->err.'/' : '')
				.'edit'.$this->diafan->save.'/'
			);
		}

		// Если к странице прикреплен модуль, то редирект на модуль
		$addmodule = ! empty($_POST["addmodule"]) ? preg_replace('/[^A-Za-z-]+/', '', $_POST["addmodule"]) : '';
		if($this->diafan->rewrite == "site" && $addmodule && $addmodule != "reminding"
		   && file_exists(ABSOLUTE_PATH.'modules/'.$addmodule.'/'.$addmodule.'.admin.php'))
		{
			$this->diafan->goto_diafan(BASE_PATH_HREF.$addmodule.'/site'.$this->diafan->save.'/'
			.($this->diafan->err ? 'error'.$this->diafan->err.'/' : ''));
			return TRUE;
		}

		// "Сохранить и выйти" - редирект на show_module
		$this->diafan->goto_diafan(URL.($this->diafan->err ? 'error'.$this->diafan->err.'/' : ''));
		return TRUE;
	}


	//----------------------------------------------------------------------//
	// Функции сохранения полей

	/**
	 * Сохранение поля "Категория"
	 * 
	 * @return boolean
	 */
	protected function save_cat_id_module()
	{
		if(! $this->diafan->configmodules("cat", $this->diafan->module, '', $this->diafan->site))
		{
			return FALSE;
		}
		$this->query[] = "cat_id=%d";
		$this->value[] = $_POST["cat_id"];

		DB::query("DELETE FROM {category} WHERE element_id=%d AND module_name='%s'", $this->diafan->save, $this->diafan->module);
		DB::query("INSERT INTO {category} (element_id, cat_id, module_name) VALUES('%d', '%d', '%s')",
		          $this->diafan->save, $_POST["cat_id"], $this->diafan->module);

		if(!empty($_POST["cat_ids"]) && is_array($_POST["cat_ids"]))
		{
			foreach($_POST["cat_ids"] as $cat_id)
			{
				if($cat_id != $_POST["cat_id"])
				{
					DB::query("INSERT INTO {category} (element_id, cat_id, module_name) VALUES('%d', '%d', '%s')",
					          $this->diafan->save, $cat_id, $this->diafan->module);
				}
			}
		}
		return TRUE;
	}

	/**
	 * Сохранение поля "Страница"
	 * 
	 * @return boolean true
	 */
	private function save_site_id_module()
	{
		$this->query[] = "site_id='%d'";
		if(! $this->site_id)
		{
			$this->get_site_id();
		}
		$this->value[] = $this->site_id;
		return TRUE;
	}

	/**
	 * Сохранение поля "Страница"
	 * 
	 * @return boolean true
	 */
	private function get_site_id()
	{
		$this->site_id = ! empty($_POST["site_id"]) ? $_POST["site_id"] : 0;
		if($this->config_module["element"])
		{
			$this->site_id = DB::query_result("SELECT site_id FROM {".$this->table."_category} WHERE id=%d", $_POST["cat_id"]);
		}
		elseif($this->config_module["category"])
		{
			if($_POST["parent_id"])
			{
				$this->site_id = DB::query_result("SELECT site_id FROM {".$this->table."} WHERE id=%d", $_POST["parent_id"]);
			}
			else
			{
				$this->site_id = $_POST["site_id"];
			}
		}
		return TRUE;
	}

	/**
	 * Сохранение поля "Псевдоссылка"
	 * 
	 * @return boolean
	 */
	protected function save_rewrite_module()
	{
		if(! $this->site_id)
		{
			$this->get_site_id();
		}
		if(! $_POST["rewrite"])
		{
			
			DB::query("DELETE FROM {rewrite} WHERE module_name='%s' AND ".($this->config_module["category"] ? "cat" : "element")."_id=%d",
			           $this->diafan->module, $this->diafan->save);
			return TRUE;
		}
		if(! $this->diafan->savenew)
		{
			$row = DB::fetch_array(DB::query("SELECT rewrite, id FROM {rewrite} WHERE module_name='%s' AND "
			                                 .($this->config_module["category"] ? "cat" : "element")."_id=%d LIMIT 1",
			                                 $this->diafan->module, $this->diafan->save));
		}
		$rewrite = ! empty($row["rewrite"]) ? $row["rewrite"] : '';
		if(! empty($row["id"]))
		{
			DB::query("DELETE FROM {rewrite} WHERE module_name='%s' AND ".($this->config_module["category"] ? "cat" : "element")
			          ."_id=%d AND id<>%d",
			          $this->diafan->module, $this->diafan->save, $row["id"]);
		}
		if($_POST["rewrite"] == $rewrite)
		{
			if($this->site_id != $this->oldrow["site_id"])
			{
				DB::query("UPDATE {rewrite} SET site_id=%d WHERE module_name='%s' AND ".($this->config_module["category"] ? "cat" : "element")."_id=%d",
				          $this->site_id, $this->diafan->module, $this->diafan->save);
				if($this->config_module["category"])
				{
					$child = $this->diafan->get_children($this->diafan->save, $this->table);
					DB::query("UPDATE {rewrite} SET site_id=%d WHERE module_name='%s' AND cat_id IN (%s)",
					          $this->site_id, $this->diafan->module, implode(",", $child));
					DB::query("UPDATE {".str_replace("_category", "", $this->table)."} SET site_id=%d WHERE cat_id IN (%s)",
					          $this->site_id, implode(",", $child));
					DB::query("UPDATE {rewrite} SET site_id=%d WHERE module_name='%s' AND element_id IN"
					          ." (SELECT id FROM {".str_replace("_category", "", $this->table)."} WHERE cat_id IN (%s))",
					          $this->site_id, $this->diafan->module, $this->diafan->module, implode(",", $child));
				}
				$this->diafan->_cache->delete("", "rewrite");
			}
			return FALSE;
		}
		$rewrite_site = '';
		if(! $rewrite)
		{
			if($this->config_module["element"])
			{
				if(! $rewrite_site = DB::query_result("SELECT rewrite FROM {rewrite} WHERE module_name='%s' AND cat_id=%d",
				                                      $this->diafan->module, $_POST["cat_id"]))
				{
					$rewrite_site = DB::query_result("SELECT rewrite FROM {rewrite} WHERE module_name='site' AND site_id=%d", $this->site_id);
				}
			}
			elseif($this->config_module["category"])
			{
				if(! $_POST["parent_id"]
				   || ! $rewrite_site = DB::query_result("SELECT rewrite FROM {rewrite} WHERE module_name='%s' AND cat_id=%d",
				                                         $this->diafan->module, $_POST["parent_id"]))
				{
					$rewrite_site = DB::query_result("SELECT rewrite FROM {rewrite} WHERE module_name='site' AND site_id=%d", $_POST["site_id"]);
				}
			}
			else
			{
				$rewrite_site = DB::query_result("SELECT rewrite FROM {rewrite} WHERE module_name='site' AND site_id=%d", $this->site_id);
			}
		}
		//$new_rewrite = ($rewrite_site ? $rewrite_site.'/' : '').$this->diafan->translit($_POST["rewrite"]);
		$new_rewrite = ($rewrite_site ? $rewrite_site.'/' : '').$_POST["rewrite"];
		if($rew_id = DB::query_result("SELECT id FROM {rewrite} WHERE rewrite='%h'", $new_rewrite))
		{
			$new_rewrite .= $rew_id;
		}
		if($rewrite)
		{
			DB::query("UPDATE {rewrite} SET rewrite='%h', site_id=%d WHERE module_name='%s' AND "
			          .($this->config_module["category"] ? "cat" : "element")."_id=%d",
			          $new_rewrite, $this->site_id, $this->diafan->module, $this->diafan->save);
		}
		else
		{
			DB::query("INSERT INTO {rewrite} (rewrite, module_name, ".($this->config_module["category"] ? "cat" : "element")
			          ."_id, site_id) VALUES ('%h', '%s', %d, %d)",
			          $new_rewrite, $this->diafan->module, $this->diafan->save, $this->site_id);
		}
		$this->diafan->_cache->delete("", "rewrite");
		return TRUE;
	}

	/**
	 * Сохранение поля "Изображения"
	 * 
	 * @return boolean true
	 */
	private function save_photo_module()
	{
		return TRUE;
	}

	/**
	 * Сохранение поля "Комментарии"
	 * 
	 * @return boolean true
	 */
	private function save_comments_module()
	{
		return TRUE;
	}

	/**
	 * Сохранение поля "Тэги"
	 * 
	 * @return boolean true
	 */
	private function save_tags_module()
	{
		return TRUE;
	}

	/**
	 * Сохранение поля "Рейтинг"
	 * 
	 * @return boolean true
	 */
	private function save_rate_module()
	{
		return TRUE;
	}

	/**
	 * Сохранение поля "Сортировка"
	 * 
	 * @return boolean
	 */
	private function save_sort_module()
	{
		if($this->diafan->savenew)
		{
			$this->query[] = "sort='%d'";
			$this->value[] = $this->diafan->save;
			return TRUE;
		}
		//не сортируются неактивные на сайте и в меню элементы
		if(! empty($this->variable_table["act"]) && ! DB::query_result("SELECT act FROM {".$this->table."} WHERE id=%d", $this->diafan->save)
		   || $this->config_module['menu']  && ! DB::query_result("SELECT actm FROM {".$this->table."} WHERE id=%d", $this->diafan->save))
			return FALSE;

		//переменная $_POST["sort"] - id элемента, перед которым должен выводится редактируемый элемент
		if(! empty($_POST["sort"]) && $_POST["sort"] == $this->diafan->save)
			return FALSE;

		if(empty($this->oldrow["sort"]))
		{
			$this->query[] = "sort='%d'";
			$this->value[] = $this->diafan->save;
			return TRUE;
		}

		$sort_new = '';
		//"down" - установить ниже всех
		if(! empty($_POST["sort"]) && $_POST["sort"] == "down")
		{
			$sort_new = DB::where($this->table,
			                      "1=1"
			                      .($this->config_module["parent"]
			                        ? " AND parent_id='".$_POST["parent_id"]."'"
			                        : ($this->config_module["element"] ? ' AND cat_id="'.$_POST["cat_id"].'"' : '')
			                       )
			                      .($this->config_module["trash"] ? " AND trash='0'" : '')
			                      .(! empty($this->variable_table["act"]) ? " AND act='1'" : '')
			                      ." ORDER BY sort DESC",
			                      "sort") + 1;
		}
		elseif(! empty($_POST["sort"]))
		{
			$sort_new = DB::title($this->table, $_POST["sort"], "sort");
		}

		if($this->oldrow["sort"] > $sort_new)
		{
			DB::query("UPDATE {".$this->table."} SET sort=sort+1 WHERE sort>='%d'"
			          .($this->config_module["parent"]
			            ? " AND parent_id='".$_POST["parent_id"]."'"
			            : ($this->config_module["element"] ? ' AND cat_id="'.$_POST["cat_id"].'"' : '')
			           )
			          .($this->config_module["trash"] ? " AND trash='0'" : '')
			          .(! empty($this->variable_table["act"]) ? " AND act='1'" : ''),
			          $sort_new);
		}
		else
		{
			$sort_new--;
			DB::query("UPDATE {".$this->table."} SET sort=sort-1 WHERE sort>'%d' AND sort<='%d'"
			          .($this->config_module["parent"]
			            ? " AND parent_id='".$_POST["parent_id"]."'"
			            : ($this->config_module["element"] ? ' AND cat_id="'.$_POST["cat_id"].'"' : '')
			           )
			          .($this->config_module["trash"] ? " AND trash='0'" : '')
			          .(! empty($this->variable_table["act"]) ? " AND act='1'" : ''),
			          $this->oldrow["sort"], $sort_new);
		}
		$this->query[] = "sort='%d'";
		$this->value[] = $sort_new;
		return TRUE;
	}

	/**
	 * Сохранение поля "Время редактирования"
	 * 
	 * @return boolean true
	 */
	private function save_timeedit_module()
	{
		$this->query[] = "timeedit='%s'";
		$this->value[] = time();
		return TRUE;
	}

	/**
	 * Сохранение поля "Номер страницы"
	 * @return boolean true
	 */
	protected function save_number_module()
	{
		return TRUE;
	}
}