<?php
/**
 * Diafan.CMS
 *
 * @author     diafan.ru
 * @version    4.3
 * @license http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2011 OOO "Диафан". (http://diafan.ru)
 */

if(! defined('DIAFAN'))
{
	include(dirname(dirname(dirname(__FILE__))).'/includes/404.php');
}

/**
 * User_admin
 * 
 * Работа с пользователями в административной части
 */
class User_admin extends Core
{
	/**
	 * @var integer идентификатор пользователя
	 */
	public $id = 0;

	/**
	 * @var string логин пользователя
	 */
	public $name;

	/**
	 * @var string ФИО пользователя
	 */
	public $fio;

	/**
	 * @var string e-mail пользователя
	 */
	public $mail;

	/**
	 * @var integer дата регистрации текущего пользователя
	 */
	public $created;

	/**
	 * @var array права доступа текущего пользователя
	 */
	public $roles = array();

	/**
	 * @var string данные, хранящиеся в сессии
	 */
	public $session;

	/**
	 * @var string ошибка авторизации
	 */
	public $errauth;

	/**
	 * @var array характеристики пользователя
	 */
	private $fields = array('id', 'name', 'fio', 'mail', 'created', 'roles');

	/**
	 * Конструктор класса
	 * 
	 * @return void
	 */
	public function __construct()
	{
	}

	/**
	 * Определяет текущего пользователя
	 * 
	 * @param object $user данные о текущем пользователе
	 * @return boolean true
	 */
	public function set($user)
	{
		foreach($this->fields as $field)
		{
			if(! empty($user->$field))
			{
				$this->$field = $user->$field;
			}
		}
		if(! empty($user->session))
		{
			$this->session = $user->session;
		}
		return TRUE;
	}

	/**
	 * Определяет, что текущий пользователь не авторизован
	 * 
	 * @param string $session данные, записанные в сессии
	 * @return boolean true
	 */
	public function anonymous($session = '')
	{
		$this->session = $session;
		return TRUE;
	}

	/**
	 * Отчищает информацию о текущем пользователе
	 * 
	 * @return boolean true
	 */
	public function logout()
	{
		$this->anonymous();
		session_destroy();
		$this->goto_diafan(BASE_PATH_HREF);
		return TRUE;
	}

	/**
	 * Проверяет авторизован ли пользователь
	 * 
	 * @return boolean
	 */
	public function auth($form_values)
	{
		if (! $form_values['name'] || ! $form_values['pass'])
		{
			$this->errauth = 'wrong_login_or_pass';
			return FALSE;
		}

		if(DB::query_result("SELECT name FROM {users} WHERE (trash='1' OR act='0') AND LOWER(name)=LOWER('%s')", trim($form_values['name'])))
		{
			$this->errauth = 'blocked';
			return FALSE;
		}

		if($this->_log())
		{
			$this->errauth = 'blocked_30_min';
			return FALSE;
		}

		$result = DB::query("SELECT * FROM {users} u WHERE trash='0' AND act='1' AND LOWER(name)=LOWER('%s') AND password='%s'",
				    trim($form_values['name']), code_password(trim($form_values['pass'])));

		if (DB::num_rows($result))
		{
			$user = DB::fetch_object($result);
			$this->set($user);

			$this->goto_diafan(BASE_PATH_HREF.($_GET["rewrite"] ? $_GET["rewrite"]."/" : ''));
			return TRUE;
		}
		else
		{
			$this->update_log();
			$this->errauth = 'wrong_login_or_pass';
			return FALSE;
		}
	}

	/**
	 * Проверяет лог авторизации для блокировки попытки подбора паролей
	 * 
	 * @return boolean
	 */
	private function _log()
	{
		DB::query("DELETE FROM {log} WHERE created<'%d'", time());
		$ip = getenv('REMOTE_ADDR');
		$date = time() + 1800;
		$result = DB::query("SELECT `count` FROM {log} WHERE ip = '%h'", $ip); 
		if(DB::num_rows($result) > 0)
		{
			$row = DB::fetch_array($result);
			if($row['count'] > 4)
			{
				return TRUE;
			}
		}
		return FALSE;
	}

	/**
	 * Обновляет лог авторизации для блокировки попытки подбора паролей
	 * 
	 * @return boolean
	 */
	private function update_log()
	{
		if(getenv('HTTP_X_FORWARDED_FOR'))
		{
			$ip = getenv('HTTP_X_FORWARDED_FOR');
		}
		else
		{
			$ip = getenv('REMOTE_ADDR');
		}
		$date = time() + 1800;
		$result = DB::query("SELECT `count` FROM {log} WHERE ip = '%h'", $ip); 
		if(DB::num_rows($result) > 0)
		{
			DB::query("UPDATE {log} SET `count` = `count` + 1, created = '%d' WHERE ip = '%h'", $date, $ip);
		}
		else
		{
			$info = getenv('HTTP_USER_AGENT');
			DB::query("INSERT INTO {log} (ip, created, info) VALUES ('%s', '%d', '%s')", $ip, $date, $info);
		}
		return FALSE;
	}
}