<?php
/**
 * Diafan.CMS
 *
 * @author     diafan.ru
 * @version    4.3
 * @license http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2011 OOO "Диафан". (http://diafan.ru)
 */

if(! defined('DIAFAN'))
{
	include(dirname(dirname(__FILE__)).'/includes/404.php');
}

/**
 * Cache
 * 
 * Кэширование
 */
class Cache
{
	/**
	 * @var name метка кэша
	 */
	private $name;

	/**
	 * @var string название модуля
	 */
	private $module;

	/**
	 * @var array текущий кэш
	 */
	private $cache;

	/**
	 * Читает кэш модуля $module с меткой $name.
	 *
	 * @param string|array $name метка кэша
	 * @param string $module название модуля
	 * @param boolean $ignore_mod_dev_cache игнорировать отключение кэша в параметрах сайта
	 * @return mixed
	 */
	public function get($name, $module = 'site', $ignore_mod_dev_cache = FALSE)
	{
		if(MOD_DEVELOPER_CACHE && ! $ignore_mod_dev_cache)
			return FALSE;

		$this->transform_param($name, $module);
		if(empty($this->cache[$this->module][$this->name]))
		{
			$this->inc_cache_file();
		}

		if(! isset($this->cache[$this->module][$this->name]))
		{
			return FALSE;
		}
		return unserialize($this->cache[$this->module][$this->name]);
	}

	/**
	 * Сохраняет данные $data для модуля $module с меткой $name
	 *
	 * @param mixed $data данные, сохраняемые в кэше
	 * @param string|array $name метка кэша
	 * @param string $module название модуля
	 * @param boolean $ignore_mod_dev_cache игнорировать отключение кэша в параметрах сайта
	 * @return boolean true
	 */
	public function save($data, $name, $module = 'site', $ignore_mod_dev_cache = FALSE)
	{
		if(MOD_DEVELOPER_CACHE && ! $ignore_mod_dev_cache)
			return FALSE;

		$this->transform_param($name, $module);
		$this->inc_cache_file();

		$this->cache[$this->module][$this->name] = serialize($data);
		$this->write_cache();
		return TRUE;
	}

	/**
	 * Удаляет кэш для модуля $module с меткой $name. Если функция вызвана с пустой меткой, то удаляется весь кэш для модуля $module
	 *
	 * @param string|array $name метка кэша
	 * @param string|array $module название модуля
	 * @return boolean true
	 */
	public function delete($name, $module = 'site')
	{
		if(is_array($module))
		{
			if($module)
			{
				foreach($module as $m)
				{
					$this->delete($name, $m);
				}
				return TRUE;
			}
			$d = dir(ABSOLUTE_PATH.'cache');
			while($entry = $d->read())
			{
				if ($entry != "." and $entry != ".." and $entry != ".htaccess")
				{
					if(is_dir(ABSOLUTE_PATH.'cache/'.$entry))
					{
						$this->module = $entry;
						$dm = dir(ABSOLUTE_PATH.'cache/'.$this->module);
						while($entrym = $dm->read())
						{
							if($entrym != "." and $entrym != "..")
							{
								if(! is_dir(ABSOLUTE_PATH.'cache/'.$this->module.'/'.$entrym))
								{
									$this->name = $entrym;
									$this->cache[$this->module][$this->name] = '';
									$this->write_cache();
								}
							}
						}
						$dm->close();
					}
					else
					{
						unlink(ABSOLUTE_PATH.'cache/'.$entry);
					}
				}
			}
			$d->close();
			return TRUE;
		}

		$this->transform_param($name, $module);
		if(! $name)
		{
			if(is_dir(ABSOLUTE_PATH.'cache/'.$this->module))
			{
				$d = dir(ABSOLUTE_PATH.'cache/'.$this->module);
				while($entry = $d->read())
				{
					if ($entry != "." and $entry != "..")
					{
						if(! is_dir(ABSOLUTE_PATH.'cache/'.$this->module.'/'.$entry))
						{
							$this->name = $entry;
							$this->cache[$this->module][$this->name] = '';
							$this->write_cache();
						}
					}
				}
				$d->close();
			}
		}
		else
		{
			$this->inc_cache_file();
			$this->cache[$this->module][$this->name] = '';
			$this->write_cache();
		}
		return TRUE;
	}

	/**
	 * Преобразует метку и название модуля для работы с кэшем
	 *
	 * @param string|array $name метка кэша
	 * @param string $module название модуля
	 * @return boolean true
	 */
	private function transform_param($name, $module)
	{
		if(! is_array($name))
		{
			$this->name = md5($name);
		}
		else
		{
			$name_string = '';
			foreach($name as $key => $value)
			{
				$name_string .= '_'.$key.'='.$value;
			}
			$this->name = md5($name_string);
		}
		$this->module = md5($module);
		return TRUE;
	}

	/**
	 * Подключает файл с кэшем модуля
	 *
	 * @return boolean true
	 */
	private function inc_cache_file()
	{
		if(empty($this->cache[$this->module][$this->name]) && file_exists(ABSOLUTE_PATH.'cache/'.$this->module.'/'.$this->name))
		{
			$this->cache[$this->module][$this->name] = file_get_contents(ABSOLUTE_PATH.'cache/'.$this->module.'/'.$this->name);
		}
		return TRUE;
	}

	/**
	 * Записывает кэш в файл
	 *
	 * @return boolean true
	 */
	private function write_cache()
	{
		if(empty($this->cache[$this->module][$this->name]))
		{
			$this->cache[$this->module][$this->name] = '';
		}

		if(! is_dir(ABSOLUTE_PATH."cache/".$this->module))
		{
			mkdir(ABSOLUTE_PATH."cache/".$this->module, 0777);
		}

		$fp = fopen(ABSOLUTE_PATH."cache/".$this->module.'/'.$this->name, "w");
		fwrite($fp, $this->cache[$this->module][$this->name]);
		fclose($fp);

		if(empty($this->cache[$this->module][$this->name]))
		{
			unlink(ABSOLUTE_PATH.'cache/'.$this->module.'/'.$this->name);
		}

		return TRUE;
	}
}