<?php
/**
 * Diafan.CMS
 *
 * @author     diafan.ru
 * @version    4.3
 * @license http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2011 OOO "Диафан". (http://diafan.ru)
 */

if(! defined('DIAFAN'))
{
	include(dirname(dirname(__FILE__)).'/includes/404.php');
}

/**
 * Files
 * 
 * Работа с файлами
 */
class Files
{
	/**
	 * @var object основной объект системы
	 */
	private $diafan;

	/**
	 * Конструктор класса. Определяет свойства класса
	 * 
	 * @return void
	 */
	public function __construct($diafan)
	{
		$this->diafan = $diafan;
	}

	/**
	 * Загружает файлы
	 *
	 * @param array $file загружаемый файл/файлы
	 * @param string $module_name название модуля
	 * @param integer $element_id номер элемента, к которому прикрепляется файл
	 * @param integer|boolean false $n номер файла в массиве файлов, если предан массив
	 * @return boolean|string
	 */
	public function file_upload($file, $module_name, $element_id, $n = FALSE)
	{
		if($n !== FALSE)
		{
			$name     = $file['name'][$n];
			$tmp_name = $file['tmp_name'][$n];
			$type     = $file['type'][$n];
		}
		else
		{
			$name     = $file['name'];
			$tmp_name = $file['tmp_name'];
			$type     = $file['type'];
		}
		$size = filesize($tmp_name);
		if ($name == '')
			return FALSE;

		$extension = strtolower(substr(strrchr($name, '.'), 1));
		$name = $this->diafan->translit($name);
		if(strlen($name) > 50)
		{
			$name = substr($name, -50);
		}
		$attachment_extensions = explode(',', str_replace(' ', '', strtolower($this->diafan->configmodules('attachment-extensions'))));
		$mimes = array(
			'ez' => 'application/andrew-inset',
			'atom' => 'application/atom+xml',
			'cgi' => 'application/cgi',
			'hqx' => 'application/mac-binhex40',
			'cpt' => 'application/mac-compactpro',
			'mathml' => 'application/mathml+xml',
			'doc' => 'application/msword',
			'bin' => 'application/octet-stream',
			'dms' => 'application/octet-stream',
			'lha' => 'application/octet-stream',
			'lzh' => 'application/octet-stream',
			'exe' => 'application/octet-stream',
			'class' => 'application/octet-stream',
			'so' => 'application/octet-stream',
			'dll' => 'application/octet-stream',
			'dmg' => 'application/octet-stream',
			'iso' => 'application/octet-stream',
			'oda' => 'application/oda',
			'ogg' => 'application/ogg',
			'pdf' => 'application/pdf',
			'pl' => 'application/perl',
			'plx' => 'application/perl',
			'ppl' => 'application/perl',
			'perl' => 'application/perl',
			'pm' => 'application/perl',
			'ai' => 'application/postscript',
			'eps' => 'application/postscript',
			'ps' => 'application/postscript',
			'rdf' => 'application/rdf+xml',
			'rb' => 'application/ruby',
			'smi' => 'application/smil',
			'smil' => 'application/smil',
			'gram' => 'application/srgs',
			'grxml' => 'application/srgs+xml',
			'mif' => 'application/vnd.mif',
			'xul' => 'application/vnd.mozilla.xul+xml',
			'xls' => 'application/vnd.ms-excel',
			'ppt' => 'application/vnd.ms-powerpoint',
			'rm' => 'application/vnd.rn-realmedia',
			'wbxml' => 'application/vnd.wap.wbxml',
			'wmlc' => 'application/vnd.wap.wmlc',
			'wmlsc' => 'application/vnd.wap.wmlscriptc',
			'vxml' => 'application/voicexml+xml',
			'bcpio' => 'application/x-bcpio',
			'vcd' => 'application/x-cdlink',
			'pgn' => 'application/x-chess-pgn',
			'Z' => 'application/x-compress',
			'cpio' => 'application/x-cpio',
			'csh' => 'application/x-csh',
			'dcr' => 'application/x-director',
			'dir' => 'application/x-director',
			'dxr' => 'application/x-director',
			'dvi' => 'application/x-dvi',
			'spl' => 'application/x-futuresplash',
			'gtar' => 'application/x-gtar',
			'gz' => 'application/x-gzip',
			'tgz' => 'application/x-gzip',
			'hdf' => 'application/x-hdf',
			'php' => 'application/x-httpd-php',
			'php3' => 'application/x-httpd-php',
			'php4' => 'application/x-httpd-php',
			'php5' => 'application/x-httpd-php',
			'php6' => 'application/x-httpd-php',
			'phps' => 'application/x-httpd-php-source',
			'img' => 'application/x-img',
			'js' => 'application/x-javascript',
			'skp' => 'application/x-koan',
			'skd' => 'application/x-koan',
			'skt' => 'application/x-koan',
			'skm' => 'application/x-koan',
			'latex' => 'application/x-latex',
			'nc' => 'application/x-netcdf',
			'cdf' => 'application/x-netcdf',
			'crl' => 'application/x-pkcs7-crl',
			'sh' => 'application/x-sh',
			'shar' => 'application/x-shar',
			'swf' => 'application/x-shockwave-flash',
			'sit' => 'application/x-stuffit',
			'sv4cpio' => 'application/x-sv4cpio',
			'sv4crc' => 'application/x-sv4crc',
			'tgz' => 'application/x-tar',
			'tar' => 'application/x-tar',
			'tcl' => 'application/x-tcl',
			'tex' => 'application/x-tex',
			'texinfo' => 'application/x-texinfo',
			'texi' => 'application/x-texinfo',
			't' => 'application/x-troff',
			'tr' => 'application/x-troff',
			'roff' => 'application/x-troff',
			'man' => 'application/x-troff-man',
			'me' => 'application/x-troff-me',
			'ms' => 'application/x-troff-ms',
			'ustar' => 'application/x-ustar',
			'src' => 'application/x-wais-source',
			'crt' => 'application/x-x509-ca-cert',
			'xhtml' => 'application/xhtml+xml',
			'xht' => 'application/xhtml+xml',
			'xml' => 'application/xml',
			'xsl' => 'application/xml',
			'dtd' => 'application/xml-dtd',
			'xslt' => 'application/xslt+xml',
			'zip' => 'application/zip',
			'au' => 'audio/basic',
			'snd' => 'audio/basic',
			'mid' => 'audio/midi',
			'midi' => 'audio/midi',
			'kar' => 'audio/midi',
			'a-latm' => 'audio/mp4',
			'm4p' => 'audio/mp4',
			'm4a' => 'audio/mp4',
			'mp4' => 'audio/mp4',
			'mpga' => 'audio/mpeg',
			'mp2' => 'audio/mpeg',
			'mp3' => 'audio/mpeg',
			'aif' => 'audio/x-aiff',
			'aiff' => 'audio/x-aiff',
			'aifc' => 'audio/x-aiff',
			'm3u' => 'audio/x-mpegurl',
			'wax' => 'audio/x-ms-wax',
			'wma' => 'audio/x-ms-wma',
			'ram' => 'audio/x-pn-realaudio',
			'ra' => 'audio/x-pn-realaudio',
			'wav' => 'audio/x-wav',
			'pdb' => 'chemical/x-pdb',
			'xyz' => 'chemical/x-xyz',
			'bmp' => 'image/bmp',
			'cgm' => 'image/cgm',
			'gif' => 'image/gif',
			'ief' => 'image/ief',
			'jpeg' => 'image/jpeg',
			'jpg' => 'image/jpeg',
			'jpe' => 'image/jpeg',
			'png' => 'image/png',
			'svg' => 'image/svg+xml',
			'tiff' => 'image/tiff',
			'tif' => 'image/tiff',
			'djvu' => 'image/vnd.djvu',
			'djv' => 'image/vnd.djvu',
			'wbmp' => 'image/vnd.wap.wbmp',
			'ras' => 'image/x-cmu-raster',
			'ico' => 'image/x-icon',
			'pnm' => 'image/x-portable-anymap',
			'pbm' => 'image/x-portable-bitmap',
			'pgm' => 'image/x-portable-graymap',
			'ppm' => 'image/x-portable-pixmap',
			'rgb' => 'image/x-rgb',
			'xbm' => 'image/x-xbitmap',
			'xpm' => 'image/x-xpixmap',
			'xwd' => 'image/x-xwindowdump',
			'igs' => 'model/iges',
			'iges' => 'model/iges',
			'msh' => 'model/mesh',
			'mesh' => 'model/mesh',
			'silo' => 'model/mesh',
			'wrl' => 'model/vrml',
			'vrml' => 'model/vrml',
			'ics' => 'text/calendar',
			'ifb' => 'text/calendar',
			'css' => 'text/css',
			'shtml' => 'text/html',
			'html' => 'text/html',
			'htm' => 'text/html',
			'asc' => 'text/plain',
			'txt' => 'text/plain',
			'rtx' => 'text/richtext',
			'rtf' => 'text/rtf',
			'sgml' => 'text/sgml',
			'sgm' => 'text/sgml',
			'tsv' => 'text/tab-separated-values',
			'vbs' => 'text/vbscript',
			'wml' => 'text/vnd.wap.wml',
			'wmls' => 'text/vnd.wap.wmlscript',
			'cnf' => 'text/x-config',
			'conf' => 'text/x-config',
			'log' => 'text/x-log',
			'reg' => 'text/x-registry',
			'etx' => 'text/x-setext',
			'sql' => 'text/x-sql',
			'mpeg' => 'video/mpeg',
			'mpg' => 'video/mpeg',
			'mpe' => 'video/mpeg',
			'qt' => 'video/quicktime',
			'mov' => 'video/quicktime',
			'mxu' => 'video/vnd.mpegurl',
			'm4u' => 'video/vnd.mpegurl',
			'avi' => 'video/x-msvideo',
			'movie' => 'video/x-sgi-movie',
			'ice' => 'x-conference/x-cooltalk'
		);

		if ($this->diafan->configmodules('attachment-extensions') 
		&& ! in_array($extension, $attachment_extensions)
		|| empty($mimes[$extension]) || $mimes[$extension] != strtolower($type))
		{
			return '<br>'.sprintf(_LANG_ATTACHMENT_EXTENSIONS, $name).$this->diafan->configmodules('attachment-extensions')
			.(IS_ADMIN ? '. '._LANG_ATTACHMENT_NEW_EXTENSIONS : '');
		}

		if (! is_uploaded_file($tmp_name) || ! file_exists($tmp_name))
		{
			return '<br>'.sprintf(_LANG_ATTACHMENT_NO_SUCCESS, $name);
		}

		DB::query("INSERT INTO {attachment} (name, module_name, element_id, extension, size) VALUES ('%s', '%s', '%d', '%s', '%d')",
		          $name, $module_name, $element_id, $type, $size);
		$id = DB::next_id("attachment");

		if(! move_uploaded_file($tmp_name, ABSOLUTE_PATH.USERFILES.'/'.$module_name.'/files/'.$id))
		{
			return '<br>'.sprintf(_LANG_ATTACHMENT_NO_SUCCESS, $name);
		}
		return FALSE;
	}
}