<?php
/**
 * Diafan.CMS
 *
 * @author     diafan.ru
 * @version    4.3
 * @license http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2011 OOO "Диафан". (http://diafan.ru)
 */

if(! defined('DIAFAN'))
{
	include(dirname(dirname(__FILE__)).'/includes/404.php');
}

/**
 * Theme_functions
 * 
 * Общие шаблонные функции
 */
class Theme_functions
{
	/**
	 * @var object основной объект системы
	 */
	private $diafan;

	/**
	 * @var object представление в пользовательской части
	 */
	private $view;

	/**
	 * Конструктор класса. Определяет свойства класса
	 * 
	 * @return void
	 */
	public function __construct($view)
	{
		$this->diafan = $view->diafan;
		$this->view   = $view;
	}

	/**
	 * Выводит основной контент страницы
	 * 
	 * @return boolean true
	 */
	public function show_body()
	{		
		if(! $this->diafan->title_no_show)
		{
			echo '<h1>'.($this->diafan->titlemodule ? $this->diafan->titlemodule : $this->diafan->name).'</h1>';
		}
		
		if(! $this->diafan->show && ! $this->diafan->cat && ! $this->diafan->step && (! isset($_GET["catid"]) || $this->diafan->module != "shop"))
		{
			$this->view->get_function_in_theme($this->diafan->text);
		}

		$this->view->module->show_module();
		return TRUE;
	}

	/**
	 * Выводит ссылки меню на страницы нижнего уровня, принадлежащие текущей странице
	 * 
	 * @return boolean
	 */
	public function show_links()
	{
		//кеш
		$site_pages = $this->diafan->get_site_pages();

		$links_page = array();

		if($site_pages)
		{
			foreach($site_pages as $page)
			{
				if($page["parent_id"] == $this->diafan->cid && $page["actm"])
				{
					$links_page[$page["sort"]][$page["id"]] = '<li><a href="'
					.(! empty($page['othurl'])
					 ? $page["othurl"].'" target="_blank'
					 : BASE_PATH_HREF.(empty($this->diafan->rewrites[$page["id"]]["rewrite"])
							  ? ''
							  : $this->diafan->rewrites[$page["id"]]["rewrite"].'/'
							  )
					 )
					.'">'.$page["name"].'</a></li>';
				}
			}
		}
		if(! $links_page)
		{
			return FALSE;
		}

		ksort($links_page);
		foreach($links_page as $l)
		{
			ksort($l);
			foreach($l as $l1)
			{
				$links[] = $l1;
			}
		}
		echo '<ul>'.implode("\n", $links).'</ul>';
		return TRUE;
	}

	/**
	 * Выводит ссылки на предыдующую и следующую страницы
	 * 
	 * @return boolean
	 */
	public function show_previous_next()
	{
		if($this->diafan->module && ($this->diafan->cat || $this->diafan->show) || ! $this->diafan->actm)
		{
			return TRUE;
		}
		//кеш
		$site_pages = $this->diafan->get_site_pages();

		$links_page = array();
		$previous   = 0;
		$next       = 0;

		if($site_pages)
		{
			foreach($site_pages as $page)
			{
				if($page["parent_id"] == $this->diafan->parent_id && $page["actm"])
				{
					if($this->diafan->cid == $page["id"])
					{
						$next = 99999;
					}
					elseif(! $next)
					{
						$previous = $page["id"];
					}
					else
					{
						$next = $page["id"];
						break;
					}
					
					$links_page[$page["sort"]][$page["id"]] = '<li><a href="'
					.(! empty($page['othurl'])
					 ? $page["othurl"].'" target="_blank'
					 : BASE_PATH_HREF.(empty($this->diafan->rewrites[$page["id"]]["rewrite"])
							  ? ''
							  : $this->diafan->rewrites[$page["id"]]["rewrite"].'/'
							  )
					 )
					.'">'.$page["name"].'</a></li>';
				}
			}
		}
		if($previous)
		{
			echo '<div class="previous-link"><a href="'
				.(! empty($site_pages[$previous." "]['othurl'])
				 ? $site_pages[$previous." "]["othurl"].'" target="_blank'
				 : BASE_PATH_HREF.(empty($this->diafan->rewrites[$previous]["rewrite"])
						  ? ''
						  : $this->diafan->rewrites[$previous]["rewrite"].'/'
						  )
				 )
				.'">&larr; '.$site_pages[$previous." "]["name"].'</a></div>';
		}
		if($next && $next != 99999)
		{
			echo '<div class="next-link"><a href="'
				.(! empty($site_pages[$next." "]['othurl'])
				 ? $site_pages[$next." "]["othurl"].'" target="_blank'
				 : BASE_PATH_HREF.(empty($this->diafan->rewrites[$next]["rewrite"])
						  ? ''
						  : $this->diafan->rewrites[$next]["rewrite"].'/'
						  )
				 )
				.'">'.$site_pages[$next." "]["name"].' &rarr;</a></div>';
		}
		return TRUE;
	}

	/**
	 * Выводит меню
	 * 
	 * @return boolean true
	 */
	public function show_menu($a)
	{
		$a = $this->view->get_attributes($a, 'tag_start_1', 'tag_end_1', 'tag_active_start_1', 'tag_active_end_1', 'tag_level_start_1',
						 'tag_level_end_1', 'tag_active_child_start_1', 'tag_active_child_end_1', 'hide_parent_link',
						 'separator_1', array('koren', 0), 'count_level', 'show_all_level', 'level');

		$a["show_all_level"] = $a["show_all_level"] ? TRUE : FALSE;

		include_once(ABSOLUTE_PATH.'includes/menu.php');
		$menu = new Menu($this->diafan, $a);
		$menu->table = "site";
		$menu->cid = $this->diafan->cid;
		$menu_data = $menu->get_menu();
		$menu->draw_menu($menu_data);
		return TRUE;
	}

	/**
	 * Выводит навигации по сайту «Хлебные крошки»
	 * 
	 * @return boolean
	 */
	public function show_path($attributes)
	{
		$attributes = $this->view->get_attributes($attributes, 'separator');

		if($this->diafan->cid == 1 && ! $this->diafan->cat && ! $this->diafan->show)
			return FALSE;

		$separator = $attributes["separator"] ? str_replace('src="/', 'src="'.BASE_PATH, $attributes["separator"]) : '/';

		if($this->diafan->parent_id)
		{
			$site_pages = $this->diafan->get_site_pages();
			$id = $this->diafan->parent_id;
			while ($id <> 0)
			{
				if($site_pages[$id." "])
				{
					$path_array[] = '<a href="'.BASE_PATH_HREF
					.(! empty($this->diafan->rewrites[$id]["rewrite"]) ? $this->diafan->rewrites[$id]["rewrite"].'/' : '')
					.'">'.$site_pages[$id.' ']["name"].'</a>';
					$id = $site_pages[$id.' ']["parent_id"];
				}
				else
				{
					$id = 0;
				}
			}

			$path_array = array_reverse($path_array);
		}

		if($this->diafan->cid != 1)
		{
			echo  '<div class="path"><a href="'.BASE_PATH_HREF.'">'._LANG_HOME.'</a> '.$separator.' ';
		}
		if($this->diafan->parent_id)
		{
			foreach($path_array as $link)
			{
				echo $link.' '.$separator.' ';
			}
		}
		if($this->diafan->path)
		{
			foreach($this->diafan->path as $path)
			{
				echo '<a href="'.BASE_PATH_HREF.$path["link"].'">'.$path["name"].'</a> '.$separator.' ';
			}
		}

		if($this->diafan->cid != 1)
		{
			echo '</div>';
		}
		return TRUE;
	}

	/**
	 * Выводит ссылки на альтернативные языковые версии сайта
	 * 
	 * @return boolean
	 */
	public function show_language()
	{
		if(count($this->diafan->language_all) < 2)
		{
			return FALSE;
		}
		
		echo '<div class="language">';
		foreach ($this->diafan->language_all as $key => $language)
		{
			echo
			($language[0] != _LANG
			? '<a href="'.BASE_PATH.($language[1] ? $language[1].'/' : '')
			  //.($_GET["rewrite"]?$_GET["rewrite"].'/':'')
			  .'">'
			: ''
			)
			.$language[2].($language[0] != _LANG ? '</a>' : '').' ';
		}
		echo '</div>';
		return TRUE;
	}

	/**
	 * Выводит период функционирования сайта в годах
	 * 
	 * @return boolean true
	 */
	public function show_year($attributes)
	{
		$attributes = $this->view->get_attributes($attributes, 'year');
		
		$year = preg_replace('/[^0-9]+/', '', $attributes["year"]);

		echo ($year ? $year : date("Y")).(date("Y") != $year && $year ? ' - '.date("Y")._LANG_YEARS : _LANG_YEAR);
		return TRUE;
	}

	/**
	 * Выводит содержимое страницы сайта, номер которой передан в виде атрибута id
	 * 
	 * @return boolean true
	 */
	public function show_block($attributes)
	{
		$attributes = $this->view->get_attributes($attributes, 'id');

		$this->view->get_function_in_theme(DB::title("site", $attributes["id"], "text"._LANG));
		return TRUE;
	}

	/**
	 * Выводит заголовок. Используется между тегами <title></title> в шапке сайта
	 * 
	 * @return boolean true
	 */
	public function show_title()
	{
		if($this->diafan->titlemodule_meta)
		{
			echo $this->diafan->titlemodule_meta;
			return TRUE;
		}
		elseif($this->diafan->title_meta)
		{
			echo ($this->diafan->titlemodule ? $this->diafan->titlemodule.' — ' : '').$this->diafan->title_meta;
			return TRUE;
		}
		echo ($this->diafan->titlemodule ? $this->diafan->titlemodule.' — ' : '').$this->diafan->name.(TITLE ? ' — '.TITLE : '');
		return TRUE;
	}

	/**
	 * Выводит мета-тег description страницы
	 * 
	 * @return boolean true
	 */
	public function show_description()
	{
		echo $this->diafan->descr;
		return TRUE;
	}

	/**
	 * Выводит ключевые слова страницы. Используется для мета-тега keywords
	 * 
	 * @return boolean true
	 */
	public function show_keywords()
	{
		echo $this->diafan->keywords;
		return TRUE;
	}

	/**
	 * Выводит JS код запрещающий копирование контента на сайте
	 * 
	 * @return boolean
	 */
	public function show_protect()
	{
		if(! $this->diafan->configmodules('protect', 'site'))
			return FALSE;
		echo ' oncopy="return false"';
		return TRUE;
	}

	/**
	 * Подключает JS файлы, обеспечивающие анимацию при открытии изображений
	 * 
	 * @return boolean
	 */
	public function show_prettyphoto()
	{
		if($this->diafan->configmodules('use_animation') || $this->diafan->configmodules('use_animation', 'site'))
		{
			echo '
		        <script src="'.BASE_PATH.'js/jquery.prettyPhoto.js" type="text/javascript" charset="utf-8"></script>
			<link rel="stylesheet" href="'.BASE_PATH.'css/prettyPhoto.css" type="text/css" media="screen"'
			.' title="prettyPhoto main stylesheet" charset="utf-8" />';
			return TRUE;
		}
		return FALSE;
	}

	/**
	 * Выводит фотографии, прикрепленные к странице сайта (если в конфигурации модуля «Страницы сайта» включен параметры «Использовать изображения»)
	 * 
	 * @return boolean
	 */
	public function show_photo()
	{
		if($this->diafan->configmodules('image', 'site'))
		{
			$site_pages = $this->diafan->get_site_pages();
			if(isset($site_pages[$this->diafan->cid." "]["photo"]))
			{
				echo $site_pages[$this->diafan->cid." "]["photo"];
				return FALSE;
			}
			
			$photos = '';
			$result = DB::query("SELECT id,name FROM {images} WHERE module_name='site' AND element_id='%d' ORDER BY sort ASC", $this->diafan->cid);

			while ($row = DB::fetch_array($result))
			{
				if(! $this->diafan->configmodules('use_animation', 'site'))
				{
					list($w, $h) = getimagesize(ABSOLUTE_PATH.USERFILES.'/'.$module.'/'.$row["name"].'_'.$row["id"].'.jpg');
					$link = 'href="#" onclick="javascript: window.open(\''.BASE_PATH.USERFILES.'/'.$module.'/'.$row["name"].'_'.$row["id"].'.jpg\',
					\'\', \'toolbar=no,location=no,status=no,menubar=no,scrollbars=yes,resizable=yes,width='.($w + 40).',height='.($h + 40)."'); return false;\"";
				}
				else
				{
					$link = 'href="'.BASE_PATH.USERFILES.'/site/'.$row["name"].'_'.$row["id"].'.jpg" rel="prettyPhoto[gallerySite]"';
				}
				list($width, $height) = getimagesize(USERFILES.'/site/medium/'.$row["name"].'_'.$row["id"].'.jpg');
				$photos .= '<a '.$link.'>'
				.'<img src="'.BASE_PATH.USERFILES.'/site/medium/'.$row["name"].'_'.$row["id"].'.jpg" width="'.$width.'" height="'.$height.'" alt="'.$this->diafan->name.'"></a> ';
			}
			$site_pages[$this->diafan->cid." "]["photo"] = $photos;
			if($this->diafan->_user->id)
			{
				$this->diafan->_cache->save($site_pages, "site_pages_only_user"._LANG);
			}
			else
			{
				$this->diafan->_cache->save($site_pages, "site_pages"._LANG);
			}

			echo $photos;

			return TRUE;
		}
		return FALSE;
	}

	/**
	 * Выводит голосование
	 * 
	 * @return boolean true
	 */
	public function show_votes($attributes)
	{
		$attributes = $this->view->get_attributes($attributes, 'id');
		
		$id = preg_replace('/[^0-9]+/', '', $attributes["id"]);
		$this->diafan->_votes->votes_show_votes($id);
		return TRUE;
	}

	/**
	 * Выводит статистику пользователей на сайте
	 * 
	 * @return boolean true
	 */
	public function show_users()
	{
		$timestamp = time() - 900;
		$count_user      = DB::query_result("SELECT COUNT(session_id) FROM {sessions_site} WHERE user_id = 0 AND timestamp >= %d", $timestamp);
		$count_user_auth = DB::query_result("SELECT COUNT(session_id) FROM {sessions_site} WHERE user_id > 0 AND timestamp >= %d", $timestamp);
		
		echo '<div class="show-users">'.sprintf(_LANG_USERS_NOW, $count_user, $count_user_auth).'</div>';
		return TRUE;
	}

	/**
	 * Выводит комментарии, прикрепленные к странице сайта (если в конфигурации модуля «Страницы сайты» подключены комментарии)
	 * 
	 * @return boolean
	 */
	public function show_comments()
	{
		if(! ($this->diafan->configmodules("comments") || $this->diafan->configmodules("comments_cat"))
		   && $this->diafan->configmodules("comments", "site"))
		{
			echo $this->diafan->_comments->get();
			return TRUE;
		}
		return FALSE;
	}

	/**
	 * Выводит ссылку на страницу сайта. Если текущая страница соответствует адресу, на которую ведет ссылка, то ссылка становится неактивной
	 * 
	 * @return boolean true
	 */
	public function show_href($a)
	{
		$a = $this->view->get_attributes($a, 'rewrite', 'img', 'img_act', 'width', 'height', 'alt', 'alt'._LANG);
		
		$rewrite = $this->diafan->translit($a["rewrite"]);
		$img = ($this->diafan->rewrite != $rewrite || ! $a["img_act"]
		       ? preg_replace('/[^A-Za-z0-9-_\/\.]+/', '', $a["img"])
		       : preg_replace('/[^A-Za-z0-9-_\/\.]+/', '', $a["img_act"])
		       );
		
		$width = preg_replace('/[^0-9]+/', '', $a["width"]);
		$height = preg_replace('/[^0-9]+/', '', $a["height"]);
		
		$name = ($a["alt"] == "title"
			? TITLE
			: ($a["alt"] == "url"
			  ? BASE_PATH
			  : $this->diafan->get_param($a, "alt".($a["alt"._LANG] ? _LANG : ''), '', 1)
			  )
			);
		
		echo ($this->diafan->rewrite != $rewrite || $this->diafan->show || $this->diafan->cat ? '<a href="'.BASE_PATH_HREF.($rewrite ? $rewrite.'/' : '').'">' : '')
		.($img ? '<img src="'.BASE_PATH.$img.'" alt="'.$name.'"'.($width ? ' width="'.$width.'"' : '').($height ? ' height="'.$height.'"' : '').'>' : $name)
		.($this->diafan->rewrite != $rewrite || $this->diafan->show || $this->diafan->cat ? '</a>' : '');
		return TRUE;
	}

	/**
	 * Выводит теги (слова-якори), прикрепленные к странице сайта (если в конфигурации модуля «Страницы сайты» подключены теги)
	 * 
	 * @return boolean
	 */
	public function show_tags()
	{
		if($this->diafan->configmodules("tags", "site"))
		{
			include_once(ABSOLUTE_PATH."modules/tags/tags.inc.php");
			$tags = new Tags_inc($this->diafan);
			echo $tags->get($this->diafan->cid, "site");
			return TRUE;
		}
		return FALSE;
	}
}