<?php
/**
 * Diafan.CMS
 *
 * @author     diafan.ru
 * @version    4.3
 * @license http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2011 OOO "Диафан". (http://diafan.ru)
 */

if(! defined('DIAFAN'))
{
	include(dirname(dirname(__FILE__)).'/includes/404.php');
}

/**
 * Image
 * Работа с изображениями
 */
class Image
{
	/**
	 * Изменяет размеры изображения
	 * @param string $src_image путь к файлу
	 * @param integer $dest_width новая ширина изображения
	 * @param integer $dest_height новая высота изображения
	 * @param integer $quality качество изображения
	 * @param  boolean $max изменять по максимальной стороне
	 */
	public function resize($src_image, $dest_width, $dest_height, $quality = 80, $max = FALSE)
	{ 
		if(! $quality)
		{
			$quality = 80;
		}
		if(! $src_image) 
			return false;
	
		$dest_image = $src_image;
	
		$info = getImageSize($src_image); 
		switch ($info[2])
		{ 
			case 1: 
				$src_img = imageCreateFromGIF($src_image); 
				break; 
			case 2: 
				$src_img = imageCreateFromJPEG($src_image); 
				break; 
			case 3: 
				$src_img = imageCreateFromPNG($src_image); 
				break; 
			default: 
				return FALSE; 
		}
		
		$src_width = imageSX ($src_img); 
		$src_height = imageSY ($src_img); 
		$mc1 = $dest_width / $src_width; 
		$mc2 = $dest_height / $src_height;
	
		if($max)
		{
			$k = max($mc1, $mc2);
		}
		else
		{
			$k = min($mc1, $mc2);
		}
	
		$dest_width = round($src_width * $k); 
		$dest_height = round($src_height * $k); 
		$dst_img = imageCreateTrueColor($dest_width, $dest_height); 
		imageCopyResampled($dst_img, $src_img, 0, 0, 0, 0, $dest_width, $dest_height, $src_width, $src_height); 
		imageJPEG($dst_img, $dest_image, $quality); 
		imageDestroy($src_img); 
		imageDestroy($dst_img);
	
		return TRUE; 
	}

	/**
	 * Получает изображения, прикрепленные к элементу модуля
	 *
	 * @param integer $id номер элемента, к которому прикреплены изображения
	 * @param string $alt альтернативный текст получаемых изображений
	 * @param string $module_name название модуля, по умолчанию модуль, прикрепленный к текущей странице
	 * @param integer $count количество изображений
	 * @param string|integer $common_link ссылка, на которую ведет изображение | 1 — сылка ведет на увеличенную копию изображений
	 * @param boolean $is_category изображения прикреплены к категории
	 * @return array
	 */
	public function get($id, $alt, $module_name, $count = 0, $common_link = 1, $is_category = FALSE)
	{
		global $diafan;
		if($diafan->configmodules('image'.($is_category ? '_cat' : ''), $module_name) || $module_name == "photo")
		{
			$photos = array();
			$result = DB::query("SELECT id, name, size, [alt], [title] FROM {images} WHERE module_name='%s' AND element_id='%d' AND trash='0' ORDER BY sort ASC"
					    .($count ? " LIMIT ".$count : ''), $module_name.($is_category ? 'cat' : ''), $id);
			
			while($row = DB::fetch_array($result))
			{
				if(! file_exists(ABSOLUTE_PATH.USERFILES.'/'.$module_name.'/'.$row["name"].'_'.$row["id"].'.jpg'))
				{
					DB::query("DELETE FROM {images} WHERE id=%d", $row["id"]);
					continue;
				}
				if($common_link == 1)
				{
					if(! $diafan->configmodules('use_animation', $module_name))
					{
						list($w, $h) = getimagesize(ABSOLUTE_PATH.USERFILES.'/'.$module_name.'/'.$row["name"].'_'.$row["id"].'.jpg');
						$img["link"] = USERFILES.'/'.$module_name.'/'.$row["name"].'_'.$row["id"].'.jpg"'
						.' onclick="javascript: window.open(\''.BASE_PATH.USERFILES.'/'.$module_name.'/'.$row["name"].'_'.$row["id"].'.jpg\','
						.'\'\', \'toolbar=no,location=no,status=no,menubar=no,scrollbars=yes,resizable=yes,width='.($w + 40).',height='.($h + 40)."'); return false;";
					}
					else
					{
						$img["link"] = USERFILES.'/'.$module_name.'/'.$row["name"].'_'.$row["id"].'.jpg" rel="prettyPhoto[gallery'.$id.$module_name.']';
					}
				}
				else
				{
					$img["link"] = $common_link;
				}
				list($width, $height) = getimagesize(ABSOLUTE_PATH.USERFILES.'/'.$module_name.'/medium/'.$row["name"].'_'.$row["id"].'.jpg');
				$img["tag"]  = '<img src="'.BASE_PATH.USERFILES.'/'.$module_name.'/medium/'.$row["name"].'_'.$row["id"].'.jpg" width="'.$width.'"'
				.' height="'.$height.'" alt="'.($row["alt"] ? $row["alt"] : $alt).'" title="'.($row["title"] ? $row["title"] : $alt).'">';
				$img["size"] = $diafan->convert($row["size"]);
				
				if($count == 1)
				{
					return $img;
				}
				$photos[] = $img;
			}
			return $photos;
		}
	}

	/**
	 * Получает большие копиии изображений, прикрепленных к элементу модуля
	 *
	 * @param integer $id номер элемента, к которому прикреплены изображения
	 * @param string $alt альтернативный текст получаемых изображений
	 * @param string $module_name название модуля, по умолчанию модуль, прикрепленный к текущей странице
	 * @param integer $count количество изображений
	 * @param boolean $is_category изображения прикреплены к категории
	 * @return array
	 */
	public function get_big($id, $alt, $module_name, $count = 0, $is_category = FALSE)
	{
		global $diafan;
		if($diafan->configmodules('image'.($is_category ? '_cat' : ''), $module_name) || $module_name == "photo")
		{
			$photos = array();
			$result = DB::query("SELECT id, name, size, [alt], [title] FROM {images} WHERE module_name='%s' AND element_id='%d' AND trash='0' ORDER BY sort ASC"
					    .($count ? " LIMIT ".$count : ''), $module_name.($is_category ? 'cat' : ''), $id);
			
			while($row = DB::fetch_array($result))
			{
				if(! file_exists(ABSOLUTE_PATH.USERFILES.'/'.$module_name.'/'.$row["name"].'_'.$row["id"].'.jpg'))
				{
					DB::query("DELETE FROM {images} WHERE id=%d", $row["id"]);
					continue;
				}
				list($width, $height) = getimagesize(ABSOLUTE_PATH.USERFILES.'/'.$module_name.'/'.$row["name"].'_'.$row["id"].'.jpg');
				$img["tag"]  = '<img src="'.BASE_PATH.USERFILES.'/'.$module_name.'/'.$row["name"].'_'.$row["id"].'.jpg" width="'.$width.'"'
				.' height="'.$height.'" alt="'.($row["alt"] ? $row["alt"] : $alt).'" title="'.($row["title"] ? $row["title"] : $alt).'">';
				
				if($count == 1)
				{
					return $img;
				}
				$photos[] = $img;
			}
			return $photos;
		}
	}
}