<?php
/**
 * Diafan.CMS
 *
 * @author     diafan.ru
 * @version    4.3
 * @license http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2011 OOO "Диафан". (http://diafan.ru)
 */

if(! defined('DIAFAN'))
{
	include(dirname(dirname(__FILE__)).'/includes/404.php');
}

/**
 * Model
 * 
 * Функции для форматирующие информацию в моделе модуля
 */
abstract class Model
{
	/**
	 * Форматирует дату в соответствии с конфигурацией модуля
	 * 
	 * @param integer $date дата в формет UNIX
	 * @param string $module_name название модуля, по умолчанию модуль, прикрепленный к текущей странице
	 * @param integer $site_id номер страницы сайта
	 * @return string
	 */
	protected function format_date($date, $module_name = '', $site_id = 0)
	{
		$months_array = array(
					'01' => _LANG_JANUARYTH,
					'02' => _LANG_FEBRUARYTH,
					'03' => _LANG_MARCHTH,
					'04' => _LANG_APRILTH,
					'05' => _LANG_MAYTH,
					'06' => _LANG_JUNETH,
					'07' => _LANG_JULYTH,
					'08' => _LANG_AUGUSTTH,
					'09' => _LANG_SEPTEMBERTH,
					'10' => _LANG_OCTOBERTH,
					'11' => _LANG_NOVEMBERTH,
					'12' => _LANG_DECEMBERTH
				);
		$week_array = array(
					'1' => _LANG_MONDAYTH,
					'2' => _LANG_TUESDAY,
					'3' => _LANG_WEDNESDAY,
					'4' => _LANG_THURSDAY,
					'5' => _LANG_FRIDAY,
					'6' => _LANG_SATURDAY,
					'0' => _LANG_SANDAY
				);
		if(! $module_name)
		{
			$module_name = $this->diafan->module;
		}
		if(! $site_id)
		{
			$site_id = $this->diafan->cid;
		}

		$config_format = $this->diafan->configmodules("format_date", $module_name, '', $site_id);

		switch($config_format)
		{
			case 1:
				return date("d ", $date).$months_array[date("m", $date)].date(" Y ", $date)._LANG_SHORT_YEAR;

			case 2:
				return date("d ", $date).$months_array[date("m", $date)];

			case 3:
				return date("d ", $date).$months_array[date("m", $date)].date(" Y, ", $date).$week_array[date("w", $date)];

			case 4:
				return '';

			case 5:
				return $this->format_date_5($date);

			default:
				return date("d.m.Y", $date);
		}
	}

	/**
	 * Получает псевдоссылку страницы сайта по названию модуля
	 * 
	 * @param string $module_name название модуля
	 * @return string
	 */
	protected function get_rewrite_from_module($module_name)
	{
		$site_pages = $this->diafan->get_site_pages();
		foreach($site_pages as $page)
		{
			if($page["addmodule"] == $module_name)
			{
				if(! empty($this->diafan->rewrites[$page["id"]]))
				{
					return $this->diafan->rewrites[$page["id"]]["rewrite"];
				}
				return '';
			}
		}
	}

	/**
	 * Получает имя и никнейм пользователя сайта
	 * 
	 * @param integer $author номер пользователя сайта
	 * @return string
	 */
	protected function get_author($author)
	{
		if(! $author) return _LANG_GUEST;

		global $authors;
		if(empty($authors[$author]))
		{
			if($this->diafan->_user->id != $author)
			{
				$authors[$author] = DB::fetch_array(DB::query("SELECT fio, name FROM {users_site} WHERE act='1' AND trash='0' AND id=%d", $author));
			}
			else
			{
				$authors[$author] = array("fio" => $this->diafan->_user->fio, "name" => $this->diafan->_user->name);
			}
		}
		return $authors[$author] ? $authors[$author]["fio"].' ('.$authors[$author]["name"].')' : '';
	}

	/**
	 * Осуществляет "Умное" форматирование даты
	 * 
	 * @param integer $date дата в формет UNIX
	 * @return string
	 */
	private function format_date_5($date)
	{
		if(! $date)
			return;

		$months_array = array(
					'01' => _LANG_JANUARYTH,
					'02' => _LANG_FEBRUARYTH,
					'03' => _LANG_MARCHTH,
					'04' => _LANG_APRILTH,
					'05' => _LANG_MAYTH,
					'06' => _LANG_JUNETH,
					'07' => _LANG_JULYTH,
					'08' => _LANG_AUGUSTTH,
					'09' => _LANG_SEPTEMBERTH,
					'10' => _LANG_OCTOBERTH,
					'11' => _LANG_NOVEMBERTH,
					'12' => _LANG_DECEMBERTH
				);
		$week_array = array(
					'1' => _LANG_MONDAYTH,
					'2' => _LANG_TUESDAY,
					'3' => _LANG_WEDNESDAY,
					'4' => _LANG_THURSDAY,
					'5' => _LANG_FRIDAY,
					'6' => _LANG_SATURDAY,
					'0' => _LANG_SANDAY
				);
		if(time() - $date < 3600)
		{
			$min = round((time() - $date) / 60);
			if($min < 2)
			{
				return _LANG_1_MIN_AGO;
			}
			if($min % 10 == 1 && $min > 20)
			{
				return sprintf(_LANG_SOME_1_MIN_AGO, $min);
			}
			if($min % 10 < 4 && ($min > 20 || $min < 10))
			{
				return sprintf(_LANG_SOME_2_MIN_AGO, $min);
			}
			return sprintf(_LANG_SOME_MIN_AGO, $min);
		}

		if($date >= mktime(0, 0, 0, date("m"), date("d"), date("Y")))
		{
			return _LANG_TODAY.', '.date("H:i", $date);
		}
		
		if($date >= mktime(0, 0, 0, date("m"), date("d"), date("Y")) - 86400)
		{
			return _LANG_YESTERDAY.', '.date("H:i", $date);
		}
		
		if($date >= time() - 86400 * 30)
		{
			return date("d ", $date).$months_array[date("m", $date)].', '.$week_array[date("w", $date)];
		}
		
		if(date("Y",$date) == date("Y"))
		{
			return date("d ", $date).$months_array[date("m", $date)];
		}
		return date("d ", $date).$months_array[date("m", $date)].date(" Y ", $date).' '._LANG_SHORT_YEAR;
	}

	/**
	 * Выдает ошибку о неправильном заполнении формы
	 * 
	 * @param string $module название модуля
	 * @param string $field название поля
	 * @return string
	 */
	protected function get_error($module = '', $field = '')
	{
		if(empty($_SERVER["HTTP_REFERER"]))
			return '';

		$ref = parse_url($_SERVER["HTTP_REFERER"]);

		if($ref["host"] != $_SERVER["HTTP_HOST"])
			return '';

		$field = 'mess'.($field ? '-'.$field : '');
		if(empty($_GET["error"]) || $_GET["error"] != $module || empty($_GET[$field]))
			return '';

		return $this->diafan->get_param($_GET, $field, '', 1);
	}

	/**
	 * Генерирует данные для навигации "Хлебные крошки"
	 *
	 * @param string $module модуль
	 * @return array
	 */
	protected function get_path($module)
	{
		$path = '';
		if(! empty($this->diafan->cat) || ! empty($this->diafan->show))
		{
			$path[] = array("link" => $this->diafan->rewrite ? $this->diafan->rewrite.'/' : '', "name" => $this->diafan->name);
		}

		if($this->diafan->cat)
		{
			$parents = array_reverse($this->diafan->get_parents($this->diafan->cat, $module.'_category'));

			if(! $this->diafan->show && ! isset($_GET["catid"]))
			{
				unset($parents[count($parents)-1]);
			}
			if(! empty($parents))
			{
				$result = DB::query("SELECT id, [name], site_id FROM {".$module."_category} WHERE id IN (%s)", implode(',', $parents));
				while ($row = DB::fetch_array($result))
				{
					$rows[$row["id"]] = $row;
				}
				foreach($parents as $p)
				{
					$path[] = array("link" => $this->get_link_category($rows[$p], $module), "name" => $rows[$p]["name"]);
				}
			}
		}
		return $path;
	}

	/**
	 * Генерирует ссылку для категории
	 *
	 * @param array $row данные о категории
	 * @param string $module модуль
	 * @return array
	 */
	protected function get_link_category($row, $module)
	{
		return (! empty($this->diafan->rewrites[$module]["cat".$row["id"]])
			? $this->diafan->rewrites[$module]["cat".$row["id"]]["rewrite"].'/'
			: (! empty($this->diafan->rewrites[$row["site_id"]]["rewrite"])
			   ? $this->diafan->rewrites[$row["site_id"]]["rewrite"].'/' : '').'cat'.$row["id"].'/'
			);
	}
}