<?php
/**
 * Diafan.CMS
 *
 * @author     diafan.ru
 * @version    4.3
 * @license http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2011 OOO "Диафан". (http://diafan.ru)
 */

if(! defined('DIAFAN'))
{
	include(dirname(dirname(__FILE__)).'/includes/404.php');
}

define('_LANG', '');

/**
 * Sitemap
 * 
 * Карта сайта
 */
class Sitemap
{
	/**
	 * @var array языковые версии сайта
	 */
	private $lang;

	/**
	 * @var array ЧПУ сайта
	 */
	private $rewrites;

	/**
	 * @var string перечень всех полей, указывающих на активность элемента
	 */
	private $act;

	/**
	 * @var object основной объект системы
	 */
	private $diafan;

	/**
	 * Конструктор класса
	 * 
	 * @return void
	 */
	public function __construct()
	{
		global $diafan;
		$this->diafan = $diafan;
		$this->get_language();
		$this->get_rewrites();
		header('Content-type: application/xml'); 
		echo '<?xml version="1.0" encoding="UTF-8"?>
		<urlset xmlns="http://www.sitemaps.org/schemas/sitemap/0.9">';
		$this->site_map();
		echo '</urlset>';
		
	}

	/**
	 * Получает языковые версии сайта
	 * 
	 * @return boolean true
	 */
	private function get_language()
	{
		$result = DB::query("SELECT id, shortname FROM {language}");
		while ($row = DB::fetch_array($result))
		{
			$this->lang[] = array("name" => $row["shortname"] ? $row["shortname"].'/' : '', "id" => $row["id"] !=1 ? $row["id"] : '');
			$this->act .= ", act".($row["id"] !=1 ? $row["id"] : '')."";
		}
		return TRUE;
	}

	/**
	 * Получает ЧПУ сайта
	 * 
	 * @return boolean true
	 */
	private function get_rewrites()
	{
		$result = DB::query("SELECT rewrite, element_id, cat_id, site_id, module_name FROM {rewrite}");
		while($row = DB::fetch_array($result))
		{
			if($row["module_name"] == "site")
			{
				$this->rewrites[$row["site_id"]] = $row["rewrite"];
			}
			elseif($row["cat_id"])
			{
				$this->rewrites[$row["module_name"]]["cat".$row["cat_id"]] = $row["rewrite"];
			}
			else
			{
				$this->rewrites[$row["module_name"]]["element".$row["element_id"]] = $row["rewrite"];
			}
		}
		return TRUE;
	}

	/**
	 * Формирует URL страниц сайта
	 * 
	 * @return boolean true
	 */
	function site_map()
	{
		$result = DB::query("SELECT id, addmodule, timeedit".$this->act." FROM {site} WHERE trash='0' AND othurl=''");
		while($row = DB::fetch_array($result))
		{
			if(in_array($row["addmodule"], array('map', 'search', 'reminding', 'registration', 'order', 'basket')))
			{
				continue;
			}
			foreach($this->lang as $l)
			{
				if(! $row["act".$l["id"]])
					continue;
				echo '<url>'
				.'<loc>'. BASE_PATH.$l["name"].(! empty($this->rewrites[$row["id"]]) ? $this->rewrites[$row["id"]].'/' : '').'</loc>'
				.'<lastmod>'.date('Y-m-d', $row["timeedit"]).'</lastmod>'
				.'</url>';
			}
			if($row["addmodule"] == "files" || $row["addmodule"] == "shop" || $row["addmodule"] == "photo"
			   || $row["addmodule"] == "clauses" || $row["addmodule"] == "news" || $row["addmodule"] == "faq")
			{
				if($this->diafan->configmodules("cat", $row["addmodule"], '', $row["id"]))
				{
					$this->category_map($row["addmodule"], $row["id"]);
					$this->element_map($row["addmodule"], $row["id"]);
				}
				else
				{
					$this->element_map($row["addmodule"], $row["id"]);			
				}
			}
			if($row["addmodule"] == "tags")
			{
				$this->tags_map($row["id"], $row["timeedit"]);
			}
		}
		return TRUE;
	}
	
	/**
	 * Формирует URL категорий модулей
	 * 
	 * @param string $module название модуля
	 * @param integer $site_id номер страницы
	 * @return boolean true
	 */
	function category_map($module, $site_id)
	{
		$result = DB::query("SELECT id, timeedit, site_id".$this->act." FROM {".$module."_category} WHERE trash='0' AND site_id=%d", $site_id);
		while ($row = DB::fetch_array($result))
		{
			$count = DB::query_result("SELECT COUNT(*) FROM {category} WHERE cat_id=%d AND module_name='%s'", $row["id"], $module);
			$nastr = $this->diafan->configmodules("nastr", $module, '', $row["site_id"]);
			$link = '';
			if(! empty($this->rewrites[$module]["cat".$row["id"]]))
			{
				$link = $this->rewrites[$module]["cat".$row["id"]].'/';
			}
			elseif(! empty($this->rewrites[$row["site_id"]]))
			{
				$link = $this->rewrites[$row["site_id"]].'/cat'.$row["id"].'/';
			}
			foreach($this->lang as $l)
			{
				if(! $row["act".$l["id"]])
					continue;
				echo '<url>'
				.'<loc>'. BASE_PATH.$l["name"].$link.'</loc>'
				.'<lastmod>'.date('Y-m-d', $row["timeedit"]).'</lastmod>'
				.'</url>';
				if($count > $nastr)
				{
					for($i = 2; $i <= ceil($count/$nastr); $i++)
					{
						echo '<url>'
						.'<loc>'.$count.' '.$nastr.' '.BASE_PATH.$l["name"].$link.'page'.$i.'/</loc>'
						.'<lastmod>'.date('Y-m-d', $row["timeedit"]).'</lastmod>'
						.'</url>';
					}
				}
			}
		}
		return TRUE;
	}
	
	/**
	 * Формирует URL элементов модулей, которые делятся категориями
	 * 
	 * @param string $module название модуля
	 * @param integer $site_id номер страницы
	 * @return boolean true
	 */
	function element_map($module, $site_id)
	{
		$result = DB::query("SELECT id, timeedit, site_id, cat_id".$this->act." FROM {".$module."} WHERE trash='0' AND site_id=%d", $site_id);
		while ($row = DB::fetch_array($result))
		{ 
			$link = '';
			if(! empty($this->rewrites[$module]["element".$row["id"]]))
			{
				$link = $this->rewrites[$module]["element".$row["id"]].'/';
			}
			elseif($this->diafan->configmodules("cat", $module, '', $site_id) && $row["cat_id"] && ! empty($this->rewrites[$module]["cat".$row["cat_id"]]))
			{
				$link = $this->rewrites[$module]["cat".$row["cat_id"]].'/show'.$row["id"].'/';
			}
			elseif(! empty($this->rewrites[$row["site_id"]]))
			{
				$link = $this->rewrites[$row["site_id"]]
				.($this->diafan->configmodules("cat", $module, '', $site_id) && $row["cat_id"] ? '/cat'.$row["cat_id"] : '')
				.'/show'.$row["id"].'/';
			}
			foreach($this->lang as $l)
			{
				if(! $row["act".$l["id"]])
					continue;
				echo '<url>'
				.'<loc>'. BASE_PATH.$l["name"].$link.'</loc>'
				.'<lastmod>'.date('Y-m-d', $row["timeedit"]).'</lastmod>'
				.'</url>';
			}
		}
		return TRUE;
	}
	
	/**
	 * Формирует URL тэгов
	 * 
	 * @param integer $site_id номер страницы сайта
	 * @param integer $timeedit время редактирования страницы
	 * @return boolean true
	 */
	function tags_map($site_id, $timeedit)
	{
		$result = DB::query("SELECT id FROM {tags_name} WHERE trash='0'");
		while ($row = DB::fetch_array($result))
		{
			$count = DB::query_result("SELECT COUNT(*) FROM {tags} WHERE tags_name_id=%d AND trash='0'", $row["id"]);
			$nastr = 10;
			$link = '';
			if(! empty($this->rewrites[$site_id]))
			{
				$link = $this->rewrites[$site_id].'/show'.$row["id"].'/';
			}
			foreach($this->lang as $l)
			{
				echo '<url>'
				.'<loc>'. BASE_PATH.$l["name"].$link.'</loc>'
				.'<lastmod>'.date('Y-m-d', $timeedit).'</lastmod>'
				.'</url>';
				if($count > $nastr)
				{
					for($i = 2; $i <= ceil($count/$nastr); $i++)
					{
						echo '<url>'
						.'<loc>'.$count.' '.$nastr.' '.BASE_PATH.$l["name"].$link.'page'.$i.'/</loc>'
						.'<lastmod>'.date('Y-m-d', $timeedit).'</lastmod>'
						.'</url>';
					}
				}
			}
		}
		return TRUE;
	}
}


$sitemap = new Sitemap();
exit;
