<?php
/**
 * Diafan.CMS
 *
 * @author     diafan.ru
 * @version    4.3
 * @license http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2011 OOO "Диафан". (http://diafan.ru)
 */

if(! defined('DIAFAN'))
{
	include(dirname(dirname(dirname(__FILE__))).'/includes/404.php');
}

/**
 * Faq_ajax
 *
 * Обработка запроса при отправки сообщения из формы "Задайте вопрос"
 */
class Faq_ajax
{
	/**
	 * @var object основной объект системы
	 */
	public $diafan;

	/**
	 * @var array внутренние сообщения
	 */
	private $result;

	/**
	 * Конструктор класса
	 * 
	 * @return void
	 */
	public function __construct($diafan)
	{
		$this->diafan = $diafan;
	}

	/**
	 * Обрабатывает полученные данные из формы
	 * 
	 * @return boolean
	 */
	public function ajax_request()
	{
		if(! empty($_POST['ajax_faq']) && ($this->diafan->configmodules('security', "faq") !=  1 || $this->diafan->_user->id))
		{
			$this->check_captcha();
			$this->empty_field();
			$this->valid_email();

			if($this->send_errors())
				return TRUE;

			if($this->error_insert())
				return TRUE;

			if(! DB::query("INSERT INTO {faq} (created, mail, [anons], [name], cat_id, site_id) VALUES (%d, '%h', '%h', '%h', %d, %d)",
				      time(), $_POST["email"], $_POST["question"], $_POST["name"], $this->diafan->cat, $this->diafan->cid))
			{
				$mes = $this->diafan->configmodules('error-add-message', 'faq');
				$this->result["errors"][0] = $mes ? $mes : ' ';
				return $this->send_errors();
			}
			$save = DB::next_id("faq");

			$err = '';
			$this->result["altnames"] = '';
			if (! empty($_FILES['attachment']) && $this->diafan->configmodules('files', "faq"))
			{
				include_once (ABSOLUTE_PATH."includes/files.inc.php");
				$files = new Files($this->diafan);
				foreach ($_FILES['attachment']['tmp_name'] as $n => $dummy)
				{
					if($this->diafan->configmodules("max-count-files", "faq") && $n > $this->diafan->configmodules("max-count-files", "faq"))
						break;

					$err .= $files->file_upload($_FILES['attachment'], $this->diafan->module, $save, $n);
					$this->result["altnames"] .= " ".$_FILES['attachment']['name'][$n];
				}
			}

			if($err)
			{
				DB::query("DELETE FROM {faq} WHERE id='%d'", $save);
				DB::query("DELETE FROM {attachment} WHERE module_name='%h' AND element_id='%d'", $this->diafan->module, $save);
				$this->result["errors"]["attachments"] = $err;
				return $this->send_errors();
			}

			$this->send_mail();

			$mes = $this->diafan->configmodules('add-message', 'faq');
			$this->result["errors"][0] = $mes ? $mes : ' ';
			$this->result["success"] = TRUE;
			return $this->send_errors();
		}
	}

	/**
	 * Проверяет правильность каптчи
	 * 
	 * @return boolean
	 */
	private function check_captcha()
	{
		$this->result["code"] = '';
		if($this->diafan->configmodules('security', 'faq') == 2)
		{
			require_once(ABSOLUTE_PATH.'includes/captcha/captcha.inc.php');	
			$mes = Captcha_inc::is_right($this->diafan->module);
			$this->result["code"] = Captcha_inc::get('faq', '', TRUE);				
			if($mes)
			{
				$this->result["errors"]["code"] = $mes;
				return TRUE;
			}
		}
		return FALSE;
	}

	/**
	 * Проверяет e-mail на валидность
	 * 
	 * @return boolean true
	 */
	private function valid_email()
	{
		if($_POST["email"])
		{
			require_once(ABSOLUTE_PATH.'includes/validate.php');
			$mes = Validate::mail($_POST["email"]);
			if($mes)
			{
				$this->result["errors"]["email"] = $mes;
			}
		}
		return TRUE;
	}

	/**
	 * Проверяет на заполнение обязательных полей
	 * 
	 * @return boolean true
	 */
	private function empty_field()
	{
		if(empty($_POST['name']))
		{
			$this->result["errors"]["name"] = _LANG_ENTER_NAME;
		}
		if(empty($_POST['question']))
		{
			$this->result["errors"]["question"] = _LANG_ENTER_QUESTION;
		}
		return TRUE;
	}

	/**
	 * Проверяет на попытку отправить сообщение повторно
	 * 
	 * @return boolean
	 */
	private function error_insert()
	{
		$mes = '';
		$num = DB::query_result("SELECT COUNT(id) FROM {faq} where mail='%h' AND anons='%h'", $_POST["email"], $_POST["question"]);
		if ($num > 0)
		{ 
			$mes = $this->diafan->configmodules('error-insert-message');
			$this->result["errors"][0] = $mes ? $mes : ' ';
		}
		return $this->send_errors();
	}

	/**
	 * Уведомляет администратора по e-mail
	 * 
	 * @return boolean
	 */
	private function send_mail()
	{
		if(! $this->diafan->configmodules("sendmailadmin"))
			return FALSE;
			
		$subject = str_replace(
			array('%title', '%url'),
			array(TITLE, BASE_URL),
			$this->diafan->configmodules("subject-admin")
		);
			
		$message = str_replace(
			array('%name', '%title', '%url', '%question', '%email', '%files'),
			array(
				$this->diafan->get_param($_POST, "name", '', 1),
				TITLE,
				BASE_URL,
				$this->diafan->get_param($_POST, "question", '', 1),
				$this->diafan->get_param($_POST, "email", '', 1),
				($this->result["altnames"] ? $this->result["altnames"] : '-')
			),
			$this->diafan->configmodules("message-admin")
		);

		$to   = $this->diafan->configmodules("emailconfadmin") ? $this->diafan->configmodules("email-admin") : EMAIL_CONFIG;
		$from = $this->diafan->configmodules("emailconf") ? $this->diafan->configmodules("email") : '';

		include_once(ABSOLUTE_PATH.'includes/mail.php');
		send_mail($to, $subject, $message, $from);

		return TRUE;
	}

	/**
	 * Отправляет ответ
	 *
	 * @return boolean
	 */
	private function send_errors()
	{
		if(! empty($this->result["errors"]))
		{
			if(! empty($_POST['ajax']))
			{
				include_once(ABSOLUTE_PATH.'plugins/json.php');
				echo to_json($this->result);
			}
			else
			{
				$query = '';
				if(! empty($this->result["errors"]))
				{
					foreach($this->result["errors"] as $field => $mes)
					{
						if(empty($_POST["update"]))
						{
							$query .= '&mess'.($field ? '-'.$field : '').'='.$mes;
						}
					}
				}
				$this->diafan->goto_diafan($this->diafan->path_module_variable().($query ? '?error=faq'.$query : ''));
			}
			return TRUE;
		}
		return FALSE;
	}
}