<?php
/**
 * Diafan.CMS
 *
 * @author     diafan.ru
 * @version    4.3
 * @license http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2011 OOO "Диафан". (http://diafan.ru)
 */

if(! defined('DIAFAN'))
{
	include(dirname(dirname(dirname(__FILE__))).'/includes/404.php');
}

/**
 * Faq_model
 *
 * Модель модуля "Вопрос-Ответ"
 */
class Faq_model extends Model
{
	/**
	 * @var array сгенерированные в моделе данные, передаваемые в шаблон
	 */
	protected $result;

	/**
	 * @var object основной объект системы
	 */
	protected $diafan;

	/**
	 * Конструктор класса
	 * 
	 * @return void
	 */
	public function __construct($diafan)
	{
		$this->diafan = $diafan;
	}

	/**
	 * Генерирует данные для списка всех вопросов без деления на категории
	 * 
	 * @return array
	 */
	public function list_()
	{
		if($this->diafan->cat)
		{
			$this->diafan->goto_diafan($this->diafan->path_module_variable("cat"));
		}
		$cache_meta = array(
				    "name"     => "list",
				    "language" => _LANG,
				    "page"     => $this->diafan->page > 1 ? $this->diafan->page : 1,
				    "site_id"  => $this->diafan->cid
				    );

		//кеширование
		if(! $this->result = $this->diafan->_cache->get($cache_meta, $this->diafan->module))
		{
			$this->result = array();

			////navigation//
			include_once(ABSOLUTE_PATH.'includes/paginator/paginator.inc.php');
			$paginator = new Paginator_inc();
			$paginator->page    = $this->diafan->page;
			$paginator->navlink = $this->diafan->path_module_variable("page");
			$paginator->nen = DB::query_result("SELECT COUNT(id) FROM {faq} WHERE [act]='1' AND trash='0' AND site_id=%d", $this->diafan->cid);
			$links = $paginator->get();
			////navigation///

			$result = DB::query_range("SELECT id, created, [name], [anons], [text], timeedit, cat_id, site_id FROM {faq}"
						 ." WHERE [act]='1' AND trash='0' AND site_id=%d"
						 ." ORDER BY created DESC, id DESC",
						 $this->diafan->cid, $paginator->polog, $paginator->nastr);

			$this->result["rows"] = $this->get_elements($result);

			$this->result["paginator"] = $links;
			
			//сохранение кеша
			$this->diafan->_cache->save($this->result, $cache_meta, $this->diafan->module);
		}
		else
		{
			include_once(ABSOLUTE_PATH.'includes/paginator/paginator.view.php');
		}

		if($this->result["rows"])
		{
			if($this->diafan->configmodules("rating"))
			{
				include_once(ABSOLUTE_PATH.'modules/rating/rating.inc.php');
				$rating = new Rating_inc($this->diafan);
			}
			if($this->diafan->configmodules("tags"))
			{
				include_once(ABSOLUTE_PATH."modules/tags/tags.inc.php");
				$tags = new Tags_inc($this->diafan);
			}

			for($i = 0; $i < count($this->result["rows"]); $i++)
			{
				$this->result["rows"][$i]["tags"] = $this->diafan->configmodules("tags")
								    ? $tags->get($this->result["rows"][$i]["id"])
								    : '';

				$this->result["rows"][$i]["show-rating"] = $this->diafan->configmodules("rating")
									   ? $rating->get($this->result["rows"][$i]["id"])
									   : '';
			}
		}
		return $this->result;
		
	}

	/**
	 * Генерирует данные для первой страницы модуля "Вопрос-Ответ"
	 * 
	 * @return array
	 */
	public function first_page()
	{
		if($this->diafan->page)
		{
			$this->diafan->goto_diafan($this->diafan->path_module_variable("page"));
		}
		//кеширование
		$cache_meta = array(
				    "name"     => "first_page",
				    "language" => _LANG,
				    "site_id"  => $this->diafan->cid
				    );
		if(! $this->result = $this->diafan->_cache->get($cache_meta, $this->diafan->module))
		{
			$this->result = array();

			$result = DB::query("SELECT id, [name], [anons], timeedit, site_id FROM {faq_category}"
					    ." WHERE [act]='1' AND parent_id=0 AND trash='0' AND site_id=%d"
					    ." ORDER by sort ASC, id ASC", $this->diafan->cid);

			while ($row = DB::fetch_array($result))
			{
				$this->result["categories"][$row["id"]]["name"] = $row["name"];
				$this->result["categories"][$row["id"]]["anons"] = $row["anons"];

				if(empty($this->result["timeedit"]) || $row["timeedit"] > $this->result["timeedit"])
				{
					$this->result["timeedit"] = $row["timeedit"];
				}

				$this->result["categories"][$row["id"]]["children"] = array();
				$result_children = DB::query("SELECT id, [name], [anons], site_id FROM {faq_category}"
							     ." WHERE [act]='1' AND parent_id=%d AND trash='0'"
							     ." AND site_id=%d ORDER BY sort ASC, id ASC",
							     $row["id"], $this->diafan->cid);

				while ($row_children = DB::fetch_array($result_children))
				{
					$child["link"]  = $this->get_link_category($row_children, 'faq');
					$child["name"]  = $row_children["name"];
					$child["anons"] = $row_children["anons"];
					$this->result["categories"][$row["id"]]["children"][] = $child;
				}

				if($this->diafan->configmodules("childrens_elements"))
				{
					$children = $this->diafan->get_children($row["id"],"faq_category");
				}

				$result_img = DB::query_range("SELECT e.id, e.[name], e.cat_id, e.timeedit, e.[anons], e.[text], e.site_id, e.created"
							      ." FROM {faq} AS e"
							      ." INNER JOIN {category} AS c ON e.id=c.element_id"
							      ." WHERE c.module_name='faq' AND e.cat_id IN (%s) AND e.[act]='1' AND e.trash='0'"
							      ." GROUP BY e.id ORDER BY e.created DESC, e.id DESC",
							      $this->diafan->configmodules("childrens_elements") ? implode(',', $children) : $row["id"],
							      0, $this->diafan->configmodules("count_list")
							      );
				$this->result["categories"][$row["id"]]["rows"] = $this->get_elements($result_img);

				$this->result["categories"][$row["id"]]["allfaq"]["link"] = $this->get_link_category($row, 'faq');
				$this->result["categories"][$row["id"]]["allfaq"]["text"] = $this->diafan->configmodules("allfaq").' '._LANG_LQUOT.$row["name"]._LANG_RQUOT;
			}

			//сохранение кеша
			$this->diafan->_cache->save($this->result, $cache_meta, $this->diafan->module);
		}

		if(! empty($this->result["categories"]))
		{
			if($this->diafan->configmodules("rating"))
			{
				include_once(ABSOLUTE_PATH.'modules/rating/rating.inc.php');
				$rating = new Rating_inc($this->diafan);
			}
			if($this->diafan->configmodules("tags"))
			{
				include_once(ABSOLUTE_PATH."modules/tags/tags.inc.php");
				$tags = new Tags_inc($this->diafan);
			}

			foreach($this->result["categories"] as $cat => $rows)
			{
				if(! $rows)
					continue;

				foreach($rows["rows"] as $k => $row)
				{
					$this->result["categories"][$cat]["rows"][$k]["tags"] = ($this->diafan->configmodules("tags")
									   ? $tags->get($this->result["categories"][$cat]["rows"][$k]["id"])
									   : '');
	
					$this->result["categories"][$cat]["rows"][$k]["show-rating"] = ($this->diafan->configmodules("rating")
									  ? $rating->get($this->result["categories"][$cat]["rows"][$k]["id"])
									  : '');
				}
			}
		}
		return $this->result;
	}

	/**
	 * Генерирует данные для списка вопросов в категории
	 * 
	 * @return array
	 */
	public function list_category()
	{
		//кеширование
		$cache_meta = array(
				    "name"     => "list",
				    "cat_id"   => $this->diafan->cat,
				    "language" => _LANG,
				    "page"     => $this->diafan->page > 1 ? $this->diafan->page : 1,
				    "site_id"  => $this->diafan->cid
				    );
		if(! $this->result = $this->diafan->_cache->get($cache_meta, $this->diafan->module))
		{
			$this->result = array();

			$this->result["level"] = count($this->diafan->get_parents($this->diafan->cat, 'faq_category'));
			
			$this->result["path"] = $this->get_path('faq');

			$result = DB::query("SELECT id, [name], timeedit, [descr], [keywords], sort, parent_id, [title_meta] FROM {faq_category}"
					    ." WHERE [act]='1' and id=%d AND trash='0' AND site_id=%d"
					    ." ORDER BY sort ASC, id ASC",
					    $this->diafan->cat, $this->diafan->cid);
				
			if(! $row = DB::fetch_array($result))
			{
				include_once(ABSOLUTE_PATH.'includes/404.php');
			}
				
			if($this->diafan->configmodules("childrens_elements"))
			{
				$children = $this->diafan->get_children($this->diafan->cat, "faq_category");
			}

			$this->result["titlemodule"] = $row["name"];
			$this->result["title_meta"]  = $row["title_meta"];
			$this->result["timeedit"]    = $row["timeedit"];
			$this->result["keywords"]    = $row["keywords"];
			$this->result["descr"]       = $row["descr"];

			$previous = DB::fetch_array(DB::query("SELECT [name], id FROM {faq_category} WHERE [act]='1' AND trash='0' AND site_id=%d"
							      ." AND (sort<%d OR sort=%d AND id<%d) AND parent_id=%d ORDER BY sort DESC, id DESC LIMIT 1",
							      $this->diafan->cid, $row["sort"], $row["sort"], $this->diafan->cat, $row["parent_id"]));
			if($previous)
			{
				$this->result["previous"]["text"] = $previous["name"];
				if(! empty($this->diafan->rewrites["faq"]["cat".$previous["id"]]))
				{
					$this->result["previous"]["link"] = $this->diafan->rewrites["faq"]["cat".$previous["id"]]["rewrite"].'/';
				}
				else
				{
					$this->result["previous"]["link"] = ($this->diafan->rewrite ? $this->diafan->rewrite.'/' : '')
					                                    .'cat'.$previous["id"].'/';
				}
			}
			$next = DB::fetch_array(DB::query("SELECT [name], id FROM {faq_category} WHERE [act]='1' AND trash='0' AND site_id=%d"
							  ." AND (sort>%d OR sort=%d AND id>%d) AND parent_id=%d ORDER BY sort ASC, id ASC LIMIT 1",
							  $this->diafan->cid, $row["sort"], $row["sort"], $this->diafan->cat, $row["parent_id"]));
			if($next)
			{
				$this->result["next"]["text"] = $next["name"];
				if(! empty($this->diafan->rewrites["faq"]["cat".$next["id"]]))
				{
					$this->result["next"]["link"] = $this->diafan->rewrites["faq"]["cat".$next["id"]]["rewrite"].'/';
				}
				else
				{
					$this->result["next"]["link"] = ($this->diafan->rewrite ? $this->diafan->rewrite.'/' : '')
									.'cat'.$next["id"].'/';
				}
			}
			
			$this->result["children"] = array();
			$result_children = DB::query("SELECT id, [name], [anons], site_id FROM {faq_category}"
						    ." WHERE [act]='1' AND parent_id=%d AND trash='0'"
						    ." AND site_id=%d ORDER BY sort ASC, id ASC",
						    $row["id"], $this->diafan->cid);
				
			while ($row_children = DB::fetch_array($result_children))
			{
					$child["link"]  = $this->get_link_category($row_children, 'faq');
					$child["name"]  = $row_children["name"];
					$child["anons"] = $row_children["anons"];
					$this->result["children"][] = $child;
			}

			////navigation//
			include_once(ABSOLUTE_PATH.'includes/paginator/paginator.inc.php');
			$paginator = new Paginator_inc();
			$paginator->page    = $this->diafan->page;
			$paginator->navlink = $this->diafan->path_module_variable("page");
			$paginator->nen = DB::query_result("SELECT COUNT(DISTINCT e.id) FROM {faq} as e"
							   ." INNER JOIN {category} AS c ON e.id=c.element_id AND c.module_name='faq'"
							   ." AND e.id=c.element_id AND c.cat_id IN (%s)"
							   ." WHERE e.[act]='1' AND e.trash='0' AND e.site_id=%d",

							   $this->diafan->configmodules("childrens_elements")
							   ? implode(',', $children) : $this->diafan->cat,
	
							   $this->diafan->cid
							   );
			$links = $paginator->get();
			////navigation///

			$result_img = DB::query_range("SELECT e.id, e.[name], e.cat_id, e.timeedit, e.[anons], e.[text], e.site_id, e.created FROM"
						      ." {faq} AS e"
						      ." INNER JOIN {category} AS c ON c.module_name='faq' AND e.id=c.element_id AND c.cat_id IN (%s)"
						      ." WHERE e.[act]='1' AND e.trash='0' GROUP BY e.id"
						      ." ORDER BY e.created DESC, e.id DESC",
						      $this->diafan->configmodules("childrens_elements") ? implode(',', $children) : $this->diafan->cat,
						      $paginator->polog, $paginator->nastr
						     );

			$this->result["rows"] = $this->get_elements($result_img);

			$this->result["paginator"] = $links;

			//сохранение кеша
			$this->diafan->_cache->save($this->result, $cache_meta, $this->diafan->module);
		}
		else
		{
			include_once(ABSOLUTE_PATH.'includes/paginator/paginator.view.php');
		}

		$this->result["text"] = DB::query_result("SELECT [text] FROM {faq_category} WHERE id=%d", $this->diafan->cat);

		if($this->diafan->configmodules("cat") && ! empty($this->diafan->cat) && $this->diafan->configmodules("comments_cat"))
		{
			$this->result["comments"] = $this->diafan->_comments->get();
		}

		if($this->result["rows"])
		{
			if($this->diafan->configmodules("rating"))
			{
				include_once(ABSOLUTE_PATH.'modules/rating/rating.inc.php');
				$rating = new Rating_inc($this->diafan);
			}
			if($this->diafan->configmodules("tags"))
			{
				include_once(ABSOLUTE_PATH."modules/tags/tags.inc.php");
				$tags = new Tags_inc($this->diafan);
			}

			for($i = 0; $i < count($this->result["rows"]); $i++)
			{
				$this->result["rows"][$i]["tags"] = ($this->diafan->configmodules("tags")
								    ? $tags->get($this->result["rows"][$i]["id"])
								    : '');

				$this->result["rows"][$i]["show-rating"] = ($this->diafan->configmodules("rating")
									   ? $rating->get($this->result["rows"][$i]["id"])
									   : '');
			}
		}
		return $this->result;
	}

	/**
	 * Генерирует данные для страницы вопроса
	 * 
	 * @return array|boolean false
	 */
	public function id()
	{
		//кеширование
		$cache_meta = array(
				    "name"     => "show",
				    "cat_id"   => $this->diafan->cat,
				    "show"     => $this->diafan->show,
				    "language" => _LANG,
				    "site_id"  => $this->diafan->cid
				    );
		if(! $this->result = $this->diafan->_cache->get($cache_meta, $this->diafan->module))
		{
			$row = DB::fetch_array(DB::query("SELECT id, [name], timeedit, created, cat_id, [keywords], [descr], [anons], site_id, [title_meta]"
							 ." FROM {faq}  WHERE [act]='1'"
							 ." AND id = %d AND trash='0' AND site_id=%d",
							 $this->diafan->show, $this->diafan->cid));
			if(empty($row))
			{
				include(ABSOLUTE_PATH.'includes/404.php');
				return FALSE;
			}

			if(! $this->diafan->rewrite_module && ($this->diafan->configmodules("cat") && $this->diafan->cat != $row["cat_id"]
							       || ! $this->diafan->configmodules("cat") && $this->diafan->cat))
			{
				include(ABSOLUTE_PATH.'includes/404.php');
				return FALSE;
			}
			$this->diafan->cat           = $this->diafan->configmodules("cat") ? $row["cat_id"] : 0;
			$this->result["cat_id"]      = $this->diafan->configmodules("cat") ? $row["cat_id"] : 0;

			if($row["cat_id"] && ! empty($this->diafan->rewrites["faq"]["cat".$row["cat_id"]]))
			{
				$this->result["allfaq"]["link"] = $this->diafan->rewrites["faq"]["cat".$row["cat_id"]]["rewrite"].'/';
			}
			else
			{
				$this->result["allfaq"]["link"] = (! empty($this->diafan->rewrites[$row["site_id"]]["rewrite"])
								   ? $this->diafan->rewrites[$row["site_id"]]["rewrite"].'/' : '')
				                                  .($row["cat_id"] ? 'cat'.$row["cat_id"].'/' : '');
			}

			$previous = DB::fetch_array(DB::query("SELECT [anons], id, cat_id FROM {faq} WHERE [act]='1' AND trash='0' AND site_id=%d"
							      .($this->diafan->configmodules("cat") ? " AND cat_id='".$this->diafan->cat."'" : '')
							      ." AND (created<%d OR created=%d AND id<%d) ORDER BY created DESC, id DESC LIMIT 1",
							      $this->diafan->cid, $row["created"], $row["created"], $this->diafan->show));
			if($previous)
			{
				$this->result["previous"]["text"] = $this->diafan->short_text($previous["anons"]);
				if(! empty($this->diafan->rewrites["faq"]["element".$previous["id"]]))
				{
					$this->result["previous"]["link"] = $this->diafan->rewrites["faq"]["element".$previous["id"]]["rewrite"].'/';
				}
				elseif($previous["cat_id"] && $this->diafan->configmodules("cat") && ! empty($this->diafan->rewrites["faq"]["cat".$previous["cat_id"]]))
				{
					$this->result["previous"]["link"] = $this->diafan->rewrites["faq"]["cat".$previous["cat_id"]]["rewrite"].'/'
									    .'show'.$previous["id"].'/';
				}
				else
				{
					$this->result["previous"]["link"] = ($this->diafan->rewrite ? $this->diafan->rewrite.'/' : '')
					                                    .($previous["cat_id"] && $this->diafan->configmodules("cat")
									      ? 'cat'.$previous["cat_id"].'/' : '')
									    .'show'.$previous["id"].'/';
				}
			}
			$next = DB::fetch_array(DB::query("SELECT [anons], id, cat_id FROM {faq} WHERE [act]='1' AND trash='0' AND site_id=%d"
							  .($this->diafan->configmodules("cat") ? " AND cat_id='".$this->diafan->cat."'" : '')
							  ." AND (created>%d OR created=%d AND id>%d) ORDER BY created ASC, id ASC LIMIT 1",
							  $this->diafan->cid, $row["created"], $row["created"], $this->diafan->show));
			if($next)
			{
				$this->result["next"]["text"] = $this->diafan->short_text($next["anons"]);
				if(! empty($this->diafan->rewrites["faq"]["element".$next["id"]]))
				{
					$this->result["next"]["link"] = $this->diafan->rewrites["faq"]["element".$next["id"]]["rewrite"].'/';
				}
				elseif($next["cat_id"] && $this->diafan->configmodules("cat") && ! empty($this->diafan->rewrites["faq"]["cat".$next["cat_id"]]))
				{
					$this->result["next"]["link"] = $this->diafan->rewrites["faq"]["cat".$next["cat_id"]]["rewrite"].'/'
					                                .'show'.$next["id"].'/';
				}
				else
				{
					$this->result["next"]["link"] = ($this->diafan->rewrite ? $this->diafan->rewrite.'/' : '')
					                                .($next["cat_id"] && $this->diafan->configmodules("cat")
									  ? 'cat'.$next["cat_id"].'/' : '')
									.'show'.$next["id"].'/';
				}
			}

			$this->result["allfaq"]["text"] = $this->diafan->configmodules("back-to-list");
			$this->result["date"]           = $this->format_date($row['created']);
			$this->result["anons"]          = $row["anons"];

			//$this->result["titlemodule"]    = $row["name"];
			$this->result["title_meta"]     = $row["title_meta"];
			$this->result["timeedit"]       = $row["timeedit"];
				
			$this->result["keywords"]       = $row["keywords"];
			$this->result["descr"]          = $row["descr"];
			
			$this->result["path"]           = $this->get_path('faq');

			//сохранение кеша
			$this->diafan->_cache->save($this->result, $cache_meta, $this->diafan->module);
		}
		$this->diafan->cat = $this->result["cat_id"];

		$this->result["text"] = DB::query_result("SELECT [text] FROM {faq} WHERE id=%d", $this->diafan->show);

		if($this->diafan->configmodules("rating"))
		{
			include_once(ABSOLUTE_PATH.'modules/rating/rating.inc.php');
			$rating = new Rating_inc($this->diafan);
			$this->result["show-rating"] = $rating->get();
			$this->result["add-rating"] = $rating->add();			
		}

		if($this->diafan->configmodules("comments"))
		{
			$this->result["comments"] = $this->diafan->_comments->get();
		}

		if($this->diafan->configmodules("tags"))
		{
			include_once(ABSOLUTE_PATH."modules/tags/tags.inc.php");
			$tags = new Tags_inc($this->diafan);
			$this->result["tags"] = $tags->get();
		}

		return $this->result;
	}

	/**
	 * Генерирует данные для формы добавления вопроса
	 *
	 * @param integer $site_id номер страницы
	 * @param integer $cat_id номер категории
	 * @param boolean $insert_form форма выводится с помощью шаблонного тэга
	 * @return array|boolean false
	 */
	public function form($site_id = 0, $cat_id = 0, $insert_form = FALSE)
	{
		if(! $insert_form)
		{
			$site_id = $this->diafan->cid;
			$cat_id = $this->diafan->cat;
		}
		else
		{
			if(! $site_id)
			{
				$site_id = DB::query_result("SELECT id FROM {site} WHERE addmodule='faq' LIMIT 1");
			}
			if(! $cat_id && $this->diafan->configmodules("cat", "faq", _LANG, $site_id))
			{
				$cat_id = DB::query_result("SELECT id FROM {faq_category} WHERE site_id='%d' LIMIT 1", $site_id);
			}
		}
		if(! $site_id)
		{
			return FALSE;
		}

		if(! $this->diafan->configmodules("cat", "faq", _LANG, $site_id))
		{
			$cat_id = 0;
		}

		if(($insert_form ||
		   $this->diafan->configmodules('show-form', 'faq') 
		   && (
			   $this->diafan->configmodules('show-form', 'faq', _LANG, $site_id) == 3 
			|| $this->diafan->configmodules('show-form', 'faq', _LANG, $site_id) == 1 && $this->diafan->show
			|| $this->diafan->configmodules('show-form', 'faq', _LANG, $site_id) == 2 && ! $this->diafan->show
		   ))
		   && ($this->diafan->configmodules('security', 'faq', _LANG, $site_id) != 1 || $this->diafan->_user->id))
		{
			$this->result["form"]["captcha"] = '';
			if($this->diafan->configmodules('security', 'faq', _LANG, $site_id) == 2)
			{
				require_once(ABSOLUTE_PATH.'includes/captcha/captcha.inc.php');
				$this->result["form"]["captcha"] = Captcha_inc::get("faq", $this->get_error("faq", "code"));
			}
			$this->result["form"]["ok"]     = 1;
			$this->result["form"]["action"] = '';
			if($cat_id && !empty($this->diafan->rewrites["faq"]["cat".$cat_id]))
			{
				$this->result["form"]["action"] = $this->diafan->rewrites["faq"]["cat".$cat_id]["rewrite"].'/';
			}
			elseif(! empty($this->diafan->rewrites[$site_id]["rewrite"]))
			{
				$this->result["form"]["action"] = ($this->diafan->rewrites[$site_id]["rewrite"]
								   ? $this->diafan->rewrites[$site_id]["rewrite"].'/' : '')
				                                  .($cat_id ? 'cat'.$cat_id.'/' : '');
			}

			$this->result["form"]["files"]  = 0;

			if($this->diafan->configmodules('files', 'faq', _LANG, $site_id) && $this->diafan->configmodules("max-count-files", 'faq', _LANG, $site_id))
			{
				$this->result["form"]["files"] = $this->diafan->configmodules("max-count-files", 'faq', _LANG, $site_id);
			}
			$this->result["form"]["error"]             = $this->get_error("faq");
			$this->result["form"]["error-name"]        = $this->get_error("faq", 'name');
			$this->result["form"]["error-question"]    = $this->get_error("faq", 'question');
			$this->result["form"]["error-email"]       = $this->get_error("faq", 'email');
			$this->result["form"]["error-attachments"] = $this->get_error("faq", 'attachments');
		}
		return $this->result;
	}

	/**
	 * Генерирует данные для шаблонной функции: вопросов и ответов
	 * 
	 * @param integer $count количество статей
	 * @param integer $cat_id категория
	 * @param boolean $often часто задаваемые вопросы
	 * @param string $sort сортировка date - по дате, rand - случайно
	 * @param integer $site_id страница сайта
	 * @return array
	 */
	public function show_block($count, $cat_id, $often, $sort, $site_id)
	{
		//кеширование
		$cache_meta = array(
				    "name"     => "block",
				    "cat_id"   => $cat_id,
				    "count"    => $count,
				    "language" => _LANG,
				    "site_id"  => $site_id,
				    "sort"     => $sort
				    );

		if($sort == "rand" || ! $this->result = $this->diafan->_cache->get($cache_meta, "faq"))
		{
			if(! $site_id)
			{
				$site_id = DB::query_result("SELECT id FROM {site} WHERE addmodule='faq' AND trash='0' AND act='1'");
			}
			if($cat_id && $this->diafan->configmodules("cat", "faq"))
			{
				if($this->diafan->configmodules("childrens_elements", "faq"))
				{ 
					$children = $this->diafan->get_children($cat_id, "faq_category");
				}
			}

			if($sort == "rand")
			{
				$max_count = DB::query_result("SELECT COUNT(DISTINCT e.id) FROM {faq} as e"
							      .($cat_id && $this->diafan->configmodules("cat", "faq")
							       ? " INNER JOIN {category} as c ON c.module_name='faq' AND c.element_id=e.id"
							         .(! empty($children) ? ' AND c.cat_id IN ('.implode(',', $children).')' : ' AND c.cat_id='.$cat_id)
							       : ''
							       )
							      ." WHERE e.[act]='1' AND e.trash='0'"
							      .($often ? " AND often='1'" : "")
							      .($site_id
								? " AND e.site_id=".$site_id
								: ''
								)
							      );
				$rands = array();
				for($i = 1; $i <= min($max_count, $count); $i++)
				{
					do
					{
						$rand = mt_rand(0, $max_count - 1);
					}
					while(in_array($rand, $rands));
					$rands[] = $rand;
				}
			}
			else
			{
				$rands[0] = 1;
			}
			$this->result["rows"] = array();

			foreach($rands as $rand)
			{
				$result = DB::query_range("SELECT e.id, e.[name], e.[anons], e.[text], e.cat_id, e.timeedit, e.site_id, e.created"
							  ." FROM {faq} AS e"
							  .($cat_id && $this->diafan->configmodules("cat", "faq")
							    ? " INNER JOIN {category} as c ON c.module_name='faq' AND c.element_id=e.id"
							      .(! empty($children) ? ' AND c.cat_id IN ('.implode(',', $children).')':' AND c.cat_id='.$cat_id)
							    : ''
							    )
							  ." WHERE e.[act]='1' AND e.trash='0'"
							  .($often ? " AND often='1'" : "")
							  .($site_id
							    ? " AND e.site_id=".$site_id
							    : ''
							    )
							  ." GROUP BY e.id"
							  .($sort == "date" ? ' ORDER BY e.created DESC, e.id DESC' : ''),
							  $sort == "rand" ? $rand : 0,
							  $sort == "rand" ? 1     : $count);

				$this->result["rows"] = array_merge($this->result["rows"], $this->get_elements($result));
			}

			$this->result["name"] = $often ? _LANG_FREGUENTLY_ASKED_QUESTIONS : '';
			if(! empty($this->result["rows"]) && $cat_id && $this->diafan->configmodules("cat", "faq"))
			{
				$cat = DB::fetch_array(DB::query("SELECT [name], site_id, id FROM {faq_category} WHERE id=%d", $cat_id));
				
				$this->result["name"] = $often ? $this->result["name"] : $cat["name"];
				$this->result["allfaq"]["link"] = $this->get_link_category($cat, 'faq');
				$this->result["allfaq"]["text"] = _LANG_ALL_QUESTIONS.' '._LANG_IN_CATEGORY.' '._LANG_LQUOT.$cat["name"]._LANG_RQUOT;
			}
			elseif(! empty($this->result["rows"]) && $site_id)
			{
				$this->result["name"] = $often ? $this->result["name"] : DB::query_result("SELECT [name] FROM {site} WHERE id=%d", $site_id);
				$this->result["allfaq"]["link"] = ! empty($this->diafan->rewrites[$site_id]["rewrite"]) ? $this->diafan->rewrites[$site_id]["rewrite"].'/' : '';
				$this->result["allfaq"]["text"] = _LANG_ALL_QUESTIONS;
			}
			//сохранение кеша
			if($sort != "rand")
			{
				$this->diafan->_cache->save($this->result, $cache_meta, "faq");
			}
		}

		return $this->result;
	}

	/**
	 * Форматирует данные о вопросам и ответах для списка
	 *
	 * @param resource $result результат выполнения SQL-запроса
	 * @return array
	 */
	private function get_elements($result)
	{
		if(empty($this->result["timeedit"]))
		{
			$this->result["timeedit"] = '';
		}
		$rows = array();
		while ($row = DB::fetch_array($result))
		{
			if($row["timeedit"] < $this->result["timeedit"])
			{
				$this->result["timeedit"] = $row["timeedit"];
			}
			unset($row["timeedit"]);

			if(! empty($this->diafan->rewrites["faq"]["element".$row["id"]]))
			{
				$row["link"] = $this->diafan->rewrites["faq"]["element".$row["id"]]["rewrite"].'/';
			}
			elseif($row["cat_id"] && $this->diafan->configmodules("cat", "faq", '', $row["site_id"]) && ! empty($this->diafan->rewrites["faq"]["cat".$row["cat_id"]]))
			{
				$row["link"] = $this->diafan->rewrites["faq"]["cat".$row["cat_id"]]["rewrite"].'/show'.$row["id"].'/';
			}
			else
			{
				$row["link"] = (! empty($this->diafan->rewrites[$row["site_id"]]["rewrite"]) ? $this->diafan->rewrites[$row["site_id"]]["rewrite"].'/' : '')
				               .($row["cat_id"] && $this->diafan->configmodules("cat", "faq", '', $row["site_id"])? 'cat'.$row["cat_id"].'/' : '')
					       .'show'.$row["id"].'/';
			}
			unset($row["cat_id"]);

			$row["anons"] = $this->diafan->short_text($row["anons"], 160);
			$row["text"]  = $this->diafan->short_text($row["text"], 160);
			$row["date"] = $this->format_date($row['created'], "faq", $row["site_id"]);
			unset($row["created"]);

			$rows[] = $row;	
		}
		return $rows;
	}
}