<?php
/**
 * Diafan.CMS
 *
 * @author     diafan.ru
 * @version    4.3
 * @license http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2011 OOO "Диафан". (http://diafan.ru)
 */

if(! defined('DIAFAN'))
{
	include(dirname(dirname(dirname(__FILE__))).'/includes/404.php');
}

/**
 * Feedback_ajax
 *
 * Обработка запроса при отправке сообщения из формы обратной связи
 */
class Feedback_ajax
{
	/**
	 * @var object основной объект системы
	 */
	public $diafan;

	/**
	 * @var array внутренние сообщения
	 */
	private $result;

	/**
	 * @var array поля формы
	 */
	private $param;

	/**
	 * Конструктор класса
	 * 
	 * @return void
	 */
	public function __construct($diafan)
	{
		$this->diafan = $diafan;
	}

	/**
	 * Обрабатывает полученные данные из формы
	 * 
	 * @return boolean
	 */
	public function ajax_request()
	{
		if(! empty($_POST['ajax_feedback']) && ($this->diafan->configmodules('security') != 1 || $this->diafan->_user->id))
		{
			$this->param = $this->get_param();

			$this->check_captcha();
			$this->empty_required_field();

			if($this->send_errors())
				return TRUE;

			if(! DB::query("INSERT INTO {feedback} (created, site_id, language) VALUES ('%d','%d','%d')",
				       time(), $this->diafan->cid, _LANG))
			{
				$mes = $this->diafan->configmodules('error-add-message', 'feedback');
				$this->result["errors"][0] = $mes ? $mes : ' ';
				return $this->send_errors();
			}
			$save = DB::next_id("feedback");

			$this->result["altnames"] = '';
			$err = '';
			if (! empty($_FILES['attachment']) && $this->diafan->configmodules('files', 'feedback'))
			{
				include_once (ABSOLUTE_PATH."includes/files.inc.php");
				$files = new Files($this->diafan);
				foreach ($_FILES['attachment']['tmp_name'] as $n => $dummy)
				{
					if($this->diafan->configmodules("max-count-files", 'feedback') && $n > $this->diafan->configmodules("max-count-files", 'feedback'))
						break;

					$err .= $files->file_upload($_FILES['attachment'], $this->diafan->module, $save, $n);
					$this->result["altnames"] .= " ".$_FILES['attachment']['name'][$n];
				}
			}

			if(! empty($err))
			{
				DB::query("DELETE FROM {feedback} WHERE id='%d'", $save);
				DB::query("DELETE FROM {attachment} WHERE module_name='%h' AND element_id='%d'", $this->diafan->module, $save);
				$this->result["errors"]["attachments"] = $err;
				return $this->send_errors();
			}

			$this->insert_values($save);
			$this->send_mail();

			$mes = $this->diafan->configmodules('add-message', 'feedback');
			$this->result["errors"][0] = $mes ? $mes : ' ';
			$this->result["success"] = TRUE;
			return $this->send_errors();
		}
		return FALSE;
	}

	/**
	 * Проверяет правильность каптчи
	 * 
	 * @return boolean
	 */
	private function check_captcha()
	{
		$this->result["code"] = '';
		if($this->diafan->configmodules('security', 'feedback') == 2)
		{
			require_once(ABSOLUTE_PATH.'includes/captcha/captcha.inc.php');	
			$mes = Captcha_inc::is_right($this->diafan->module);
			$this->result["code"] = Captcha_inc::get('feedback', '', TRUE);				
			if($mes)
			{
				$this->result["errors"]["code"] = $mes;
				return TRUE;
			}
		}
		return FALSE;
	}

	/**
	 * Получает массив полей формы обратной связи
	 * 
	 * @return array
	 */
	private function get_param()
	{

		$cache_meta = array(
				    "name"     => "form_param",
				    "language" => _LANG,
				    "site_id"  => $this->diafan->cid
				    );
		if(! $rows_param = $this->diafan->_cache->get($cache_meta, "feedback/param"))
		{
			$rows_param = array();
			$result = DB::query("SELECT id, [name], type, required FROM {param} WHERE site_id IN(0, %d)"
					    ." AND trash='0' AND module_name='feedback' ORDER BY sort ASC", $this->diafan->cid);

			while($row = DB::fetch_array($result))
			{
				if($row["type"] == 'select' || $row["type"] == 'multiple')
				{
					$result_select = DB::query("SELECT [name], value FROM {param_select} WHERE param_id=%d"
							    ." ORDER BY value ASC", $row["id"]);
					while ($row_select = DB::fetch_array($result_select))
					{
						$row["select_array"][$row_select["value"]] = $row_select["name"];
					}
				}
				$rows_param[] = $row;
			}
			//сохранение кеша
			$this->diafan->_cache->save($rows_param, $cache_meta, "feedback/param");
		}
		return $rows_param;
	}

	/**
	 * Проверяет на заполнение обязательных полей
	 * 
	 * @return boolean
	 */
	private function empty_required_field()
	{
		$message = array();
		foreach($this->param as $row)
		{
			if($row["required"] && empty($_POST['p'.$row["id"]])
			    && ($row["type"] != "date" || empty($_POST["p".$row["id"]."d"]) || empty($_POST["p".$row["id"]."m"]) || empty($_POST["p".$row["id"]."y"])
				|| $_POST["p".$row["id"]."d"] == "dd" || $_POST["p".$row["id"]."m"] == "mm" || $_POST["p".$row["id"]."y"] == "yyyy")
			)
			{
				$this->result["errors"]["p".$row["id"]] = $this->diafan->configmodules('error-empty-fields').' '.$row["name"];
			}
			if(empty($_POST["p".$row["id"]]) && $row["type"] != "checkbox"
			   && ($row["type"] != "date" || empty($_POST["p".$row["id"]."d"]) || empty($_POST["p".$row["id"]."m"]) || empty($_POST["p".$row["id"]."y"]))
			)
				continue;
			
			if($row["type"] == "email")
			{
				$this->valid_email($_POST["p".$row["id"]], "p".$row["id"]);
			}
			
			if($row["type"] == "text" || $row["type"] == "textarea" || $row["type"] == "email")
			{
				$_POST["p".$row["id"]] = $this->diafan->get_param($_POST, "p".$row["id"], '', 1);
				$message[] = $row["name"].': '.$_POST["p".$row["id"]];
			}
			elseif($row["type"] == "numtext")
			{
				$_POST["p".$row["id"]] = $this->diafan->get_param($_POST, "p".$row["id"], '', 2);
				$message[] = $row["name"].': '.$_POST["p".$row["id"]];
			}
			elseif($row["type"] == "date")
			{
				if($_POST["p".$row["id"]."d"] == "dd" || $_POST["p".$row["id"]."m"] == "mm" || $_POST["p".$row["id"]."y"] == "yyyy")
					continue;
				$_POST["p".$row["id"]."d"] = $this->diafan->get_param($_POST, "p".$row["id"]."d", '', 2);
				$_POST["p".$row["id"]."m"] = $this->diafan->get_param($_POST, "p".$row["id"]."m", '', 2);
				$_POST["p".$row["id"]."y"] = $this->diafan->get_param($_POST, "p".$row["id"]."y", '', 2);
				$message[] = $row["name"].': '.$_POST["p".$row["id"]."d"].'.'.$_POST["p".$row["id"]."m"].'.'.$_POST["p".$row["id"]."y"];
				
				$_POST["p".$row["id"]] = mktime(0, 0, 0, $_POST["p".$row["id"]."m"], $_POST["p".$row["id"]."d"], $_POST["p".$row["id"]."y"]);
			}
			elseif($row["type"] == "checkbox")
			{
				$value = ! empty($_POST["p".$row["id"]]) ? 1 : 0;
				$value_sel = DB::query_result("SELECT [name] FROM {param_select} WHERE value=%d AND param_id=%d", $value, $row["id"]);
				if(! $value_sel && $value == 1)
				{
					$message[] = $row["name"];
				}
				else
				{
					$message[] = $row["name"].': '.$value_sel;
				}
			}
			elseif($row["type"] == "select")
			{
				if(! empty($row["select_array"][$_POST["p".$row["id"]]]))
				{
					$_POST["p".$row["id"]] = $this->diafan->get_param($_POST, "p".$row["id"], '', 1);
					$message[] = $row["name"].': '.$row["select_array"][$_POST["p".$row["id"]]];
				}
			}
			elseif($row["type"] == "multiple")
			{
				if(! empty($row["select_array"]))
				{
					if(! empty($_POST["p".$row["id"]]) && is_array($_POST["p".$row["id"]]))
					{
						$vals = array();
						foreach($_POST["p".$row["id"]] as $val)
						{
							$vals[] = $row["select_array"][$val];
						}
						$message[] = $row["name"].': '.implode(", ", $vals);
					}
				}
			}
		}
		$this->result["message"] = implode('<br>', $message);
		return FALSE;
	}

	/**
	 * Проверка e-mail на валидность
	 * 
	 * @return boolean
	 */
	private function valid_email($email, $filed)
	{
		if($email)
		{
			require_once(ABSOLUTE_PATH.'includes/validate.php');
			$mes = Validate::mail($email);
			if($mes)
			{
				$this->result["errors"][$filed] = $mes;
				return TRUE;
			}
		}
		return FALSE;
	}

	/**
	 * Добавляет значение полей формы в базу данных
	 * 
	 * @return boolean true
	 */
	private function insert_values($save)
	{
		foreach($this->param as $row)
		{
			if(empty($_POST["p".$row["id"]]))
				continue;

			if($row["type"] == "multiple")
			{
				foreach($_POST["p".$row["id"]] as $val)
				{
					DB::query("INSERT INTO {param_element} (value,param_id,element_id,module_name) VALUES ('%h','%d','%d','%s')",
						  $val, $row["id"], $save, $this->diafan->module);
				}
			}
			else
			{
				DB::query("INSERT INTO {param_element} (value,param_id,element_id,module_name) VALUES ('%h','%d','%d','%s')",
				  $_POST["p".$row["id"]], $row["id"], $save, $this->diafan->module);
			}
		}
		return TRUE;
	}

	/**
	 * Уведомление администратора по e-mail
	 * 
	 * @return boolean
	 */
	private function send_mail()
	{
		if(! $this->diafan->configmodules("sendmailadmin"))
			return FALSE;

		$subject = str_replace(
			array('%title', '%url'),
			array(TITLE, BASE_URL),
			$this->diafan->configmodules("subject-admin")
		);

		$message = str_replace(
			array('%title', '%url', '%message', '%files'),
			array(
				TITLE,
				BASE_URL,
				$this->result["message"],
				(! empty($this->result["altnames"]) ? $this->result["altnames"] : '-')
			),
			$this->diafan->configmodules("message-admin")
		);

		$to   = $this->diafan->configmodules("emailconfadmin") ? $this->diafan->configmodules("email-admin") : EMAIL_CONFIG;
		$from = $this->diafan->configmodules("emailconf") ? $this->diafan->configmodules("email") : '';

		//$this->result["errors"]["attachments"] = $subject.'<br><br>'.$message;
		include_once(ABSOLUTE_PATH.'includes/mail.php');
		send_mail($to, $subject, $message, $from);

		return TRUE;
	}

	/**
	 * Отправляет несколько ошибок
	 *
	 * @return boolean
	 */
	private function send_errors()
	{
		if(! empty($this->result["errors"]))
		{
			if(! empty($_POST['ajax']))
			{
				include_once(ABSOLUTE_PATH.'plugins/json.php');
				echo to_json($this->result);
			}
			else
			{
				$query = '';
				if(! empty($this->result["errors"]))
				{
					foreach($this->result["errors"] as $field => $mes)
					{
						if(empty($_POST["update"]))
						{
							$query .= '&mess'.($field ? '-'.$field : '').'='.$mes;
						}
					}
				}
				$this->diafan->goto_diafan($this->diafan->path_module_variable().($query ? '?error=feedback'.$query : ''));
			}
			return TRUE;
		}
		return FALSE;
	}
}