<?php
/**
 * Diafan.CMS
 *
 * @author     diafan.ru
 * @version    4.3
 * @license http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2011 OOO "Диафан". (http://diafan.ru)
 */

if(! defined('DIAFAN'))
{
	include(dirname(dirname(dirname(__FILE__))).'/includes/404.php');
}

/**
 * Feedback_model
 *
 * Модель модуля "Обратная связь"
 */
class Feedback_model extends Model
{
	/**
	 * @var array сгенерированные в моделе данные, передаваемые в шаблон
	 */
	protected $result;

	/**
	 * @var object основной объект системы
	 */
	protected $diafan;

	/**
	 * Конструктор класса
	 * 
	 * @return void
	 */
	public function __construct($diafan)
	{
		$this->diafan = $diafan;
	}

	/**
	 * Генерирует данные для формы добавления сообщения
	 * 
	 * @param integer $site_id номер страницы
	 * @param boolean $insert_form форма выводится с помощью шаблонного тэга
	 * @return array|boolean false
	 */
	public function form($site_id = 0, $insert_form = FALSE)
	{
		if(! $insert_form)
		{
			$site_id = $this->diafan->cid;
		}
		else
		{
			if(! $site_id)
			{
				$site_id = DB::query_result("SELECT id FROM {site} WHERE addmodule='feedback' LIMIT 1");
			}
		}
		if(! $site_id)
		{
			return FALSE;
		}

		if($this->diafan->configmodules('security', "feedback", _LANG, $site_id) != 1 || $this->diafan->_user->id)
		{
			$this->result["files"] = 0;
			$this->result["captcha"] = '';
			if($this->diafan->configmodules('security', "feedback", _LANG, $site_id) == 2)
			{
				require_once (ABSOLUTE_PATH.'includes/captcha/captcha.inc.php');
				$this->result["captcha"] = Captcha_inc::get("feedback", $this->get_error("feedback", "code"));
			}

			if($this->diafan->configmodules('files', "feedback", _LANG, $site_id) && $this->diafan->configmodules("max-count-files", "feedback", _LANG, $site_id))
			{
				$this->result["files"] = $this->diafan->configmodules("max-count-files", "feedback", _LANG, $site_id);
				$this->result["error-attachments"] = $this->get_error("feedback", 'attachments');
			}
		}
		else
		{
			$this->result["text"] = _LANG_NECESSARY_REGISTRATION;
		}

		$cache_meta = array(
				    "name"     => "form_param",
				    "language" => _LANG,
				    "site_id"  => $site_id
				    );
		if(! $rows_param = $this->diafan->_cache->get($cache_meta, "feedback/param"))
		{
			$rows_param = array();
			$result = DB::query("SELECT id, [name], type, required FROM {param} WHERE site_id IN(0, %d)"
					    ." AND trash='0' AND module_name='feedback' ORDER BY sort ASC", $site_id);

			while($row = DB::fetch_array($result))
			{
				if($row["type"] == 'select' || $row["type"] == 'multiple')
				{
					$result_select = DB::query("SELECT [name], value FROM {param_select} WHERE param_id=%d"
								   ." ORDER BY value ASC", $row["id"]);
					while($row_select = DB::fetch_array($result_select))
					{
						$row["select_array"][$row_select["value"]] = $row_select["name"];
					}
				}
				$rows_param[] = $row;
			}
			//сохранение кеша
			$this->diafan->_cache->save($rows_param, $cache_meta, "feedback/param");
		}
		foreach($rows_param as $row)
		{
			$this->result['error-p'.$row["id"]] = $this->get_error("feedback", 'p'.$row["id"]);
		}

		$this->result["error"]  = $this->get_error("feedback");
		$this->result["rows"]   = $rows_param;
		$this->result["action"] = '';
		if(! empty($this->diafan->rewrites[$site_id]))
		{
			$this->result["action"] = $this->diafan->rewrites[$site_id]["rewrite"] ? $this->diafan->rewrites[$site_id]["rewrite"].'/' : '';
		}
		return $this->result;
	}
}