<?php
/**
 * Diafan.CMS
 *
 * @author     diafan.ru
 * @version    4.3
 * @license http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2011 OOO "Диафан". (http://diafan.ru)
 */

if(! defined('DIAFAN'))
{
	include(dirname(dirname(dirname(__FILE__))).'/includes/404.php');
}

/**
 * Forum_ajax
 *
 * Обработка запроса
 */
class Forum_ajax
{
	/**
	 * @var object основной объект системы
	 */
	protected $diafan;

	/**
	 * @var array внутренние сообщения
	 */
	private $result;

	/**
	 * @var object работа с сообщениями
	 */
	protected $message;

	/**
	 * Конструктор класса
	 * 
	 * @return void
	 */
	public function __construct($diafan)
	{
		$this->diafan = $diafan;

		include_once(ABSOLUTE_PATH.'modules/forum/message/message.inc.php');
		$this->message = new Forum_message_inc($diafan);
	}

	/**
	 * Обрабатывает запрос
	 * 
	 * @return boolean
	 */
	public function ajax_request()
	{
		if(empty($_POST["ajax_forum"]) || empty($_POST["action"]))
		{
			return FALSE;
		}
		switch($_POST["action"])
		{
			case "save-category":
				return $this->save();

			case "savenew-category":
				return $this->savenew();

			case "delete-category":
				return $this->delete();

			case "block-category":
				return $this->block(TRUE);

			case "unblock-category":
				return $this->block(FALSE);

			case "close-category":
				return $this->close(TRUE);

			case "open-category":
				return $this->close(FALSE);

			case "prior-category":
				return $this->prior(TRUE);

			case "unprior-category":
				return $this->prior(FALSE);
		}
		return FALSE;
	}

	/**
	 * Сохраняет новую категорию форума
	 * 
	 * @return boolean
	 */
	public function savenew()
	{
		//доступ на добавление только для зарегистрированных
		if($this->diafan->configmodules("access_add") == 2 && ! $this->diafan->_user->id)
		{
			$this->result["errors"][0] = 'ERROR';
			return $this->send_errors();
		}
		//доступ на добавление по каптче
		elseif($this->diafan->configmodules("access_add") == 1 && ! $this->diafan->_user->id)
		{
			$this->result["code"] = '';
			require_once(ABSOLUTE_PATH.'includes/captcha/captcha.inc.php');	
			$mes = Captcha_inc::is_right($this->diafan->module);
			$this->result["code"] = Captcha_inc::get('forum', '', TRUE);				
			if($mes)
			{
				$this->result["errors"]["code"] = $mes;
				return $this->send_errors();
			}
		}
		if(empty($_POST["parent_id"]) || ! DB::query_result("SELECT parent_id FROM {forum_category} WHERE id=%d AND trash='0' AND act='1'", $_POST["parent_id"]))
		{
			$this->result["errors"][0] = 'ERROR';
			return $this->send_errors();	
		}
		if(empty($_POST["name"]))
		{
			$this->result["errors"][0] = _LANG_EMPTY_THEME_NAME;
			return $this->send_errors();
		}

		DB::query("INSERT INTO {forum_category} (name, author, created, message_update, act, parent_id) "
		          ." VALUES ('%h', '%d', '%d', '%d', '%d', '%d')", $_POST["name"], $this->diafan->_user->id,
		          time(), time(), $this->diafan->configmodules('premoderation_theme') && ! $this->diafan->_user->moderator ? 0 : 1,
		          $_POST["parent_id"]
		);
		$save = DB::next_id("forum_category");

		$this->save_news($save);
		if($this->diafan->configmodules('premoderation_theme') && ! $this->diafan->_user->moderator)
		{
			$this->result["success"] = 1;
			$this->result["errors"][0] = _LANG_THEME_ADDED.' '._LANG_THEME_ADDED_MODERATOR;
		}
		else
		{
			
			$this->result["redirect"] = URL.'show'.$save.'/';
		}
		return $this->send_errors();
	}

	/**
	 * Сохраняет редактируемую категорию форума
	 * 
	 * @return boolean
	 */
	public function save()
	{
		$row = DB::fetch_array(DB::query("SELECT id, author, parent_id, act FROM {forum_category} WHERE id = %d AND del='0' AND trash='0'", $_POST["id"]));

		//редактирует только автор и модератор
		if(! $row || ! $this->diafan->_user->moderator && ($row["author"] != $this->diafan->_user->id || ! $row["act"]))
		{
			$this->result["errors"][0] = 'ERROR';
			return $this->send_errors();
		}

		if(empty($row["parent_id"]) || ! DB::query_result("SELECT parent_id FROM {forum_category} WHERE id=%d AND trash='0' AND del='0' AND act='1'", $row["parent_id"]))
		{
			$this->result["errors"][0] = 'ERROR';
			return $this->send_errors();	
		}

		$edit_or_add = 2;
		if(DB::query_result("SELECT id FROM {forum_category} WHERE id='%d' AND name='%h'", $_POST["id"], $_POST["name"]))
		{
			$edit_or_add = 0;
		}

		if($edit_or_add)
		{
			DB::query("UPDATE {forum_category} SET name='%h', date_update='%d', message_update='%d', user_update='%d', act='%d' WHERE id=%d",
				  $_POST["name"], time(), time(),
				  //модератор
				  $row["author"] != $this->diafan->_user->id ? $this->diafan->_user->id : '',
				  $this->diafan->configmodules('premoderation_theme') && ! $this->diafan->_user->moderator ? 0 : 1,
				  $_POST["id"]
			);
			$this->save_news($_POST["id"]);
		}

		if($this->diafan->configmodules('premoderation_theme') && ! $this->diafan->_user->moderator)
		{
			$this->result["errors"][0] = _LANG_THEME_EDITED.' '._LANG_THEME_ADDED_MODERATOR;
		}
		else
		{
			
			$this->result["redirect"] = URL.'show'.$_POST["id"].'/';
		}
		return $this->send_errors();
	}

	/**
	 * Удаляет категорию форума
	 * 
	 * @return boolean
	 */
	public function delete()
	{
		$row = DB::fetch_array(DB::query("SELECT * FROM {forum_category}  WHERE id = %d AND del='0' AND trash='0'", $_POST["id"]));

		//удаляет только автор и модератор
		if(! $row || ($row["author"] != $this->diafan->_user->id && ! $this->diafan->_user->moderator))
		{
			exit;
		}

		//удаляет автор, если в теме нет сообщений от других пользователей
		if(! $this->diafan->_user->moderator
		   && DB::result(DB::query("SELECT COUNT(*) FROM {forum} WHERE cat_id=%d AND author<>%d", $this->diafan->show, $this->diafan->_user->id)))
		{
			exit;
		}

		//удаляют все сообщения в теме
		$this->message->delete();

		//модератор только помечает, что тема удалена
		if($row["author"] != $this->diafan->_user->id)
		{
			DB::query("UPDATE {forum_category} SET del='1', user_update='%d', date_update=%d WHERE id='%d'",
				  $this->diafan->_user->id, time(), $_POST["id"]);
			$this->save_news($_POST["id"]);
			$row["del"] = '1';
			$row["user_update"] = $this->diafan->_user->id;
			$row["date_update"] = time();
		}
		else
		{
			DB::query("DELETE FROM {forum_category} WHERE id='%d'", $_POST["id"]);
			DB::query("DELETE FROM {forum_show} WHERE element_id='%d' AND table_name='forum_category'", $_POST["id"]);
		}
		$this->show_category($row);
		return $this->send_errors();
	}

	/**
	 * Публикует/блокирует категорию форума
	 *
	 * @param boolean $block категория блокируется
	 * @return boolean
	 */
	public function block($block)
	{
		$row = DB::fetch_array(DB::query("SELECT * FROM {forum_category}  WHERE id = %d AND del='0' AND trash='0'", $_POST["id"]));

		//блокировать/разблокировать может только модератор
		if(! $row || ! $this->diafan->_user->moderator)
		{
			include(ABSOLUTE_PATH.'includes/404.php');
		}
		if($block)
		{
			DB::query("DELETE FROM {forum_show} WHERE element_id='%d' AND table_name='forum_category'", $_POST["id"]);
		}
		else
		{
			$this->save_news($_POST["id"]);
		}

		DB::query("UPDATE {forum_category} SET act='%d' WHERE id='%d'", $block ? 0 : 1, $_POST["id"]);
		$row["act"] = $block ? 0 : 1;

		$this->show_category($row);
		return $this->send_errors();
	}

	/**
	 * Закрывает категорию форума
	 * 
	 * @param boolean $close категория закрывается
	 * @return boolean
	 */
	public function close($close)
	{
		$row = DB::fetch_array(DB::query("SELECT * FROM {forum_category}  WHERE id = %d AND del='0' AND trash='0'", $_POST["id"]));

		//закрывать тему может только модератор
		if(! $row || ! $this->diafan->_user->moderator)
		{
			include(ABSOLUTE_PATH.'includes/404.php');
		}

		DB::query("UPDATE {forum_category} SET close='%d' WHERE id='%d'", $close ? 1 : 0, $_POST["id"]);
		$row["close"] = $close ? 1 : 0;

		$this->show_category($row);
		return $this->send_errors();
	}

	/**
	 * Закрепляет/открепляет категорию форума
	 * 
	 * @param boolean $prior категория закрепляется
	 * @return boolean
	 */
	public function prior($prior)
	{
		$row = DB::fetch_array(DB::query("SELECT id FROM {forum_category}  WHERE id = %d AND del='0' AND trash='0'", $_POST["id"]));

		//закреплять тему может только модератор
		if(! $row || ! $this->diafan->_user->moderator)
		{
			include(ABSOLUTE_PATH.'includes/404.php');
		}

		DB::query("UPDATE {forum_category} SET prior='%d' WHERE id='%d'", $prior ? 1 : 0, $_POST["id"]);
		$this->result["redirect"] = URL_LIST.'cat'.$this->diafan->cat.'/';
		
		return $this->send_errors();
	}

	/**
	 * Сохраняет историю изменений для пользователей
	 * 
	 * @return boolean true
	 */
	private function save_news($save)
	{
		$result = DB::query("SELECT id FROM {users_site} WHERE act='1' AND trash='0' AND id<>'%d'", $this->diafan->_user->id);
		while($row = DB::fetch_array($result))
		{
			DB::query("INSERT INTO {forum_show} (element_id, user_id, table_name) VALUES ('%d', '%d', 'forum_category')", $save, $row["id"]);
		}
		return TRUE;
	}

	/**
	 * Отправляет несколько ошибок
	 *
	 * @return boolean
	 */
	private function send_errors()
	{
		if(! empty($this->result["errors"]) || ! empty($this->result["redirect"]) || ! empty($this->result["data"]))
		{
			if(! empty($_POST['ajax']))
			{
				include_once(ABSOLUTE_PATH.'plugins/json.php');
				echo to_json($this->result);
			}
			else
			{
				$query = '';
				foreach($this->result["errors"] as $field => $mes)
				{
					if(empty($_POST["update"]))
					{
						$query .= '&mess'.($field ? '-'.$field : '').'='.$mes;
					}
				}
				$this->diafan->goto_diafan($this->diafan->path_module_variable().($query ? '?error=forum'.$query : ''));
			}
			return TRUE;
		}
		return FALSE;
	}

	/**
	 * Формирует возвращаемую строку о категории
	 *
	 * @return boolean
	 */
	private function show_category($row = array())
	{
		if(! $row)
		{
			$row = DB::fetch_array(DB::query("SELECT * FROM {forum_category}  WHERE id = %d AND del='0' AND trash='0' AND act='1'", $_POST["id"]));
		}
		$model = new Forum_model($this->diafan);
		$model_result = $model->list_id_category($row);
		$this->result["data"] = Forum_view::list_id_category($model_result);
		$this->result["target"] = '#forum-category-'.$row["id"];

		return TRUE;
	}
}