<?php
/**
 * Diafan.CMS
 *
 * @author     diafan.ru
 * @version    4.3
 * @license http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2011 OOO "Диафан". (http://diafan.ru)
 */

if(! defined('DIAFAN'))
{
	include(dirname(dirname(dirname(dirname(__FILE__)))).'/includes/404.php');
}

/**
 * Forum_message_ajax
 *
 * Обработка Ajax-запроса
 */
class Forum_message_ajax extends Model
{
	/**
	 * @var object основной объект системы
	 */
	public $diafan;

	/**
	 * @var array внутренние сообщения
	 */
	private $result;

	/**
	 * @var array
	 */
	private $config;

	/**
	 * @var object работа с файлами
	 */
	private $attachments;

	/**
	 * Конструктор класса
	 * 
	 * @return void
	 */
	public function __construct($diafan)
	{		
		$this->diafan = $diafan;

		$this->attachments = new Attachments_inc($diafan);
	}

	/**
	 * Обрабатывает полученные данные из формы
	 * 
	 * @return boolean
	 */
	public function ajax_request()
	{
		if(empty($_POST["ajax_forum_message"]) || ! $this->diafan->_user->id && $this->diafan->configmodules("access_add", "forum") == 2)
		{
			return FALSE;
		}
		if(! empty($_POST["action"]))
		{
			if($_POST["action"] == "upload-message")
			{
				return $this->upload_message();
			}
			elseif($_POST["action"] == "save-message")
			{
				return $this->save_message();
			}
			elseif($_POST["action"] == "delete-message")
			{
				return $this->delete_message();
			}
			elseif($_POST["action"] == "edit-message")
			{
				return $this->edit_message();
			}
			elseif($_POST["action"] == "block-message")
			{
				return $this->block_message();
			}
		}
		return FALSE;
	}

	/**
	 * Добавляет новое сообщение
	 *
	 * @return boolean
	 */
	private function upload_message()
	{
		$element_id = $this->diafan->show;

		//доступ на добавление только для зарегистрированных
		if($this->diafan->configmodules("access_add", "forum") == 2 && ! $this->diafan->_user->id)
		{
			$this->result["errors"][0] = _LANG_FORUM_ADD_MESSAGE_ONLY_REGISTRED;
			return $this->send_errors();
		}
		if($this->check_captcha())
			return TRUE;

		if(! DB::query_result("SELECT id FROM {forum_category} WHERE id=%d AND act='1' AND trash='0'", $this->diafan->show))
		{
			$this->result["errors"][0] = 'ERROR';
			return $this->send_errors();
		}
		//тема закрыта
		if(DB::query_result("SELECT close FROM {forum_category} WHERE id=%d", $this->diafan->show))
		{
			$this->result["errors"][0] = _LANG_FORUM_THEME_CLOSE;
			return $this->send_errors();
		}
		if(! $_POST["message"])
		{
			$this->result["errors"][0] = _LANG_EMPTY_MESSAGE;
			return $this->send_errors();
		}
		if(! empty($_POST["parent_id"]) && ! DB::query_result("SELECT id FROM {forum} WHERE id=%d AND trash='0' AND act='1'", $_POST["parent_id"]))
		{
			$this->result["errors"][0] = 'ERROR';
			return $this->send_errors();	
		}
		//максимальное количество уровней вложенности - 7
		$parents = $this->diafan->get_parents($_POST["parent_id"], "forum");
		if(count($parents) > 6)
		{
			$this->result["errors"][0] = 'ERROR';
			return $this->send_errors();	
		}
		$result["level"] = count($parents);

		$message = $this->replace_bbcode($_POST["message"]);
		if(! $_POST["message"])
		{
			$this->result["errors"][0] = _LANG_EMPTY_MESSAGE;
			return $this->send_errors();
		}

		if(DB::query("INSERT INTO {forum} (created, cat_id, author, text, parent_id, act) VALUES ('%d', '%d', '%d', '%s', '%d', '%d')",
		             time(),
		             $element_id,
		             $this->diafan->_user->id,
		             $message,
		             $_POST["parent_id"],
		             $this->diafan->configmodules('premoderation_message', "forum") && ! $this->diafan->_user->moderator ? 0 : 1
		            )
		   )
		{
			$row["id"] = DB::next_id("forum");
			if($this->diafan->configmodules("files", "forum"))
			{
				$edit = $this->attachments->save($row["id"], 'forum');
				if($edit !== 0 && $edit !== 2)
				{
					$this->result["errors"]['attachments'] = $edit;
					return $this->send_errors();
				}
			}
			$this->save_news($row["id"], $element_id);
			DB::query("UPDATE {forum_category} SET message_update='%d' WHERE id=%d", time(), $element_id);

			$row['created']            = _LANG_ADDED;
			$row["author"]             = $this->get_author($this->diafan->_user->id);
			$row["text"]               = $message;
			$row["access_edit_delete"] = $this->diafan->_user->id ? TRUE : FALSE;
			$row["access_block"]       = $this->diafan->_user->moderator ? TRUE : FALSE;
			$row["children"]           = 0;
			$row["show"]             = 0;
			$row["act"]              = $this->diafan->configmodules('premoderation_message', "forum") && ! $this->diafan->_user->moderator ? 0 : 1;
			$row["del"]              = 0;

			$row["attachments"] = array();
			$result["files"] = 0;
			if($this->diafan->configmodules("files", "forum"))
			{
				$result["files"]                 = $this->diafan->configmodules("max-count-files",       "forum");
				$result["attachment-extensions"] = $this->diafan->configmodules("attachment-extensions", "forum");
				$row["attachments"]              = $this->attachments->get($row["id"], 'forum', $this->diafan->_user->id ? 1 : '');
			}

			$result["rows"][]        = $row;

			$result["action"]        = URL.'show'.$element_id.'/';
			$result["access_add"]    = TRUE;
			$result["captcha"]       = '';
			$result["premoderation"] = $this->diafan->configmodules('premoderation_message', "forum") && ! $this->diafan->_user->moderator;

			//доступ на добавление по каптче
			if($this->diafan->configmodules("access_add") == 1 && ! $this->diafan->_user->id)
			{
				require_once (ABSOLUTE_PATH.'includes/captcha/captcha.inc.php');
				$this->result["captcha"] = Captcha_inc::get("messages");
			}
			$result["current_level"]   = intval($_POST["parent_id"]);
			$this->result["form_hide"] = 1;
			$this->result["add"]       = Forum_message_view::show_list($result);
			$this->result["success"]   = 1;
			return $this->send_errors();
		}
		return FALSE;
	}

	/**
	 * Редактирует сообщение
	 *
	 * @return boolean
	 */
	private function edit_message()
	{
		//тема закрыта
		if(DB::query_result("SELECT close FROM {forum_category} WHERE id=%d", $this->diafan->show))
		{
			$this->result["errors"][0] = _LANG_FORUM_THEME_CLOSE;
			return $this->send_errors();
		}

		$element_id = $this->diafan->show;
		if(! $this->diafan->_user->id)
		{
			$this->result["errors"][0] = 'ERROR';
			return $this->send_errors();
		}

		$row = DB::fetch_array(DB::query("SELECT author, text FROM {forum} WHERE cat_id=%d AND id=%d",
						 $element_id, $_POST["id"]));
		if(! $row)
		{
			$this->result["errors"][0] = 'ERROR';
			return $this->send_errors();
		}
		if($this->diafan->_user->id != $row["author"] && ! $this->diafan->_user->moderator)
		{
			$this->result["errors"][0] = 'ERROR';
			return $this->send_errors();
		}

		$edit_id               = $this->diafan->get_param($_POST, "id", '', 2);
		$result["attachments"] = array();
		$result["files"]       = 0;
		if($this->diafan->configmodules("files", "forum"))
		{
			$result["files"]                 = $this->diafan->configmodules("max-count-files",       "forum");
			$result["attachment-extensions"] = $this->diafan->configmodules("attachment-extensions", "forum");
			$result["attachments"]           = $this->attachments->get($edit_id, 'forum', 1);
		}

		$result["id"]            = $edit_id;
		$result["text"]          = $this->add_bbcode($row["text"]);
		$result["premoderation"] = $this->diafan->configmodules('premoderation_message', "forum") && ! $this->diafan->_user->moderator;
		$result["access_add"]    = TRUE;
		$result["action"]        = URL.'show'.$element_id.'/';

		$this->result["data"]    = Forum_message_view::show_edit_module($result);
		$this->result["target"]  = "#show_message".$_POST["id"];
		return $this->send_errors();
	}

	/**
	 * Сохраняет редактируемое сообщение
	 *
	 * @return boolean
	 */
	private function save_message()
	{
		//тема закрыта
		if(DB::query_result("SELECT close FROM {forum_category} WHERE id=%d", $this->diafan->show))
		{
			$this->result["errors"][0] = _LANG_FORUM_THEME_CLOSE;
			return $this->send_errors();
		}

		if(! $this->diafan->_user->id)
		{
			$this->result["errors"][0] = 'ERROR';
			return $this->send_errors();
		}
		$element_id = $this->diafan->show;

		$row = DB::fetch_array(DB::query("SELECT * FROM {forum} WHERE cat_id=%d AND id=%d",
						 $element_id, $_POST["save_id"]));
		if(! $row)
		{
			$this->result["errors"][0] = 'ERROR';
			return $this->send_errors();
		}
		if($this->diafan->_user->id != $row["author"] && ! $this->diafan->_user->moderator)
		{
			$this->result["errors"][0] = 'ERROR';
			return $this->send_errors();
		}

		$edit = 0;
		if($this->diafan->configmodules("files", "forum"))
		{
			$edit = $this->attachments->save($row["id"], 'forum');
			if($edit !== 0 && $edit !== 2)
			{
				$this->result["errors"]['attachments'] = $edit;
				return $this->send_errors();
			}
		}

		if(! $_POST["message"])
		{
			$this->result["errors"][0] = _LANG_EMPTY_MESSAGE;
			return $this->send_errors();
		}
		$message = $this->replace_bbcode($_POST["message"]);

		if($edit || $row["text"] != $message)
		{
			DB::query("UPDATE {forum} SET text='%s',date_update='%d',user_update='%d',act='%d' WHERE id='%d'",
				  $message,
				  time(),
				  //модератор,
				  $row["author"] != $this->diafan->_user->id ? $this->diafan->_user->id : '',
				  $this->diafan->configmodules('premoderation_message', "forum") && ! $this->diafan->_user->moderator ? 0 : 1,
				  $_POST["save_id"]
				);
			DB::query("UPDATE {forum_category} SET message_update='%d' WHERE id=%d", time(), $row["cat_id"]);
			$this->save_news($row["id"], $row["cat_id"]);
		}

		$row['created']            = $this->format_date($row["created"]).($edit ? ', '._LANG_EDIT_TITLE : '');
		$row["text"]               = $message;
		$row["access_edit_delete"] = TRUE;
		$row["access_block"]       = $this->diafan->_user->moderator ? 1 : 0;
		$row["act"]                = $this->diafan->configmodules('premoderation_message', "forum") && ! $this->diafan->_user->moderator ? 0 : 1;
		$row["children"]           = 0;
		$row["show"]               = 0;
		$row["created"] .= ', '._LANG_EDIT_TITLE.': '
				   .($this->diafan->_user->id != $row["author"] ? $this->get_author($this->diafan->_user->id).', ' : '')
				   .$this->format_date(time());
		$row["author"]             = $this->get_author($row["author"]);
		$result["access_add"] = 1;
		$result["captcha"]    = '';

		$row["attachments"]   = array();
		$result["files"]      = 0;
		if($this->diafan->configmodules("files", "forum"))
		{
			$result["files"]                 = $this->diafan->configmodules("max-count-files",       "forum");
			$result["attachment-extensions"] = $this->diafan->configmodules("attachment-extensions", "forum");
			$row["attachments"]              = $this->attachments->get($row["id"], 'forum', 1);
		}

		$result["rows"][] = $row;

		$result["action"] = URL.'show'.$element_id.'/';
		$result["premoderation"] = $this->diafan->configmodules('premoderation_message', "forum") && ! $this->diafan->_user->moderator;

		$this->result["data"]   = Forum_message_view::show_list($result);
		$this->result["target"] = "#show_message".$_POST["save_id"];
		return $this->send_errors();
	}

	/**
	 * Блокирует|разблокирует сообщение
	 *
	 * @return boolean
	 */
	private function block_message()
	{
		if(empty($_POST["id"]))
		{
			$this->result["errors"][0] = 'ERROR';
			return $this->send_errors();
		}
		if(! $this->diafan->_user->moderator)
		{
			$this->result["errors"][0] = 'ERROR';
			return $this->send_errors();
		}

		$row = DB::fetch_array(DB::query("SELECT * FROM {forum} WHERE id=%d", $_POST["id"]));
		if(! $row)
		{
			$this->result["errors"][0] = 'ERROR';
			return $this->send_errors();
		}
		DB::query("UPDATE {forum} SET act='%d' WHERE id=%d", $row["act"] ? 0 : 1, $_POST["id"]);

		if($row["act"])
		{
			DB::query("DELETE FROM {forum_show} WHERE element_id='%d' AND table_name='forum'", $_POST["id"]);
		}

		$row["access_edit_delete"] = TRUE;
		$row["access_block"] = TRUE;
		$row["act"] = $row["act"] ? FALSE : TRUE;
		$row["created"] = $this->format_date($row["created"]);
		if($row["date_update"])
		{
			$row["date_update"] = $this->format_date($row["date_update"]);
			$row["created"]    .= ', '._LANG_EDIT_TITLE.($row["user_update"] ? ': '.$this->get_author($row["user_update"]).',' : '').' '.$row["date_update"];
		}
		$row["user_update"] = $this->get_author($row["user_update"]);
		$row["author"]      = $this->get_author($row["author"]);
		$row["children"]    = 0;
		$row["show"]        = 0;
		$result["access_add"] = 1;
		$result["captcha"]    = '';

		$row["attachments"]   = array();
		$result["files"]      = 0;
		if($this->diafan->configmodules("files", "forum"))
		{
			$result["files"]                 = $this->diafan->configmodules("max-count-files",       "forum");
			$result["attachment-extensions"] = $this->diafan->configmodules("attachment-extensions", "forum");
			$row["attachments"]              = $this->attachments->get($row["id"], 'forum', 1);
		}

		$result["rows"][]        = $row;

		$result["premoderation"] = $this->diafan->configmodules('premoderation_message', "forum") && ! $this->diafan->_user->moderator;
		$result["action"]        = URL.'show'.$this->diafan->show.'/';

		$this->result["data"]    = Forum_message_view::show_list($result, TRUE);
		$this->result["target"]  = "#show_message".$_POST["id"];
		return $this->send_errors();
	}

	/**
	 * Удяет сообщение
	 *
	 * @return boolean
	 */
	private function delete_message()
	{
		if(empty($_POST["id"]))
		{
			$this->result["errors"][0] = 'ERROR';
			return $this->send_errors();
		}
		if(! $this->diafan->_user->id)
		{
			$this->result["errors"][0] = 'ERROR';
			return $this->send_errors();
		}

		$row = DB::fetch_array(DB::query("SELECT author, cat_id, id, text FROM {forum} WHERE id=%d", $_POST["id"]));

		if($this->diafan->_user->id == $row["author"] && DB::query_result("SELECT COUNT(id) FROM {forum} WHERE parent_id=%d AND trash<>'1' AND del<>'1'", $_POST["id"]))
		{
			$this->result["errors"][0] = _LANG_FORUM_OTHER_MESSAGE;
			return $this->send_errors();
		}

		//модератор
		if($this->diafan->_user->id != $row["author"] && $this->diafan->_user->moderator)
		{
			$this->save_news($_POST["id"], $row["cat_id"], $row["author"]);

			$this->attachments->delete($_POST["id"], "forum");
			DB::query("UPDATE {forum} SET del='1', act='0', user_update='%d', date_update=%d WHERE id=%d", $this->diafan->_user->id, time(), $_POST["id"]);
			$row["del"]  = 1;
			$row["text"] = $this->diafan->short_text($row["text"], 50);
			$row["access_edit_delete"] = FALSE;
			$row["access_block"]       = FALSE;
			$row["children"]           = FALSE;
			$row["show"]               = FALSE;
			$row["user_update"]        = $this->get_author($this->diafan->_user->id);
			$result["rows"][]          = $row;
			$result["action"]          = URL.'show'.$this->diafan->show.'/';
			$this->result["data"]      = Forum_message_view::show_list($result);
			$this->result["target"]    = "#show_message".$_POST["id"];
		}
		//чужой комментарий
		elseif($this->diafan->_user->id != $row["author"])
		{
			$this->result["errors"][0] = _LANG_FORUM_OTHER_AUTHOR;
			return $this->send_errors();
		}
		//автор
		else
		{
			$this->attachments->delete($_POST["id"], "forum");
			DB::query("DELETE FROM {forum} WHERE id=%d", $_POST["id"]);
			$this->result["form_hide"] = TRUE;
			$this->result["target"]    = "#show_message".$_POST["id"];
		}

		DB::query("DELETE FROM {forum_show} WHERE element_id='%d' AND table_name='forum'", $_POST["id"]);

		return $this->send_errors();
	}

	/**
	 * Сохраняет изменения в новостях
	 *
	 * @param integer $save номер сообщения
	 * @param integer $theme номер темы
	 * @param integer|string $del_author номер автора сообщения, если автор удаляет сообщение
	 * @return boolean true
	 */
	private function save_news($save, $theme, $del_author = 'no')
	{
		DB::query("DELETE FROM  {forum_show} WHERE element_id=%d AND table_name='forum'", $save);
		$result = DB::query("SELECT id FROM {users_site} WHERE act='1' AND trash='0' AND id<>'%d'"
				    .($del_author !== "no" ? " AND (moderator='1' OR id='".$del_author."')" : ''),
				    $this->diafan->_user->id);
		while($row = DB::fetch_array($result))
		{
			DB::query("INSERT INTO {forum_show} (element_id, user_id, table_name) VALUES ('%d','%d','forum')", $save, $row["id"]);
			DB::query("INSERT INTO {forum_show} (element_id, user_id, table_name) VALUES ('%d','%d','forum_category')", $theme, $row["id"]);
		}
		return TRUE;
	}

	/**
	 * Заменяет HTML-теги на bbcode
	 *
	 * @param string $text текст
	 * @return string
	 */
	private function add_bbcode($text)
	{
		$text = preg_replace("/<li>(.*?)<\/li>/",  "[*]$1\n",  $text);
		$text = preg_replace('/<a href=([^>]+)>/', '[url=$1]', $text);

		$text = str_replace(
			array(
			      '<div class="code"><div class="code-header">'._LANG_FORUM_CODE.'</div><code>',
			      '</code></div>',
			      '</a>',
			      '<i>',
			      '</i>',
			      '<b>',
			      '</b>',
			      '<u>',
			      '</u>',
			      '<ul>',
			      '<ol>',
			      '</ol>',
			      '</ul>',
			      '<br>',

				'<img src="'.BASE_PATH.'modules/forum/bbeditor/smiles/acute.gif">',
				'<img src="'.BASE_PATH.'modules/forum/bbeditor/smiles/bad.gif">',
				'<img src="'.BASE_PATH.'modules/forum/bbeditor/smiles/biggrin.gif">',
				'<img src="'.BASE_PATH.'modules/forum/bbeditor/smiles/blum.gif">',
				'<img src="'.BASE_PATH.'modules/forum/bbeditor/smiles/blush.gif">',
				'<img src="'.BASE_PATH.'modules/forum/bbeditor/smiles/bomb.gif">',
				'<img src="'.BASE_PATH.'modules/forum/bbeditor/smiles/boredom.gif">',
				'<img src="'.BASE_PATH.'modules/forum/bbeditor/smiles/bye.gif">',
				'<img src="'.BASE_PATH.'modules/forum/bbeditor/smiles/clapping.gif">',
				'<img src="'.BASE_PATH.'modules/forum/bbeditor/smiles/cool.gif">',
				'<img src="'.BASE_PATH.'modules/forum/bbeditor/smiles/cray.gif">',
				'<img src="'.BASE_PATH.'modules/forum/bbeditor/smiles/dance.gif">',
				'<img src="'.BASE_PATH.'modules/forum/bbeditor/smiles/diablo.gif">',
				'<img src="'.BASE_PATH.'modules/forum/bbeditor/smiles/drinks.gif">',
				'<img src="'.BASE_PATH.'modules/forum/bbeditor/smiles/empathy.gif">',
				'<img src="'.BASE_PATH.'modules/forum/bbeditor/smiles/flag_of_truce.gif">',
				'<img src="'.BASE_PATH.'modules/forum/bbeditor/smiles/good.gif">',
				'<img src="'.BASE_PATH.'modules/forum/bbeditor/smiles/help.gif">',
				'<img src="'.BASE_PATH.'modules/forum/bbeditor/smiles/hi.gif">',
				'<img src="'.BASE_PATH.'modules/forum/bbeditor/smiles/i_am_so_happy.gif">',
				'<img src="'.BASE_PATH.'modules/forum/bbeditor/smiles/lol.gif">',
				'<img src="'.BASE_PATH.'modules/forum/bbeditor/smiles/nea.gif">',
				'<img src="'.BASE_PATH.'modules/forum/bbeditor/smiles/negative.gif">',
				'<img src="'.BASE_PATH.'modules/forum/bbeditor/smiles/new_russian.gif">',
				'<img src="'.BASE_PATH.'modules/forum/bbeditor/smiles/wink.gif">',
				'<img src="'.BASE_PATH.'modules/forum/bbeditor/smiles/smile.gif">'
			      ),

			array(
				'[code]',
				'[/code]',
				'[/url]',
				'[i]',
				'[/i]',
				'[b]',
				'[/b]',
				'[u]',
				'[/u]',
				'[LIST]',
				'[LIST=1]',
				'[/LIST]',
				'[/LIST]',
				"\n",

				'[:acute:]',
				'[:bad:]',
				'[:biggrin:]',
				'[:blum:]',
				'[:blush:]',
				'[:bomb:]',
				'[:boredom:]',
				'[:bye:]',
				'[:clapping:]',
				'[:cool:]',
				'[:cray:]',
				'[:dance:]',
				'[:diablo:]',
				'[:drinks:]',
				'[:empathy:]',
				'[:flag_of_truce:]',
				'[:good:]',
				'[:help:]',
				'[:hi:]',
				'[:i_am_so_happy:]',
				'[:lol:]',
				'[:nea:]',
				'[:negative:]',
				'[:new_russian:]',
				'[:wink:]',
				'[:smile:]'
			      ),
			$text
		);
		$text = preg_replace("/<img src=([^>]+)>/", "[img]$1[/img]", $text);
		$text = preg_replace("/<div class=\"quote\"><div class=\"quote-header\">"._LANG_FORUM_QUOTE."<\/div>([^\t]*?)<\/div>/",
				     '[quote]$1[/quote]',
				     $text);
		return $text;
	}

	/**
	 * Заменяет bbcode на HTML-теги
	 *
	 * @param string $text текст
	 * @return string
	 */
	private function replace_bbcode($text)
	{
		$text = htmlspecialchars($text);

		$text = preg_replace("/\[LIST=1\]([^\t]*?)\[\/LIST\]/", "<ol>$1</ol>", $text);
		$text = preg_replace("/\[\*\]([^\r\n]+)\r\n/", "<li>$1</li>", $text);

		include_once(ABSOLUTE_PATH.'includes/validate.php');
		$text = preg_replace_callback("/\[url=([^\]]+)\]/", array('Forum_message_ajax', '_url_callback'), $text);
		$text = preg_replace_callback("/\[url\](.*?)\[\/url\]/", array('Forum_message_ajax', '_url_full_callback'), $text);
		$text = preg_replace_callback("/\[img\](.*?)\[\/img\]/", array('Forum_message_ajax', '_img_callback'), $text);

		$text = str_replace(
			array(
				'[code]',
				'[/code]',
				'[quote]',
				'[/quote]',
				'[/url]',
				'[i]',
				'[/i]',
				'[b]',
				'[/b]',
				'[u]',
				'[/u]',
				'[LIST]',
				'[/LIST]',
				"\n",

				'[:acute:]',
				'[:bad:]',
				'[:biggrin:]',
				'[:blum:]',
				'[:blush:]',
				'[:bomb:]',
				'[:boredom:]',
				'[:bye:]',
				'[:clapping:]',
				'[:cool:]',
				'[:cray:]',
				'[:dance:]',
				'[:diablo:]',
				'[:drinks:]',
				'[:empathy:]',
				'[:flag_of_truce:]',
				'[:good:]',
				'[:help:]',
				'[:hi:]',
				'[:i_am_so_happy:]',
				'[:lol:]',
				'[:nea:]',
				'[:negative:]',
				'[:new_russian:]',
				'[:wink:]',
				'[:smile:]'
			      ),

			array(
			       '<div class="code"><div class="code-header">'._LANG_FORUM_CODE.'</div><code>',
				'</code></div>',
				'<div class="quote"><div class="quote-header">'._LANG_FORUM_QUOTE.'</div>',
				'</div>',
				'</a>',
				'<i>',
				'</i>',
				'<b>',
				'</b>',
				'<u>',
				'</u>',
				'<ul>',
				'</ul>',
				'<br>',

				'<img src="'.BASE_PATH.'modules/forum/bbeditor/smiles/acute.gif">',
				'<img src="'.BASE_PATH.'modules/forum/bbeditor/smiles/bad.gif">',
				'<img src="'.BASE_PATH.'modules/forum/bbeditor/smiles/biggrin.gif">',
				'<img src="'.BASE_PATH.'modules/forum/bbeditor/smiles/blum.gif">',
				'<img src="'.BASE_PATH.'modules/forum/bbeditor/smiles/blush.gif">',
				'<img src="'.BASE_PATH.'modules/forum/bbeditor/smiles/bomb.gif">',
				'<img src="'.BASE_PATH.'modules/forum/bbeditor/smiles/boredom.gif">',
				'<img src="'.BASE_PATH.'modules/forum/bbeditor/smiles/bye.gif">',
				'<img src="'.BASE_PATH.'modules/forum/bbeditor/smiles/clapping.gif">',
				'<img src="'.BASE_PATH.'modules/forum/bbeditor/smiles/cool.gif">',
				'<img src="'.BASE_PATH.'modules/forum/bbeditor/smiles/cray.gif">',
				'<img src="'.BASE_PATH.'modules/forum/bbeditor/smiles/dance.gif">',
				'<img src="'.BASE_PATH.'modules/forum/bbeditor/smiles/diablo.gif">',
				'<img src="'.BASE_PATH.'modules/forum/bbeditor/smiles/drinks.gif">',
				'<img src="'.BASE_PATH.'modules/forum/bbeditor/smiles/empathy.gif">',
				'<img src="'.BASE_PATH.'modules/forum/bbeditor/smiles/flag_of_truce.gif">',
				'<img src="'.BASE_PATH.'modules/forum/bbeditor/smiles/good.gif">',
				'<img src="'.BASE_PATH.'modules/forum/bbeditor/smiles/help.gif">',
				'<img src="'.BASE_PATH.'modules/forum/bbeditor/smiles/hi.gif">',
				'<img src="'.BASE_PATH.'modules/forum/bbeditor/smiles/i_am_so_happy.gif">',
				'<img src="'.BASE_PATH.'modules/forum/bbeditor/smiles/lol.gif">',
				'<img src="'.BASE_PATH.'modules/forum/bbeditor/smiles/nea.gif">',
				'<img src="'.BASE_PATH.'modules/forum/bbeditor/smiles/negative.gif">',
				'<img src="'.BASE_PATH.'modules/forum/bbeditor/smiles/new_russian.gif">',
				'<img src="'.BASE_PATH.'modules/forum/bbeditor/smiles/wink.gif">',
				'<img src="'.BASE_PATH.'modules/forum/bbeditor/smiles/smile.gif">'
			      ),
			$text
		);
		return $text;
	}

	/**
	 * Callback функция. Заменяет bbcode изображения на HTML-тег
	 *
	 * @param array $match найденные совпадения
	 * @return string
	 */
	public function _img_callback($match)
	{
		$url = explode(' ', $match[1]);
		return (Validate::url($url[0], TRUE) ? '<img src='.$url[0].'>' : '');
	}

	/**
	 * Callback функция. Заменяет bbcode ссылки на HTML-тег
	 *
	 * @param array $match найденные совпадения
	 * @return string
	 */
	public function _url_callback($match)
	{
		$url = explode(' ', $match[1]);
		return '<a href='.(Validate::url($url[0], TRUE) ? $url[0] : '').' target=_blank>';
	}

	/**
	 * Callback функция. Заменяет bbcode ссылки с текстом на HTML-тег
	 *
	 * @param array $match найденные совпадения
	 * @return string
	 */
	public function _url_full_callback($match)
	{
		$url = explode(' ', $match[1]);
		return (Validate::url($url[0], TRUE) ? '<a href='.$url[0].' target=_blank>'.$url[0].'</a>' : '');
	}

	/**
	 * Проверяет правильность каптчи
	 *
	 * @param string $module_name название модуля
	 * @return boolean
	 */
	private function check_captcha()
	{
		$this->result["code"] = '';
		if($this->diafan->configmodules("access_add") == 1 && ! $this->diafan->_user->id)
		{
			require_once(ABSOLUTE_PATH.'includes/captcha/captcha.inc.php');	
			$mes = Captcha_inc::is_right('messages');
			$this->result["code"] = Captcha_inc::get('messages', '', TRUE);				
			if($mes)
			{
				$this->result["errors"]["code"] = $mes;
			}
		}
		return $this->send_errors();
	}

	/**
	 * Отправляет ответ
	 *
	 * @return boolean
	 */
	private function send_errors()
	{
		if(! empty($this->result["errors"]) || ! empty($this->result["add"]) || ! empty($this->result["data"]) || ! empty($this->result["form_hide"]))
		{
			if(! empty($_POST['ajax']))
			{
				include_once(ABSOLUTE_PATH.'plugins/json.php');
				echo to_json($this->result);
			}
			else
			{
				$query = '';
				if(! empty($this->result["errors"]))
				{
					foreach($this->result["errors"] as $field => $mes)
					{
						if(empty($_POST["update"]))
						{
							$query .= '&mess'.($field ? '-'.$field : '').'='.$mes;
						}
					}
				}
				$this->diafan->goto_diafan($this->diafan->path_module_variable().($query ? '?error=forum'.$query : ''));
			}
			return TRUE;
		}
		return FALSE;
	}
}