<?php
/**
 * Diafan.CMS
 *
 * @author     diafan.ru
 * @version    4.3
 * @license http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2011 OOO "Диафан". (http://diafan.ru)
 */

if(! defined('DIAFAN'))
{
	include(dirname(dirname(dirname(dirname(__FILE__)))).'/includes/404.php');
}

/**
 * Forum_message_inc
 *
 * Работа с сообщениями в модулей "Форум"
 */
class Forum_message_inc extends Model
{
	/**
	 * @var object основной объект системы
	 */
	protected $diafan;

	/**
	 * @var array сгенерированные в моделе данные, передаваемые в шаблон
	 */
	protected $result;

	/**
	 * Конструктор класса
	 * 
	 * @return void
	 */
	public function __construct($diafan)
	{		
		$this->diafan = $diafan;

		include_once(ABSOLUTE_PATH.'modules/forum/attachments/attachments.inc.php');

		include_once(ABSOLUTE_PATH.'modules/forum/message/message.view.php');

		include_once(ABSOLUTE_PATH.'modules/forum/message/message.ajax.php');
		$ajax = new Forum_message_ajax($diafan);
		if($ajax->ajax_request())
		{
			exit;
		}
		$this->attachments = new Attachments_inc($diafan);
	}

	/**
	 * Генерирует список сообщений и форму добавления сообщений
	 * 
	 * @param integer $element_id номер категории форума
	 * @return string
	 */
	public function show($element_id = 0)
	{
		global $authors;
		if(! $element_id)
		{
			$element_id = $this->diafan->show;
		}

		$result = DB::query("SELECT * FROM {forum} WHERE cat_id='%d' AND  trash='0' ORDER BY created ASC", $element_id);
		$rows = array();
		$this->result["close"] = DB::query_result("SELECT close FROM {forum_category} WHERE id=%d", $this->diafan->show);
		while($row = DB::fetch_array($result))
		{
			$row["access_edit_delete"] = $this->diafan->_user->id
			                             && ($row["author"] == $this->diafan->_user->id || $this->diafan->_user->moderator)
						     && ! $this->result["close"]
						     ? TRUE : FALSE;
			$row["access_block"]       = $this->diafan->_user->moderator ? TRUE : FALSE;
			$row["author"]             = $this->get_author($row["author"]);
			$row["created"] = $this->format_date($row["created"]);
			if($row["date_update"])
			{
				$row["date_update"] = $this->format_date($row["date_update"]);
				$row["created"] .= ', '._LANG_EDIT_TITLE
				                   .($row["user_update"] ? ': '.$this->get_author($row["user_update"]).',' : '')
						   .' '.$row["date_update"];
			}
			if($row["del"])
			{
				$row["text"] = $this->diafan->short_text($row["text"], 50);
			}

			$row["attachments"] = array();
			if($this->diafan->configmodules("files"))
			{
				$row["attachments"] = $this->attachments->get($row["id"], 'forum', $row["access_edit_delete"]);
			}

			if($this->diafan->_user->id)
			{
				if($row["show"] = DB::query_result("SELECT COUNT(id) FROM {forum_show} WHERE element_id='%d' AND table_name='forum'"
								   ." AND user_id=%d", $row["id"], $this->diafan->_user->id))
				{
					DB::query("DELETE FROM {forum_show} WHERE user_id=%d AND element_id=%d AND table_name='forum'", $this->diafan->_user->id, $row["id"]);
				}
			}
			else
			{
				$row["show"] = FALSE;
			}
			$rows[$row["parent_id"]][] = $row;
		}

		if($this->diafan->_user->id)
		{
			DB::query("DELETE FROM {forum_show} WHERE user_id=%d AND element_id=%d AND table_name='forum_category'", $this->diafan->_user->id, $element_id);
		}
		$this->result["files"] = $this->diafan->configmodules("files", "forum");
		if($this->result["files"])
		{
			$this->result["attachment-extensions"] = $this->diafan->configmodules("attachment-extensions", "forum");
		}
		$this->result["rows"]          = $this->build_tree($rows);
		$this->result["action"]        = URL.'show'.$element_id.'/';
		$this->result["access_add"]    = ($this->diafan->configmodules("access_add", "forum") != 2 || $this->diafan->_user->id) && ! $this->result["close"];
		$this->result["premoderation"] = $this->diafan->configmodules('premoderation_message', "forum") && ! $this->diafan->_user->moderator;

		//доступ на добавление по каптче
		$this->result["captcha"] = '';
		if($this->diafan->configmodules("access_add", "forum") == 1 && ! $this->diafan->_user->id)
		{
			require_once (ABSOLUTE_PATH.'includes/captcha/captcha.inc.php');
			$this->result["captcha"] = Captcha_inc::get("messages", $this->get_error("messages", "code"));
		}

		if($this->result["access_add"])
		{
			$this->result["error"]             = $this->get_error("messages");
			$this->result["error-message"]     = $this->get_error("messages", 'message');
			$this->result["error-attachments"] = $this->get_error("messages", 'attachments');
		}

		$text = Forum_message_view::show_module($this->result);

		return $text;
	}

	/**
	 * Формирует дерево сообщений из полученного массива
	 *
	 * @param array $rows все сообщения
	 * @param integer $parent_id номер текущего сообщения-родителя
	 * @return string
	 */
	private function build_tree($rows, $parent_id = 0)
	{
		$result = array();
		if(! empty($rows[$parent_id]))
		{
			foreach($rows[$parent_id] as $row)
			{
				$row["children"] = $this->build_tree($rows, $row["id"]);
				$result[] = $row;
			}
		}
		return $result;
	}

	/**
	 * Генерирует данные о количестве сообщений в категории форума
	 * 
	 * @param integer $element_id номер категории форума
	 * @return integer
	 */
	public function count($element_id = 0)
	{
		if(!$element_id)
		{
			$element_id = $this->diafan->show;
		}

		$count = DB::query_result("SELECT COUNT(id) FROM {forum} WHERE cat_id='%d' AND trash='0'"
					  .(! $this->diafan->_user->moderator
					    ? " AND (act='1' OR author=".$this->diafan->_user->id.")"
					    ." AND (del='0' OR author=".$this->diafan->_user->id.")"
					    : ''), $element_id);
		if($count)
		{
			$count--;
		}
		return $count;
	}

	/**
	 * Генерирует данные о количестве новых сообщений в категории форума
	 * 
	 * @param integer $element_id номер категории форума
	 * @param boolean $parent является ли категория форума каталогом для тем
	 * @return integer
	 */
	public function count_new($element_id = 0, $parent = false)
	{
		if(! $element_id)
		{
			$element_id = $this->diafan->show;
		}

		$count = DB::query_result("SELECT COUNT(m.id) FROM {forum} as m"
					  ." INNER JOIN {forum_show} as n ON n.element_id=m.id"
					  .($parent ? " INNER JOIN {forum_category} as c ON c.id=m.cat_id" : '')
					  ." WHERE n.user_id=%d AND n.table_name='forum' AND m.trash='0'"
					  .($parent ? " AND (c.id='".$element_id."' OR c.parent_id='".$element_id."')" : " AND m.cat_id='".$element_id."'")
					  .(! $this->diafan->_user->moderator
					    ? " AND (m.act='1' OR m.author=".$this->diafan->_user->id.")"
					    ." AND (m.del='0' OR m.author=".$this->diafan->_user->id.")"
					    : ''),
					  $this->diafan->_user->id);
		if($count)
		{
			$count--;
		}
		return $count;
	}

	/**
	 * Генерирует данные о последнем пользователе, написавшем в категории форума
	 * 
	 * @param integer $element_id номер категории форума
	 * @return string
	 */
	public function last_user($element_id = 0)
	{
		if(! $element_id)
		{
			$element_id = $this->diafan->show;
		}

		$row = DB::fetch_array(DB::query("SELECT created, author FROM {forum} WHERE cat_id='%d' AND trash='0'"
						 .(! $this->diafan->_user->moderator
						   ? " AND (act='1' OR author=".$this->diafan->_user->id.")"
						     ." AND (del='0' OR author=".$this->diafan->_user->id.")"
						   : '')
						 ." ORDER BY created DESC LIMIT 1", $element_id));
		if(! $row)
			return '';

		$this->result["author"]  = $this->get_author($row["author"]);
		$this->result["created"] = $this->format_date($row["created"]);
		return $this->result;
	}

	/**
	 * Удаляет сообщения, прикрепленные к категории форума
	 *
	 * @param integer $element_id номер категории форума
	 * @return boolean true
	 */
	public function delete($element_id = 0)
	{
		if(! $element_id)
		{
			$element_id = $this->diafan->del;
		}

		$result = DB::query("SELECT id FROM {forum} WHERE cat_id='%d'", $element_id);
		while($row = DB::fetch_array($result))
		{
			$this->attachments->delete($row["id"], "forum");
			DB::query("DELETE FROM {forum_show} WHERE element_id=%d AND table_name='forum'", $row["id"]);
		}
		DB::query("DELETE FROM {forum} WHERE cat_id='%d'", $element_id);

		return TRUE;
	}
}