<?php
/**
 * Diafan.CMS
 *
 * @author     diafan.ru
 * @version    4.3
 * @license http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2011 OOO "Диафан". (http://diafan.ru)
 */

if(! defined('DIAFAN'))
{
	include(dirname(dirname(dirname(__FILE__))).'/includes/404.php');
}

/**
 * News_model
 *
 * Модель модуля "Новости"
 */
class News_model extends Model
{
	/**
	 * @var array сгенерированные в моделе данные, передаваемые в шаблон
	 */
	protected $result;

	/**
	 * @var object основной объект системы
	 */
	protected $diafan;

	/**
	 * Конструктор класса
	 * 
	 * @return void
	 */
	public function __construct($diafan)
	{
		$this->diafan = $diafan;
		include_once(ABSOLUTE_PATH."includes/image.php");
	}

	/**
	 * Генерирует данные для списка всех новостей без деления на категории
	 * 
	 * @return array
	 */
	public function list_()
	{
		if($this->diafan->cat)
		{
			$this->diafan->goto_diafan($this->diafan->path_module_variable("cat"));
		}
		$time = mktime(23, 59, 0, date("m"), date("d"), date("Y"));

		$cache_meta = array(
				    "name"     => "list",
				    "language" => _LANG,
				    "page"     => $this->diafan->page > 1 ? $this->diafan->page : 1,
				    "site_id"  => $this->diafan->cid,
				    "time"     => $time,
				    "year"     => $this->diafan->year,
				    "month"    => $this->diafan->month,
				    );

		//кеширование
		if(! $this->result = $this->diafan->_cache->get($cache_meta, $this->diafan->module))
		{
			$this->result = array();

			if($this->diafan->year || $this->diafan->month)
			{
				if($this->diafan->cat)
				{
					include(ABSOLUTE_PATH.'includes/404.php');
				}
				if($this->diafan->month)
				{	
					$month_arr = array(
						'12' => _LANG_DECEMBER,
						'11' => _LANG_NOVEMBER,
						'10' => _LANG_OCTOBER,
						'09' => _LANG_SEPTEMBER,
						'08' => _LANG_AUGUST,
						'07' => _LANG_JULY,
						'06' => _LANG_JUNE,
						'05' => _LANG_MAY,
						'04' => _LANG_APRIL,
						'03' => _LANG_MARCH,
						'02' => _LANG_FEBRUARY,
						'01' => _LANG_JANUARY
					);
			
					$this->result["titlemodule"] =
						sprintf(
						 _LANG_NEWS_MONTH_YEAR,
						 $month_arr[$this->diafan->month],
						 $this->diafan->year
						);
				}
				else
				{
					$this->result["titlemodule"] =
					sprintf(
						 _LANG_NEWS_YEAR,
						 $this->diafan->year
					);			
				}
		
				$time  = mktime(23, 59, 0, date("m"), date("d"), date("Y"));
				$time1 = mktime(0, 0, 0, ($this->diafan->month ? $this->diafan->month : '1'), 1, ($this->diafan->year ? $this->diafan->year : date("Y")));
				$time2 = mktime(0, 0, 0, ($this->diafan->month ? $this->diafan->month : 12), 31, ($this->diafan->year ? $this->diafan->year : date("Y")));
				$time2 = $time2 < $time ? $time2 : $time;

				////navigation///
				include_once(ABSOLUTE_PATH.'includes/paginator/paginator.inc.php');
				$paginator = new Paginator_inc();
				$paginator->page    = $this->diafan->page;
				$paginator->navlink = $this->diafan->path_module_variable("page");
				$paginator->nen = DB::query_result("SELECT COUNT(id) FROM {news} WHERE [act]='1' AND trash='0'"
								   ." AND site_id=%d AND created>=%d AND created<%d"
								   ." ORDER BY prior DESC, created DESC, id DESC",
								   $this->diafan->cid, $time1, $time2);
				$links = $paginator->get();
				////navigation///
		
				$result = DB::query_range("SELECT id, created, [name], [anons], timeedit, cat_id, site_id FROM {news} WHERE [act]='1'"
							  ." AND trash='0' AND site_id=%d AND created>=%d AND created<%d"
							  ." ORDER BY prior DESC, created DESC, id DESC",
							  $this->diafan->cid, $time1, $time2,
							  $paginator->polog, $paginator->nastr);
			}
			else
			{
				////navigation///
				include_once(ABSOLUTE_PATH.'includes/paginator/paginator.inc.php');
				$paginator = new Paginator_inc();
				$paginator->page    = $this->diafan->page;
				$paginator->navlink = $this->diafan->path_module_variable("page");
				$paginator->nen = DB::query_result("SELECT COUNT(id) FROM {news} WHERE [act]='1' AND"
								   ." trash='0' AND site_id=%d AND created<%d",
								   $this->diafan->cid, $time);
				$links = $paginator->get();
				////navigation///
				
				$result = DB::query_range("SELECT id, created, [name], [anons], timeedit, cat_id, site_id FROM {news}"
							 ." WHERE [act]='1' AND trash='0' AND site_id=%d"
							 ." AND created<%d ORDER BY prior DESC, created DESC, id DESC",
							 $this->diafan->cid, $time, $paginator->polog, $paginator->nastr);
			}

			$this->result["rows"] = $this->get_elements($result);
			$this->result["path"] = $this->get_path('news');

			$this->result["paginator"] = $links;
			
			//сохранение кеша
			$this->diafan->_cache->save($this->result, $cache_meta, $this->diafan->module);
		}
		else
		{
			include_once(ABSOLUTE_PATH.'includes/paginator/paginator.view.php');
		}

		if($this->result["rows"])
		{
			if($this->diafan->configmodules("rating"))
			{
				include_once(ABSOLUTE_PATH.'modules/rating/rating.inc.php');
				$rating = new Rating_inc($this->diafan);
			}
			if($this->diafan->configmodules("tags"))
			{
				include_once(ABSOLUTE_PATH."modules/tags/tags.inc.php");
				$tags = new Tags_inc($this->diafan);
			}

			for($i = 0; $i < count($this->result["rows"]); $i++)
			{
				$this->result["rows"][$i]["tags"] = $this->diafan->configmodules("tags")
								    ? $tags->get($this->result["rows"][$i]["id"])
								    : '';

				$this->result["rows"][$i]["show-rating"] = $this->diafan->configmodules("rating")
									   ? $rating->get($this->result["rows"][$i]["id"])
									   : '';
			}
		}
		return $this->result;
		
	}

	/**
	 * Генерирует данные для первой страницы новостей
	 * 
	 * @return array
	 */
	public function first_page()
	{
		if($this->diafan->page)
		{
			$this->diafan->goto_diafan($this->diafan->path_module_variable("page"));
		}
		$time = mktime(23, 59, 0, date("m"), date("d"), date("Y"));

		//кеширование
		$cache_meta = array(
				    "name"     => "first_page",
				    "language" => _LANG,
				    "time"     => $time,
				    "site_id"  => $this->diafan->cid
				    );
		if(! $this->result = $this->diafan->_cache->get($cache_meta, $this->diafan->module))
		{
			$this->result = array();

			$result = DB::query("SELECT id, [name], [anons], timeedit, site_id FROM {news_category}"
					    ." WHERE [act]='1' AND parent_id=0 AND trash='0' AND site_id=%d"
					    ." ORDER by sort ASC, id ASC", $this->diafan->cid);

			while ($row = DB::fetch_array($result))
			{
				$this->result["categories"][$row["id"]]["name"] = $row["name"];
				$this->result["categories"][$row["id"]]["anons"] = $row["anons"];

				if(empty($this->result["timeedit"]) || $row["timeedit"] > $this->result["timeedit"])
				{
					$this->result["timeedit"] = $row["timeedit"];
				}

				$this->result["categories"][$row["id"]]["children"] = array();
				$result_children = DB::query("SELECT id, [name], [anons], site_id FROM {news_category}"
							     ." WHERE [act]='1' AND parent_id=%d AND trash='0'"
							     ." AND site_id=%d ORDER BY sort ASC, id ASC",
							     $row["id"], $this->diafan->cid);

				while ($row_children = DB::fetch_array($result_children))
				{
					$child["link"]  = $this->get_link_category($row_children, 'news');
					if($this->diafan->configmodules("list_img_cat"))
					{
						$child["img"] = Image::get($row_children["id"], $row_children["name"], 'news',
									   $this->diafan->configmodules("list_img_cat") == 1 ? 1 : 0,
									   $child["link"], TRUE);
					}
					$child["name"]  = $row_children["name"];
					$child["anons"] = $row_children["anons"];
					$this->result["categories"][$row["id"]]["children"][] = $child;
				}

				if($this->diafan->configmodules("childrens_elements"))
				{
					$children = $this->diafan->get_children($row["id"],"news_category");
				}

				$result_img = DB::query_range("SELECT e.id, e.[name], e.cat_id, e.timeedit, e.[anons], e.site_id, e.created FROM {news} AS e"
							      ." INNER JOIN {category} AS c ON e.id=c.element_id"
							      ." WHERE c.module_name='news' AND e.cat_id IN (%s) AND e.[act]='1'"
							      ." AND e.trash='0' AND e.created<'%d'"
							      ." GROUP BY e.id ORDER BY e.prior DESC, e.created DESC, e.id DESC",
							      $this->diafan->configmodules("childrens_elements") ? implode(',', $children) : $row["id"],
							      $time,
							      0, $this->diafan->configmodules("count_list")
							      );
				$this->result["categories"][$row["id"]]["rows"] = $this->get_elements($result_img);

				$this->result["categories"][$row["id"]]["allnews"]["link"] = $this->get_link_category($row, 'news');

				if($this->diafan->configmodules("list_img_cat"))
				{
					$this->result["categories"][$row["id"]]["img"] = Image::get($row["id"], $row["name"], 'news',
												    $this->diafan->configmodules("list_img_cat") == 1 ? 1 : 0,
												    $this->result["categories"][$row["id"]]["allnews"]["link"],
												    TRUE);
				}
				$this->result["categories"][$row["id"]]["allnews"]["text"] = $this->diafan->configmodules("allnews").' '._LANG_LQUOT.$row["name"]._LANG_RQUOT;
			}

			//сохранение кеша
			$this->diafan->_cache->save($this->result, $cache_meta, $this->diafan->module);
		}

		if(! empty($this->result["categories"]))
		{
			if($this->diafan->configmodules("rating"))
			{
				include_once(ABSOLUTE_PATH.'modules/rating/rating.inc.php');
				$rating = new Rating_inc($this->diafan);
			}
			if($this->diafan->configmodules("tags"))
			{
				include_once(ABSOLUTE_PATH."modules/tags/tags.inc.php");
				$tags = new Tags_inc($this->diafan);
			}

			foreach($this->result["categories"] as $cat => $rows)
			{
				if(! $rows)
					continue;

				foreach($rows["rows"] as $k => $row)
				{
					$this->result["categories"][$cat]["rows"][$k]["tags"] = ($this->diafan->configmodules("tags")
									   ? $tags->get($this->result["categories"][$cat]["rows"][$k]["id"])
									   : '');
	
					$this->result["categories"][$cat]["rows"][$k]["show-rating"] = ($this->diafan->configmodules("rating")
									  ? $rating->get($this->result["categories"][$cat]["rows"][$k]["id"])
									  : '');
				}
			}
		}
		return $this->result;
	}

	/**
	 * Генерирует данные для списка новостей в категории
	 * 
	 * @return array
	 */
	public function list_category()
	{
		$time = mktime(23, 59, 0, date("m"), date("d"), date("Y"));

		//кеширование
		$cache_meta = array(
				    "name"     => "list",
				    "cat_id"   => $this->diafan->cat,
				    "language" => _LANG,
				    "page"     => $this->diafan->page > 1 ? $this->diafan->page : 1,
				    "time"     => $time,
				    "site_id"  => $this->diafan->cid
				    );
		if(! $this->result = $this->diafan->_cache->get($cache_meta, $this->diafan->module))
		{
			$this->result = array();

			$this->result["level"] = count($this->diafan->get_parents($this->diafan->cat, 'news_category'));
			
			$this->result["path"] = $this->get_path('news');

			$result = DB::query("SELECT id, [name], timeedit, [descr], [keywords], sort, parent_id, [title_meta] FROM {news_category}"
					    ." WHERE [act]='1' and id=%d AND trash='0' AND site_id=%d"
					    ." ORDER BY sort ASC, id ASC",
					    $this->diafan->cat, $this->diafan->cid);
				
			if(! $row = DB::fetch_array($result))
			{
				include_once(ABSOLUTE_PATH.'includes/404.php');
			}

			$this->result["img"] = Image::get($row["id"], $row["name"], 'news', 0, 1, TRUE);

			if($this->diafan->configmodules("childrens_elements"))
			{
				$children = $this->diafan->get_children($this->diafan->cat, "news_category");
			}

			$this->result["titlemodule"] = $row["name"];
			$this->result["title_meta"]  = $row["title_meta"];
			$this->result["timeedit"]    = $row["timeedit"];
			$this->result["keywords"]    = $row["keywords"];
			$this->result["descr"]       = $row["descr"];

			$previous = DB::fetch_array(DB::query("SELECT [name], id FROM {news_category} WHERE [act]='1' AND trash='0' AND site_id=%d"
							      ." AND (sort<%d OR sort=%d AND id<%d) AND parent_id=%d ORDER BY sort DESC, id DESC LIMIT 1",
							      $this->diafan->cid, $row["sort"], $row["sort"], $this->diafan->cat, $row["parent_id"]));
			if($previous)
			{
				$this->result["previous"]["text"] = $previous["name"];
				if(! empty($this->diafan->rewrites["news"]["cat".$previous["id"]]))
				{
					$this->result["previous"]["link"] = $this->diafan->rewrites["news"]["cat".$previous["id"]]["rewrite"].'/';
				}
				else
				{
					$this->result["previous"]["link"] = ($this->diafan->rewrite ? $this->diafan->rewrite.'/' : '')
					                                    .'cat'.$previous["id"].'/';
				}
			}
			$next = DB::fetch_array(DB::query("SELECT [name], id FROM {news_category} WHERE [act]='1' AND trash='0' AND site_id=%d"
							  ." AND (sort>%d OR sort=%d AND id>%d) AND parent_id=%d ORDER BY sort ASC, id ASC LIMIT 1",
							  $this->diafan->cid, $row["sort"], $row["sort"], $this->diafan->cat, $row["parent_id"]));
			if($next)
			{
				$this->result["next"]["text"] = $next["name"];
				if(! empty($this->diafan->rewrites["news"]["cat".$next["id"]]))
				{
					$this->result["next"]["link"] = $this->diafan->rewrites["news"]["cat".$next["id"]]["rewrite"].'/';
				}
				else
				{
					$this->result["next"]["link"] = ($this->diafan->rewrite ? $this->diafan->rewrite.'/' : '')
									.'cat'.$next["id"].'/';
				}
			}
			
			$this->result["children"] = array();
			$result_children = DB::query("SELECT id, [name], [anons], site_id FROM {news_category}"
						    ." WHERE [act]='1' AND parent_id=%d AND trash='0'"
						    ." AND site_id=%d ORDER BY sort ASC, id ASC",
						    $row["id"], $this->diafan->cid);
				
			while ($row_children = DB::fetch_array($result_children))
			{
				$child["link"]  = $this->get_link_category($row_children, 'news');
				if($this->diafan->configmodules("list_img_cat"))
				{
					$child["img"] = Image::get($row_children["id"], $row_children["name"], 'news',
								   $this->diafan->configmodules("list_img_cat") == 1 ? 1 : 0,
								   $child["link"], TRUE);
				}
				$child["name"]  = $row_children["name"];
				$child["anons"] = $row_children["anons"];
				$this->result["children"][] = $child;
			}
			////navigation//
			include_once(ABSOLUTE_PATH.'includes/paginator/paginator.inc.php');
			$paginator = new Paginator_inc();
			$paginator->page    = $this->diafan->page;
			$paginator->navlink = $this->diafan->path_module_variable("page");
			$paginator->nen = DB::query_result("SELECT COUNT(DISTINCT e.id) FROM {news} as e"
							   ." INNER JOIN {category} AS c ON e.id=c.element_id AND c.module_name='news'"
							   ." AND e.id=c.element_id AND c.cat_id IN (%s)"
							   ." WHERE e.[act]='1' AND e.trash='0' AND e.site_id=%d"
							   ." AND e.created<'%d'",

							   $this->diafan->configmodules("childrens_elements")
							   ? implode(',', $children) : $this->diafan->cat,
	
							   $this->diafan->cid, $time
							   );
			$links = $paginator->get();
			////navigation///

			$result_img = DB::query_range("SELECT e.id, e.[name], e.cat_id, e.timeedit, e.[anons], e.site_id, e.created FROM {news} AS e"
						      ." INNER JOIN {category} AS c ON c.module_name='news' AND e.id=c.element_id AND c.cat_id IN (%s)"
						      ." WHERE e.[act]='1' AND e.trash='0' AND created<%d GROUP BY e.id"
						      ." ORDER BY e.prior DESC, e.created DESC, e.id DESC",
						      $this->diafan->configmodules("childrens_elements") ? implode(',', $children) : $this->diafan->cat,
						      $time,
						      $paginator->polog, $paginator->nastr);

			$this->result["rows"] = $this->get_elements($result_img);

			$this->result["paginator"] = $links;

			//сохранение кеша
			$this->diafan->_cache->save($this->result, $cache_meta, $this->diafan->module);
		}
		else
		{
			include_once(ABSOLUTE_PATH.'includes/paginator/paginator.view.php');
		}

		$this->result["text"] = DB::query_result("SELECT [text] FROM {news_category} WHERE id=%d", $this->diafan->cat);

		if($this->diafan->configmodules("cat") && ! empty($this->diafan->cat) && $this->diafan->configmodules("comments_cat"))
		{
			$this->result["comments"] = $this->diafan->_comments->get();
		}

		if($this->result["rows"])
		{
			if($this->diafan->configmodules("rating"))
			{
				include_once(ABSOLUTE_PATH.'modules/rating/rating.inc.php');
				$rating = new Rating_inc($this->diafan);
			}
			if($this->diafan->configmodules("tags"))
			{
				include_once(ABSOLUTE_PATH."modules/tags/tags.inc.php");
				$tags = new Tags_inc($this->diafan);
			}

			for($i = 0; $i < count($this->result["rows"]); $i++)
			{
				$this->result["rows"][$i]["tags"] = ($this->diafan->configmodules("tags")
								    ? $tags->get($this->result["rows"][$i]["id"])
								    : '');

				$this->result["rows"][$i]["show-rating"] = ($this->diafan->configmodules("rating")
									   ? $rating->get($this->result["rows"][$i]["id"])
									   : '');
			}
		}
		return $this->result;
	}

	/**
	 * Генерирует данные для страницы новости
	 * 
	 * @return array|boolean false
	 */
	public function id()
	{
		//кеширование
		$cache_meta = array(
				    "name"     => "show",
				    "cat_id"   => $this->diafan->cat,
				    "show"     => $this->diafan->show,
				    "language" => _LANG,
				    "site_id"  => $this->diafan->cid
				    );
		if(! $this->result = $this->diafan->_cache->get($cache_meta, $this->diafan->module))
		{
			$time = mktime(23, 59, 0, date("m"), date("d"), date("Y"));

			$row = DB::fetch_array(DB::query("SELECT id, [name], timeedit, created, cat_id, [keywords], [descr], site_id, [title_meta] FROM"
							 ." {news} WHERE [act]='1' AND id = %d AND trash='0' AND site_id=%d AND created<%d",
							 $this->diafan->show, $this->diafan->cid, $time));
			if(empty($row))
			{
				include(ABSOLUTE_PATH.'includes/404.php');
				return FALSE;
			}

			if(! $this->diafan->rewrite_module && ($this->diafan->configmodules("cat") && $this->diafan->cat != $row["cat_id"]
							       || ! $this->diafan->configmodules("cat") && $this->diafan->cat))
			{
				include(ABSOLUTE_PATH.'includes/404.php');
				return FALSE;
			}
			$this->diafan->cat           = $this->diafan->configmodules("cat") ? $row["cat_id"] : 0;
			$this->result["cat_id"]      = $this->diafan->configmodules("cat") ? $row["cat_id"] : 0;

			$this->result["img"]         = Image::get($row["id"], $row["name"], 'news');

			if($row["cat_id"] && ! empty($this->diafan->rewrites["news"]["cat".$row["cat_id"]]))
			{
				$this->result["allnews"]["link"] = $this->diafan->rewrites["news"]["cat".$row["cat_id"]]["rewrite"].'/';
			}
			else
			{
				$this->result["allnews"]["link"] = (! empty($this->diafan->rewrites[$row["site_id"]]["rewrite"])
								   ? $this->diafan->rewrites[$row["site_id"]]["rewrite"].'/' : '')
				                                  .($row["cat_id"] ? 'cat'.$row["cat_id"].'/' : '');
			}

			$previous = DB::fetch_array(DB::query("SELECT [name], id, cat_id FROM {news} WHERE [act]='1' AND trash='0' AND site_id=%d"
							      .($this->diafan->configmodules("cat") ? " AND cat_id='".$this->diafan->cat."'" : '')
							      ." AND (created<%d OR created=%d AND id<%d) ORDER BY prior DESC, created DESC, id DESC LIMIT 1",
							      $this->diafan->cid, $row["created"], $row["created"], $this->diafan->show));
			if($previous)
			{
				$this->result["previous"]["text"] = $previous["name"];
				if(! empty($this->diafan->rewrites["news"]["element".$previous["id"]]))
				{
					$this->result["previous"]["link"] = $this->diafan->rewrites["news"]["element".$previous["id"]]["rewrite"].'/';
				}
				elseif($previous["cat_id"] && $this->diafan->configmodules("cat") && ! empty($this->diafan->rewrites["news"]["cat".$previous["cat_id"]]))
				{
					$this->result["previous"]["link"] = $this->diafan->rewrites["news"]["cat".$previous["cat_id"]]["rewrite"].'/'
									    .'show'.$previous["id"].'/';
				}
				else
				{
					$this->result["previous"]["link"] = ($this->diafan->rewrite ? $this->diafan->rewrite.'/' : '')
					                                    .($previous["cat_id"] && $this->diafan->configmodules("cat")
									      ? 'cat'.$previous["cat_id"].'/' : '')
									    .'show'.$previous["id"].'/';
				}
			}
			$next = DB::fetch_array(DB::query("SELECT [name], id, cat_id FROM {news} WHERE [act]='1' AND trash='0' AND site_id=%d"
							  .($this->diafan->configmodules("cat") ? " AND cat_id='".$this->diafan->cat."'" : '')
							  ." AND created<%d AND (created>%d OR created=%d AND id>%d)"
							  ." ORDER BY prior ASC, created ASC, id ASC LIMIT 1",
							  $this->diafan->cid, $time, $row["created"], $row["created"], $this->diafan->show));
			if($next)
			{
				$this->result["next"]["text"] = $next["name"];
				if(! empty($this->diafan->rewrites["news"]["element".$next["id"]]))
				{
					$this->result["next"]["link"] = $this->diafan->rewrites["news"]["element".$next["id"]]["rewrite"].'/';
				}
				elseif($next["cat_id"] && $this->diafan->configmodules("cat") && ! empty($this->diafan->rewrites["news"]["cat".$next["cat_id"]]))
				{
					$this->result["next"]["link"] = $this->diafan->rewrites["news"]["cat".$next["cat_id"]]["rewrite"].'/'
					                                .'show'.$next["id"].'/';
				}
				else
				{
					$this->result["next"]["link"] = ($this->diafan->rewrite ? $this->diafan->rewrite.'/' : '')
					                                .($next["cat_id"] && $this->diafan->configmodules("cat")
									  ? 'cat'.$next["cat_id"].'/' : '')
									.'show'.$next["id"].'/';
				}
			}

			$this->result["allnews"]["text"] = $this->diafan->configmodules("back-to-list");
			$this->result["date"]            = $this->format_date($row['created']);

			$this->result["titlemodule"]     = $row["name"];
			$this->result["title_meta"]      = $row["title_meta"];
			$this->result["timeedit"]        = $row["timeedit"];
				
			$this->result["keywords"]        = $row["keywords"];
			$this->result["descr"]           = $row["descr"];
			
			$this->result["path"]            = $this->get_path('news');

			//сохранение кеша
			$this->diafan->_cache->save($this->result, $cache_meta, $this->diafan->module);
		}
		$this->diafan->cat = $this->result["cat_id"];

		$row = DB::fetch_array(DB::query("SELECT [text], [anons] FROM {news} WHERE id=%d", $this->diafan->show));

		$this->result["text"]  = $row["text"];
		$this->result["anons"] = $row["anons"];

		if($this->diafan->configmodules("rating"))
		{
			include_once(ABSOLUTE_PATH.'modules/rating/rating.inc.php');
			$rating = new Rating_inc($this->diafan);
			$this->result["show-rating"] = $rating->get();
			$this->result["add-rating"] = $rating->add();			
		}

		if($this->diafan->configmodules("comments"))
		{
			$this->result["comments"] = $this->diafan->_comments->get();
		}

		if($this->diafan->configmodules("tags"))
		{
			include_once(ABSOLUTE_PATH."modules/tags/tags.inc.php");
			$tags = new Tags_inc($this->diafan);
			$this->result["tags"] = $tags->get();
		}

		return $this->result;
	}

	/**
	 * Генерирует данные для шаблонной функции: блок новостей
	 * 
	 * @param integer $count количество новостей
	 * @param integer $cat_id категория
	 * @param integer $photo количество фотографий
	 * @param integer $site_id страница сайта
	 * @return array
	 */
	public function show_block($count, $cat_id, $photo, $site_id)
	{
		$time = mktime(23, 59, 0, date("m"), date("d"), date("Y"));

		//кеширование
		$cache_meta = array(
				    "name"     => "block",
				    "cat_id"   => $cat_id,
				    "count"    => $count,
				    "language" => _LANG,
				    "time"     => $time,
				    "site_id"  => $site_id
				    );

		if(! $this->result = $this->diafan->_cache->get($cache_meta, "news"))
		{
			if(! $site_id)
			{
				$site_id = DB::query_result("SELECT id FROM {site} WHERE addmodule='news' AND trash='0' AND act='1'");
			}
			if($cat_id && $this->diafan->configmodules("cat", "news"))
			{
				if($this->diafan->configmodules("childrens_elements", "news"))
				{ 
					$children = $this->diafan->get_children($cat_id, "news_category");
				}
			}
			$this->result["rows"] = array();
			$result = DB::query_range("SELECT e.id, e.[name],e.[anons], e.cat_id, e.timeedit, e.site_id, e.created FROM {news} AS e"
						  .($cat_id && $this->diafan->configmodules("cat", "news")
						    ? " INNER JOIN {category} as c ON c.module_name='news' AND c.element_id=e.id"
						      .(! empty($children) ? ' AND c.cat_id IN ('.implode(',', $children).')':' AND c.cat_id='.$cat_id)
						    : ''
						    )
						  ." WHERE e.[act]='1' AND e.trash='0' AND e.created<%d"
						  .($site_id
						    ? " AND e.site_id=".$site_id
						    : ''
						    )
						  ." GROUP BY e.id"
						  ." ORDER BY e.prior DESC, e.created DESC, e.id DESC",
						  $time, 0, $count);
			$this->result["rows"] = $this->get_elements($result, $photo);

			if(! empty($this->result["rows"]) && $cat_id && $this->diafan->configmodules("cat","news"))
			{
				$cat = DB::fetch_array(DB::query("SELECT [name], site_id, id FROM {news_category} WHERE id=%d", $cat_id));

				$this->result["name"]            = $cat["name"];
				$this->result["allnews"]["link"] = $this->get_link_category($cat, 'news');
				$this->result["allnews"]["text"] = _LANG_ALL_NEWS.' '._LANG_IN_CATEGORY.' '._LANG_LQUOT.$cat["name"]._LANG_RQUOT;
			}
			elseif(! empty($this->result["rows"]) && $site_id)
			{
				$this->result["name"] = DB::query_result("SELECT [name] FROM {site} WHERE id=%d", $site_id);
				$this->result["allnews"]["link"] = ! empty($this->diafan->rewrites[$site_id]["rewrite"]) ? $this->diafan->rewrites[$site_id]["rewrite"].'/' : '';
				$this->result["allnews"]["text"] = _LANG_ALL_NEWS;
			}
			$this->diafan->_cache->save($this->result, $cache_meta, "news");
		}

		return $this->result;
	}

	/**
	 * Генерирует данные для шаблонной функции: календарь архива новостей
	 * 
	 * @return array
	 */
	function show_calendar($month_hidden)
	{
		if(empty($this->diafan->year))
		{
			$year_current = date("Y");
		}
		else
		{
			$year_current = $this->diafan->year;
		}

		//кеширование
		$cache_meta = array(
				    "name"           => "calendar",
				    "month_hidden"   => $month_hidden,
				    "language"       => _LANG,
				    "site_id"        => $this->diafan->cid
				    );
		if($result_cache = $this->diafan->_cache->get($cache_meta, "news"))
		{
			$result_cache["year"] = $year_current;
			return $result_cache;
		}

		$d = DB::where("news", "[act]='1' ORDER BY created ASC", "created");
		$nyear = 2008;

		if($d > 0)
		{
			$nyear = date("Y", $d);
		}

		if(date("Y")-$nyear > 10)
		{
			$nyear = 2008;
		}

		for($Ye = date("Y"); $Ye >= $nyear; $Ye--)
		{
			$result["rows"][$Ye]["year"]["link"] = $this->diafan->rewrite.'/year'.$Ye.'/';
			$result["rows"][$Ye]["year"]["name"] = $Ye;

			$month_arr = array(
				'12' => _LANG_DECEMBER,
				'11' => _LANG_NOVEMBER,
				'10' => _LANG_OCTOBER,
				'09' => _LANG_SEPTEMBER,
				'08' => _LANG_AUGUST,
				'07' => _LANG_JULY,
				'06' => _LANG_JUNE,
				'05' => _LANG_MAY,
				'04' => _LANG_APRIL,
				'03' => _LANG_MARCH,
				'02' => _LANG_FEBRUARY,
				'01' => _LANG_JANUARY
			);

			if(! $month_hidden)
			{
				for($num = 12; $num > 0; $num--)
				{
					if($Ye != date("Y") || $num <= date("m") )
					{
						if($num < 10)
						{
							$num = '0'.$num;
						}

						$val = $month_arr[$num];
						$result["rows"][$Ye]["months"][] = array(
											 "link" =>
											 DB::query_result("SELECT COUNT(id) FROM {news}"
											  ." WHERE [act]='1' "
											  ." AND trash='0'"
											  ." AND created>%d"
											  ." AND created<=%d"
											  ." AND site_id=%d",
											  mktime(0, 0, 0, $num, 1, $Ye),
											  mktime(0, 0, 0, $num, 31, $Ye),
											  $this->diafan->cid
											  )
											  ? $this->diafan->rewrite.'/year'.$Ye.'/month'.$num.'/' : '',
											  "name" => $val
											  );
					}
				}
			}
		}

		$result["link"] = $this->diafan->rewrite.'/';

		//сохранение кеша
		$this->diafan->_cache->save($result, $cache_meta, "news");

		$result["year"] = $year_current;
		return $result;	
	}

	/**
	 * Форматирует данные о новостях для списка новостей
	 *
	 * @param resource $result результат выполнения SQL-запроса
	 * @param string $photo показывать фотографии
	 * @return array
	 */
	private function get_elements($result, $photo = 'list')
	{
		if(empty($this->result["timeedit"]))
		{
			$this->result["timeedit"] = '';
		}
		$rows = array();
		while ($row = DB::fetch_array($result))
		{
			if($row["timeedit"] < $this->result["timeedit"])
			{
				$this->result["timeedit"] = $row["timeedit"];
			}
			unset($row["timeedit"]);

			if(! empty($this->diafan->rewrites["news"]["element".$row["id"]]))
			{
				$row["link"] = $this->diafan->rewrites["news"]["element".$row["id"]]["rewrite"].'/';
			}
			elseif($row["cat_id"] && $this->diafan->configmodules("cat", "news", '', $row["site_id"]) && ! empty($this->diafan->rewrites["news"]["cat".$row["cat_id"]]))
			{
				$row["link"] = $this->diafan->rewrites["news"]["cat".$row["cat_id"]]["rewrite"].'/show'.$row["id"].'/';
			}
			else
			{
				$row["link"] = (! empty($this->diafan->rewrites[$row["site_id"]]["rewrite"]) ? $this->diafan->rewrites[$row["site_id"]]["rewrite"].'/' : '')
				               .($row["cat_id"] && $this->diafan->configmodules("cat", "news", '', $row["site_id"])? 'cat'.$row["cat_id"].'/' : '')
					       .'show'.$row["id"].'/';
			}
			unset($row["cat_id"]);

			if($photo === 'list' && $this->diafan->configmodules("list_img", "news"))
			{
				$row["img"]  = Image::get($row["id"], $row["name"], 'news',
							  $this->diafan->configmodules("list_img") == 1 ? 1 : 0,
							  $row["link"]);
			}
			elseif($photo != 'list' && $photo)
			{
				$row["img"]  = Image::get($row["id"], $row["name"], 'news', $photo, $row["link"]);	
			}
			$row["date"] = $this->format_date($row['created'], "news", $row["site_id"]);
			unset($row["created"]);

			$rows[] = $row;	
		}
		return $rows;
	}
}