<?php
/**
 * Diafan.CMS
 *
 * @author     diafan.ru
 * @version    4.3
 * @license http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2011 OOO "Диафан". (http://diafan.ru)
 */

if(! defined('DIAFAN'))
{
	include(dirname(dirname(dirname(__FILE__))).'/includes/404.php');
}

/**
 * Order_model
 *
 * Модель модуля "Оформление заказа"
 */
class Order_model extends Model
{
	/**
	 * @var array сгенерированные в моделе данные, передаваемые в шаблон
	 */
	protected $result;

	/**
	 * @var object основной объект системы
	 */
	protected $diafan;

	/**
	 * Конструктор класса
	 * 
	 * @return void
	 */
	public function __construct($diafan)
	{
		$this->diafan = $diafan;
	}

	/**
	 * Генерирует данные для первого шага в офрмлении заказа
	 * 
	 * @return array|boolean
	 */
	public function show_step_1()
	{
		if($this->diafan->configmodules('security_user', 'shop') && ! $this->diafan->_user->id)
		{
			$rewrite = $this->get_rewrite_from_module("basket");
			$this->diafan->goto_diafan($rewrite.'/');
			return FALSE;
		}

		if(! DB::query_result("SELECT id FROM {site} WHERE addmodule='registration' AND [act]='1' AND trash='0'"))
		{
			return $this->show_step_2();
		}
		if(empty($_SESSION["basket"]) || !is_array($_SESSION["basket"]))
		{
			$rewrite = $this->get_rewrite_from_module("basket");
			$this->diafan->goto_diafan($rewrite.'/');
			return FALSE;
		}

		$model = new Basket_model($this->diafan);
		$this->result = $model->form();

		$this->result["to-step-2"] = BASE_PATH_HREF.$this->diafan->rewrite.'/step2/';

		$this->result["registration"] = array("fio" => '', "mail" => '', "name" => '', "captcha" => '', "user" => '');

		$rewrite_reg = $this->get_rewrite_from_module("registration");
		$this->result["registration"]["action"]         = BASE_PATH_HREF.$rewrite_reg.'/';
		$this->result["registration"]["error"]          = $this->get_error("registration");
		$this->result["registration"]["error-name"]     = $this->get_error("registration", 'name');
		$this->result["registration"]["error-fio"]      = $this->get_error("registration", 'fio');
		$this->result["registration"]["error-password"] = $this->get_error("registration", 'password');
		$this->result["registration"]["error-mail"]     = $this->get_error("registration", 'mail');
		$this->result["registration"]["user"]           = $this->diafan->_user->id;
		$this->result["registration"]["url"] 	        = $this->result["to-step-2"];

		if($this->diafan->configmodules('security_code', "usersite"))
		{
			require_once (ABSOLUTE_PATH.'includes/captcha/captcha.inc.php');
			$this->result["registration"]["captcha"] = Captcha_inc::get("registration", $this->get_error("registration", "code"));
		}
		$this->result["show_login"] = array("error" => ($this->diafan->_user->errauth ? _LANG_WRONG_LOGIN_OR_PASS : ''), "action" => '', "user" => '');

		return $this->result;
	}

	/**
	 * Генерирует данные для второго шага в офрмлении заказа
	 * 
	 * @return array|boolean
	 */
	public function show_step_2()
	{
		if($this->diafan->configmodules('security_user', 'shop') && ! $this->diafan->_user->id)
		{
			$rewrite = $this->get_rewrite_from_module("basket");
			$this->diafan->goto_diafan($rewrite.'/');
			return FALSE;
		}

		if(empty($_SESSION["basket"]) || ! is_array($_SESSION["basket"]))
		{
			$rewrite = $this->get_rewrite_from_module("basket");
			$this->diafan->goto_diafan($rewrite.'/');
			return FALSE;
		}
		
		$model = new Basket_model($this->diafan);
		$this->result = $model->form();

		$this->result["user"]   = 0;
		$this->result["phone"]  = '';
		$this->result["city"]   = '';
		$this->result["street"] = '';
		$this->result["home"]   = '';
		$this->result["corps"]  = '';
		$this->result["flat"]   = '';

		if($this->diafan->_user->id)
		{
			$this->result["user"] = 1;
			if($this->diafan->_user->address)
			{
				$conf_arr = explode("\n", $this->diafan->_user->address);
				foreach($conf_arr as $c_a)
				{
					list($k, $v) = explode("=", $c_a, 2);
					$this->result[$k] = $v;
				}
			}
		}

		$this->result["text"] = DB::where("info", "tag_name='shop_order_step_2'", "text"._LANG);
		$this->result["form"] = ! $this->diafan->configmodules("pay_systems", "shop") ? ' id="order" onsubmit="return false;"' : '';

		$this->result["error"]           = $this->get_error("order");
		$this->result["error-phone"]     = $this->get_error("order", 'phone');
		$this->result["error-fio"]       = $this->get_error("order", 'fio');
		$this->result["error-mail"]      = $this->get_error("order", 'mail');

		return $this->result;
	}

	/**
	 * Генерирует данные для третьего шага в офрмлении заказа
	 * 
	 * @return array|boolean
	 */
	public function show_step_3()
	{
		if($this->diafan->configmodules('security_user', 'shop') && ! $this->diafan->_user->id)
		{
			$rewrite = $this->get_rewrite_from_module("basket");
			$this->diafan->goto_diafan($rewrite.'/');
			return FALSE;
		}

		if(empty($_SESSION["order"]) || ! is_array($_SESSION["order"]) || empty($_SESSION["order-pay"]))
		{
			$rewrite = $this->get_rewrite_from_module("basket");
			$this->diafan->goto_diafan($rewrite.'/');
			return FALSE;
		}

		$this->result["webmoney"] = $this->diafan->configmodules("webmoney", "shop") ? 1 : 0;
		$this->result["robox"]    = $this->diafan->configmodules("robox", "shop") ? 1 : 0;
		$this->result["non_cash"] = $this->diafan->configmodules("non_cash", "shop") ? 1 : 0;
		$this->result["courier"]  = $this->diafan->configmodules("courier", "shop") ? 1 : 0;

		return $this->result;
	}

	/**
	 * Генерирует данные для четвертого шага в офрмлении заказа
	 * 
	 * @return array|boolean
	 */
	public function show_step_4()
	{
		if($this->diafan->configmodules('security_user', 'shop') && ! $this->diafan->_user->id)
		{
			$rewrite = $this->get_rewrite_from_module("basket");
			$this->diafan->goto_diafan($rewrite.'/');
			return FALSE;
		}

		$row = DB::fetch_array(DB::query("SELECT payment, summ FROM {shop_order} WHERE id=%d", $this->diafan->show));
		if($row["payment"] == 1 && $this->diafan->configmodules("webmoney", "shop"))
		{
			$this->webmoney($row["summ"]);
		}
		elseif($row["payment"] == 2 && $this->diafan->configmodules("robox", "shop"))
		{
			$this->roboxchange($row["summ"]);
		}
		elseif($row["payment"] == 3 && $this->diafan->configmodules("non_cash", "shop"))
		{
			$this->non_cash();
		}
		else
		{
			include(ABSOLUTE_PATH.'includes/404.php');
		}
		return $this->result;
	}

	/**
	 * Генерирует данные для пятого шага в офрмлении заказа
	 * 
	 * @return array
	 */
	public function show_step_5()
	{
		if($this->diafan->step == 5)
		{
			return DB::where("info", "tag_name='shop_payment_success'", "text"._LANG);
		}
		else
		{
			return DB::where("info", "tag_name='shop_payment_fail'", "text"._LANG);
		}
	}

	/**
	 * Платежная система WebMoney
	 *
	 * @param string $summ сумма заказа
	 * @return boolean true
	 */
	private function webmoney($summ)
	{
		$this->result["type"]      = "webmoney";
		$this->result["desc"]      = str_replace('%order', $this->diafan->show, $this->diafan->configmodules("desc_order", "shop"));
		$this->result["wm_target"] = WM_TARGET;
		
		$this->result["summ"]      = $summ;
		$this->result["order_id"]  = $this->diafan->show;

		$this->result["rnd"]       = strtoupper(substr(md5(uniqid(microtime(), 1)).getmypid(), 1, 8));

		// режим тестирования:
		//  0 или не отсутствует: Для всех тестовых платежей сервис будет имитировать успешное выполнение;
		//  1: Для всех тестовых платежей сервис будет имитировать выполнение с ошибкой (платеж не выполнен);
		//  2: Около 80% запросов на платеж будут выполнены успешно, а 20% - не выполнены.
		// $result["LMI_SIM_MODE"]=2;

		$this->result["text"] = DB::where("info", "tag_name='shop_wm'", "text"._LANG);

		DB::query("INSERT INTO {payment} (order_id, state, RND, timestamp) VALUES('%d', 'I', '%s', CURRENT_TIMESTAMP())",
		          $this->diafan->show, $this->result["rnd"]);
		return TRUE;
	}

	/**
	 * Платежная система Roboxchange
	 * 
	 * @param string $summ сумма заказа
	 * @return boolean true
	 */
	private function roboxchange($summ)
	{
		$this->result["type"]      = "roboxchange";
		// регистрационная информация 
		$this->result["login"]    = ROBOX_LOGIN; //логин 
		$mrh_pass1                = ROBOX_PASS_1; //пароль1
		$this->result["summ"]     = $summ;
		$this->result["order_id"] = $this->diafan->show;

		//описание покупки
		$this->result["desc"]     = str_replace('%order', $this->diafan->show, $this->diafan->configmodules("desc_order", "shop"));
		$this->result["desc"]     = $this->diafan->translit($this->result["desc"]);

		$this->result["shp_item"] = 1; // тип товара

		//формирование подписи
		$this->result["crc"]      = md5($this->result["login"].":".$summ.":".$this->diafan->show.":".$mrh_pass1.":shp_item=".$this->result["shp_item"]);

		$this->result["text"]     = DB::where("info", "tag_name='shop_robox'", "text"._LANG);

		return TRUE;
	}

	/**
	 * Способ оплаты: Банковский платеж
	 * 
	 * @return string
	 */
	private function non_cash()
	{
		$this->result["type"]     = "non_cash";
		$this->result["text"]     = DB::where("info", "tag_name='shop_non_cash'", "text"._LANG);
		$this->result["order_id"] = $this->diafan->show;
		return TRUE;
	}
}