<?php
/**
 * Diafan.CMS
 *
 * @author     diafan.ru
 * @version    4.3
 * @license http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2011 OOO "Диафан". (http://diafan.ru)
 */

if(! defined('DIAFAN'))
{
	include(dirname(dirname(dirname(__FILE__))).'/includes/404.php');
}

/**
 * Shop_model
 *
 * Модель модуля "Магазин"
 */
class Shop_model extends Model
{
	/**
	 * @var array сгенерированные в моделе данные, передаваемые в шаблон
	 */
	protected $result;

	/**
	 * @var object основной объект системы
	 */
	protected $diafan;

	/**
	 * @var array значения параметров, влияющих на цену
	 */
	private $depends;

	/**
	 * @var array направления сортировки товаров
	 */
	private $sort = array(
				1 => 'p.price ASC',
				2 => 'p.price DESC',
				3 => 's.name ASC',
				4 => 's.name DESC'
			    );

	/**
	 * Конструктор класса
	 * 
	 * @return void
	 */
	public function __construct(&$diafan)
	{
		$this->diafan = &$diafan;
		if($this->diafan->sort > count($this->sort))
		{
			$this->diafan->goto_diafan($this->diafan->path_module_variable("sort"));
		}
		include_once(ABSOLUTE_PATH."includes/image.php");
	}

	/**
	 * Генерирует данные для списка товаров, если отключена группировка товаров
	 * 
	 * @return array
	 */
	public function list_()
	{
		if($this->diafan->cat)
		{
			$this->diafan->goto_diafan($this->diafan->path_module_variable("cat"));
		}
		$cache_meta = array(
				    "name"     => "list",
				    "language" => _LANG,
				    "page"     => $this->diafan->page > 1 ? $this->diafan->page : 1,
				    "site_id"  => $this->diafan->cid,
				    "sort"     => $this->diafan->sort
				    );
		//кеширование
		if(! $this->result = $this->diafan->_cache->get($cache_meta, $this->diafan->module))
		{
			$this->result = array();

			////navigation//
			include_once(ABSOLUTE_PATH.'includes/paginator/paginator.inc.php');
			$paginator = new Paginator_inc();
			$paginator->page    = $this->diafan->page;
			$paginator->navlink = $this->diafan->path_module_variable("page");
			$paginator->nen = DB::query_result("SELECT COUNT(id) FROM {shop} WHERE [act]='1' AND trash='0' AND site_id=%d", $this->diafan->cid);
			$links = $paginator->get();
			////navigation///

			$result = DB::query_range("SELECT s.id, s.[name], s.timeedit, s.[anons], s.site_id FROM {shop} AS s"
						  .($this->diafan->sort == 1 || $this->diafan->sort == 2 ? " INNER JOIN {shop_price} AS p ON p.shop_id=s.id" : '')
						  ." WHERE s.[act]='1' AND s.trash='0' AND s.site_id=%d"
						  ." ORDER BY ".($this->diafan->sort ? $this->sort[$this->diafan->sort].',' : '')
						  ." s.sort ASC, s.id ASC", $this->diafan->cid, $paginator->polog, $paginator->nastr);
			
			$this->result["rows"] = $this->get_elements($result);

			$this->result["paginator"] = $links;

			//сохранение кеша
			$this->diafan->_cache->save($this->result, $cache_meta, $this->diafan->module);
		}
		else
		{
			include_once(ABSOLUTE_PATH.'includes/paginator/paginator.view.php');
		}
		$this->result["buy"] = ! $this->diafan->configmodules('security_user', "shop") || $this->diafan->_user->id ? TRUE : FALSE;

		if($this->result["rows"])
		{
			if($this->diafan->configmodules("rating"))
			{
				include_once(ABSOLUTE_PATH.'modules/rating/rating.inc.php');
				$rating = new Rating_inc($this->diafan);
			}
			if($this->diafan->configmodules("tags"))
			{
				include_once(ABSOLUTE_PATH."modules/tags/tags.inc.php");
				$tags = new Tags_inc($this->diafan);
			}

			for($i = 0; $i < count($this->result["rows"]); $i++)
			{
				$this->result["rows"][$i]["tags"] = $this->diafan->configmodules("tags")
								    ? $tags->get($this->result["rows"][$i]["id"])
								    : '';

				$this->result["rows"][$i]["show-rating"] = $this->diafan->configmodules("rating")
									   ? $rating->get($this->result["rows"][$i]["id"])
									   : '';
				$this->result["rows"][$i]["error"] = $this->get_error("shop".$this->result["rows"][$i]["id"]);
				$this->result["rows"][$i] = $this->select_price($this->result["rows"][$i]);
			}
		}
		foreach($this->sort as $key => $value)
		{
			$this->result["link_start"][$key] = $this->diafan->sort != $key
							    ? '<a href="'.BASE_PATH_HREF.$this->diafan->path_module_variable("sort").'sort'.$key.'/">'
							    : '<span class="active">';
			$this->result["link_finish"][$key] = $this->diafan->sort != $key
							     ? '</a>'
							     : '</span>';
		}
		return $this->result;
	}

	/**
	 * Генерирует данные для списка товаров, найденных при помощи поиска
	 * 
	 * @return array
	 */
	public function list_search()
	{
		$this->result = array();

		list($where, $where_param, $values, $getnav) = $this->get_where();

		////navigation//
		include_once(ABSOLUTE_PATH.'includes/paginator/paginator.inc.php');
		$paginator = new Paginator_inc();
		$paginator->page    = $this->diafan->page;
		$paginator->get_nav = $getnav;
		$paginator->navlink = $this->diafan->path_module_variable("page");

		$paginator->nen = DB::query_result("SELECT COUNT(DISTINCT s.id) FROM {shop} as s ".$where_param
		                                   ." WHERE s.[act]='1' AND s.trash='0' ".$where, $values);

		$links = $paginator->get();
		////navigation///

		$this->result["path"] = $this->get_path('shop');
		$this->result["titlemodule"] = _LANG_SEARCH_GOODS;
		if(! $paginator->nen)
		{
			$this->result["err"] = _LANG_IT_IS_NOT_FOUND;
			return $this->result;
		}

		$result = DB::query_range("SELECT s.id, s.[name], s.timeedit, s.[anons], s.cat_id, s.site_id FROM {shop} AS s"
					  .($this->diafan->sort == 1 || $this->diafan->sort == 2 ? " INNER JOIN {shop_price} AS p ON p.shop_id=s.id" : '')
					  .$where_param
					  ." WHERE s.[act]='1' AND s.trash='0'".$where
					  ." GROUP BY s.id"
					  ." ORDER BY "
					  .($this->diafan->sort ? $this->sort[$this->diafan->sort].',' : '')
					  ." sort ASC, id ASC", $values, $paginator->polog, $paginator->nastr);

		$this->result["rows"] = $this->get_elements($result);
		$this->result["buy"]  = ! $this->diafan->configmodules('security_user', "shop") || $this->diafan->_user->id ? TRUE : FALSE;

		$this->result["paginator"] = $links;

		if($this->result["rows"])
		{
			if($this->diafan->configmodules("rating"))
			{
				include_once(ABSOLUTE_PATH.'modules/rating/rating.inc.php');
				$rating = new Rating_inc($this->diafan);
			}
			if($this->diafan->configmodules("tags"))
			{
				include_once(ABSOLUTE_PATH."modules/tags/tags.inc.php");
				$tags = new Tags_inc($this->diafan);
			}

			for($i = 0; $i < count($this->result["rows"]); $i++)
			{
				$this->result["rows"][$i]["tags"] = $this->diafan->configmodules("tags")
								    ? $tags->get($this->result["rows"][$i]["id"])
								    : '';

				$this->result["rows"][$i]["show-rating"] = $this->diafan->configmodules("rating")
									   ? $rating->get($this->result["rows"][$i]["id"])
									   : '';
				$this->result["rows"][$i]["error"] = $this->get_error("shop".$this->result["rows"][$i]["id"]);
				$this->result["rows"][$i] = $this->select_price($this->result["rows"][$i]);
			}
		}

		foreach($this->sort as $key => $value)
		{
			$this->result["link_start"][$key] = $this->diafan->sort != $key
							    ? '<a href="'.BASE_PATH_HREF.$this->diafan->path_module_variable("sort").'sort'.$key.'/'.$getnav.'">'
							    : '<span class="active">';
			$this->result["link_finish"][$key] = $this->diafan->sort != $key
							     ? '</a>'
							     : '</span>';
		}
		return $this->result;
	}

	/**
	 * Генерирует данные для первой страницы магазина
	 * 
	 * @return array
	 */
	public function first_page()
	{
		if($this->diafan->page)
		{
			$this->diafan->goto_diafan($this->diafan->path_module_variable("page"));
		}
		//кеширование
		$cache_meta = array(
				    "name"     => "first_page",
				    "language" => _LANG,
				    "site_id"  => $this->diafan->cid
				    );
		if(! $this->result = $this->diafan->_cache->get($cache_meta, $this->diafan->module))
		{
			$this->result = array();

			$result = DB::query("SELECT id, [name], [anons], timeedit, site_id FROM {shop_category}"
					    ." WHERE [act]='1' AND parent_id=0 AND trash='0' AND site_id=%d"
					    ." ORDER by sort ASC, id ASC", $this->diafan->cid);

			while ($row = DB::fetch_array($result))
			{
				$this->result["categories"][$row["id"]]["name"] = $row["name"];
				$this->result["categories"][$row["id"]]["anons"] = $row["anons"];

				if(empty($this->result["timeedit"]) || $row["timeedit"] > $this->result["timeedit"])
				{
					$this->result["timeedit"] = $row["timeedit"];
				}

				$this->result["categories"][$row["id"]]["children"] = array();
				$result_children = DB::query("SELECT id, [name], [anons], site_id FROM {shop_category}"
							     ." WHERE [act]='1' AND parent_id=%d AND trash='0' "
							     ." AND site_id=%d ORDER BY sort ASC, id ASC",
							     $row["id"], $this->diafan->cid);

				while ($row_children = DB::fetch_array($result_children))
				{
					$child["link"]  = $this->get_link_category($row_children, 'shop');
					if($this->diafan->configmodules("list_img_cat"))
					{
						$child["img"] = Image::get($row_children["id"], $row_children["name"], 'shop',
									   $this->diafan->configmodules("list_img_cat") == 1 ? 1 : 0,
									   $child["link"], TRUE);
					}
					$child["name"]  = $row_children["name"];
					$child["anons"] = $row_children["anons"];
					$this->result["categories"][$row["id"]]["children"][] = $child;
				}

				if($this->diafan->configmodules("childrens_elements"))
				{
					$children = $this->diafan->get_children($row["id"],"shop_category");
				}

				$result_img = DB::query_range("SELECT e.id, e.[name], e.cat_id, e.timeedit, e.[anons], e.site_id FROM {shop} AS e"
							      ." INNER JOIN {category} AS c ON e.id=c.element_id"
							      ." WHERE c.module_name='shop' AND e.cat_id IN (%s) AND e.[act]='1'"
							      ." AND e.trash='0' AND e.site_id=%d GROUP BY e.id ORDER BY e.sort ASC, e.id ASC",
							      $this->diafan->configmodules("childrens_elements") ? implode(',', $children) : $row["id"],
							      $this->diafan->cid,
							      0, $this->diafan->configmodules("count_list")
							      );
				$this->result["categories"][$row["id"]]["rows"] = $this->get_elements($result_img);

				$this->result["categories"][$row["id"]]["allshop"]["link"] = $this->get_link_category($row, 'shop');

				if($this->diafan->configmodules("list_img_cat"))
				{
					$this->result["categories"][$row["id"]]["img"] = Image::get($row["id"], $row["name"], 'shop',
												    $this->diafan->configmodules("list_img_cat") == 1 ? 1 : 0,
												    $this->result["categories"][$row["id"]]["allshop"]["link"],
												    TRUE);
				}
				$this->result["categories"][$row["id"]]["allshop"]["text"] = $this->diafan->configmodules("allshop").' '._LANG_LQUOT.$row["name"]._LANG_RQUOT;
			}

			//сохранение кеша
			$this->diafan->_cache->save($this->result, $cache_meta, $this->diafan->module);
		}
		$this->result["buy"] = ! $this->diafan->configmodules('security_user', "shop") || $this->diafan->_user->id ? TRUE : FALSE;

		if(! empty($this->result["categories"]))
		{
			if($this->diafan->configmodules("rating"))
			{
				include_once(ABSOLUTE_PATH.'modules/rating/rating.inc.php');
				$rating = new Rating_inc($this->diafan);
			}
			if($this->diafan->configmodules("tags"))
			{
				include_once(ABSOLUTE_PATH."modules/tags/tags.inc.php");
				$tags = new Tags_inc($this->diafan);
			}

			foreach($this->result["categories"] as $cat => $rows)
			{
				if(! $rows || ! $rows["rows"])
					continue;

				foreach($rows["rows"] as $k => $row)
				{
					$this->result["categories"][$cat]["rows"][$k]["tags"] = $this->diafan->configmodules("tags")
									   ? $tags->get($this->result["categories"][$cat]["rows"][$k]["id"])
									   : '';

					$this->result["categories"][$cat]["rows"][$k]["show-rating"] = $this->diafan->configmodules("rating")
									  ? $rating->get($this->result["categories"][$cat]["rows"][$k]["id"])
									  : '';
					$this->result["categories"][$cat]["rows"][$k]["error"] = $this->get_error("shop".$rows["rows"][$k]["id"]);
					$this->result["categories"][$cat]["rows"][$k] = $this->select_price($this->result["categories"][$cat]["rows"][$k]);
				}
			}
		}
		return $this->result;
	}

	/**
	 * Генерирует данные для списка товаров в группе
	 * 
	 * @return array
	 */
	public function list_category()
	{
		//кеширование
		$cache_meta = array(
				    "name"     => "list",
				    "cat_id"   => $this->diafan->cat,
				    "language" => _LANG,
				    "page"     => $this->diafan->page > 1 ? $this->diafan->page : 1,
				    "site_id"  => $this->diafan->cid,
				    "sort"     => $this->diafan->sort
				    );
		if(! $this->result = $this->diafan->_cache->get($cache_meta, $this->diafan->module))
		{
			$this->result = array();

			$this->result["level"] = count($this->diafan->get_parents($this->diafan->cat, 'shop_category'));

			$this->result["path"] = $this->get_path('shop');

			$result = DB::query("SELECT id, [name], timeedit, [descr], [keywords], sort, parent_id, [title_meta] FROM {shop_category}"
					    ." WHERE [act]='1' and id=%d AND trash='0' AND site_id=%d"
					    ." ORDER BY sort ASC, id ASC",
					    $this->diafan->cat, $this->diafan->cid);

			if(! $row = DB::fetch_array($result))
			{
				include_once(ABSOLUTE_PATH.'includes/404.php');
			}

			$this->result["img"] = Image::get($row["id"], $row["name"], 'shop', 0, 1, TRUE);

			if($this->diafan->configmodules("childrens_elements"))
			{
				$children = $this->diafan->get_children($this->diafan->cat, "shop_category");
			}

			////navigation//
			include_once(ABSOLUTE_PATH.'includes/paginator/paginator.inc.php');
			$paginator = new Paginator_inc();
			$paginator->page    = $this->diafan->page;
			$paginator->navlink = $this->diafan->path_module_variable("page");
			$paginator->nen = DB::query_result("SELECT COUNT(DISTINCT e.id) FROM {shop} as e"
							   ." INNER JOIN {category} AS c ON e.id=c.element_id AND c.module_name='shop'"
							   ." AND e.id=c.element_id AND c.cat_id IN (%s)"
							   ." WHERE e.[act]='1' AND e.trash='0'",

							   $this->diafan->configmodules("childrens_elements")
							   ? implode(',', $children) : $this->diafan->cat
							   );
			$links = $paginator->get();
			////navigation///

			$this->result["titlemodule"] = $row["name"];
			$this->result["title_meta"]  = $row["title_meta"];
			$this->result["timeedit"]    = $row["timeedit"];
			$this->result["keywords"]    = $row["keywords"];
			$this->result["descr"]       = $row["descr"];

			$previous = DB::fetch_array(DB::query("SELECT [name], id FROM {shop_category} WHERE [act]='1' AND trash='0' AND site_id=%d"
							      ." AND (sort<%d OR sort=%d AND id<%d) AND parent_id=%d ORDER BY sort DESC, id DESC LIMIT 1",
							      $this->diafan->cid, $row["sort"], $row["sort"], $this->diafan->cat, $row["parent_id"]));
			if($previous)
			{
				$this->result["previous"]["text"] = $previous["name"];
				if(! empty($this->diafan->rewrites["shop"]["cat".$previous["id"]]))
				{
					$this->result["previous"]["link"] = $this->diafan->rewrites["shop"]["cat".$previous["id"]]["rewrite"].'/';
				}
				else
				{
					$this->result["previous"]["link"] = ($this->diafan->rewrite ? $this->diafan->rewrite.'/' : '')
					                                    .'cat'.$previous["id"].'/';
				}
			}
			$next = DB::fetch_array(DB::query("SELECT [name], id FROM {shop_category} WHERE [act]='1' AND trash='0' AND site_id=%d"
							  ." AND (sort>%d OR sort=%d AND id>%d) AND parent_id=%d ORDER BY sort ASC, id ASC LIMIT 1",
							  $this->diafan->cid, $row["sort"], $row["sort"], $this->diafan->cat, $row["parent_id"]));
			if($next)
			{
				$this->result["next"]["text"] = $next["name"];
				if(! empty($this->diafan->rewrites["shop"]["cat".$next["id"]]))
				{
					$this->result["next"]["link"] = $this->diafan->rewrites["shop"]["cat".$next["id"]]["rewrite"].'/';
				}
				else
				{
					$this->result["next"]["link"] = ($this->diafan->rewrite ? $this->diafan->rewrite.'/' : '')
									.'cat'.$next["id"].'/';
				}
			}

			$this->result["children"] = array();
			$result_children = DB::query("SELECT id, [name], [anons], site_id FROM {shop_category}"
						    ." WHERE [act]='1' AND parent_id=%d AND trash='0'"
						    ." AND site_id=%d ORDER BY sort ASC, id ASC",
						    $row["id"], $this->diafan->cid);

			while ($row_children = DB::fetch_array($result_children))
			{
				$child["link"]  = $this->get_link_category($row_children, 'shop');
				if($this->diafan->configmodules("list_img_cat"))
				{
					$child["img"] = Image::get($row_children["id"], $row_children["name"], 'shop',
								   $this->diafan->configmodules("list_img_cat") == 1 ? 1 : 0,
								   $child["link"], TRUE);
				}
				$child["name"]  = $row_children["name"];
				$child["anons"] = $row_children["anons"];
				$this->result["children"][] = $child;
			}

			$result_img = DB::query_range("SELECT s.id, s.[name], s.cat_id, s.timeedit, s.[anons], s.site_id FROM {shop} AS s"
						      .($this->diafan->sort == 1 || $this->diafan->sort == 2 ? " INNER JOIN {shop_price} AS p ON p.shop_id=s.id" : '')
						      ." INNER JOIN {category} AS c ON c.module_name='shop' AND s.id=c.element_id AND c.cat_id IN (%s)"
						      ." WHERE s.[act]='1' AND s.trash='0' GROUP BY s.id ORDER BY "
						      .($this->diafan->sort ? $this->sort[$this->diafan->sort].',' : '')." s.sort ASC, s.id ASC",
						      $this->diafan->configmodules("childrens_elements") ? implode(',', $children) : $this->diafan->cat,
						      $paginator->polog, $paginator->nastr
						     );

			$this->result["rows"] = $this->get_elements($result_img);

			$this->result["paginator"] = $links;

			//сохранение кеша
			$this->diafan->_cache->save($this->result, $cache_meta, $this->diafan->module);
		}
		else
		{
			include_once(ABSOLUTE_PATH.'includes/paginator/paginator.view.php');
		}
		$this->result["buy"] = ! $this->diafan->configmodules('security_user', "shop") || $this->diafan->_user->id ? TRUE : FALSE;

		$this->result["text"] = DB::query_result("SELECT [text] FROM {shop_category} WHERE id=%d", $this->diafan->cat);

		if($this->diafan->configmodules("cat") && $this->diafan->cat && $this->diafan->configmodules("comments_cat"))
		{
			$this->result["comments"] = $this->diafan->_comments->get();
		}

		if($this->result["rows"])
		{
			if($this->diafan->configmodules("rating"))
			{
				include_once(ABSOLUTE_PATH.'modules/rating/rating.inc.php');
				$rating = new Rating_inc($this->diafan);
			}

			if($this->diafan->configmodules("tags"))
			{
				include_once(ABSOLUTE_PATH."modules/tags/tags.inc.php");
				$tags = new Tags_inc($this->diafan);
			}

			for($i = 0; $i < count($this->result["rows"]); $i++)
			{
				$this->result["rows"][$i]["tags"] = ($this->diafan->configmodules("tags")
								    ? $tags->get($this->result["rows"][$i]["id"])
								    : '');

				$this->result["rows"][$i]["show-rating"] = ($this->diafan->configmodules("rating")
									   ? $rating->get($this->result["rows"][$i]["id"])
									   : '');
				$this->result["rows"][$i]["error"] = $this->get_error("shop".$this->result["rows"][$i]["id"]);
				$this->result["rows"][$i] = $this->select_price($this->result["rows"][$i]);
			}
		}

		foreach($this->sort as $key => $value)
		{
			$this->result["link_start"][$key] = ($this->diafan->sort != $key
												 ? '<a href="'.BASE_PATH_HREF.$this->diafan->path_module_variable("sort").'sort'.$key.'/'.'">'
												 : '<span class="active">');
			$this->result["link_finish"][$key] = ($this->diafan->sort != $key ? '</a>' : '</span>');
		}
		return $this->result;
	}

	/**
	 * Генерирует данные для страницы товара
	 * 
	 * @return array|boolean false
	 */
	public function id()
	{
		//кеширование
		$cache_meta = array(
				    "name"     => "show",
				    "cat_id"   => $this->diafan->cat,
				    "show"     => $this->diafan->show,
				    "language" => _LANG,
				    "site_id"  => $this->diafan->cid
				    );
		if(! $this->result = $this->diafan->_cache->get($cache_meta, $this->diafan->module))
		{
			$row = DB::fetch_array(DB::query("SELECT id, [name], timeedit, cat_id, [keywords], [descr], sort, [title_meta] FROM {shop}"
											 ." WHERE [act]='1' AND id = %d AND trash='0' AND site_id=%d",
											 $this->diafan->show, $this->diafan->cid));
			if(empty($row))
			{
				include(ABSOLUTE_PATH.'includes/404.php');
				return FALSE;
			}

			if(! $this->diafan->rewrite_module && ($this->diafan->configmodules("cat") && $this->diafan->cat != $row["cat_id"]
							       || ! $this->diafan->configmodules("cat") && $this->diafan->cat))
			{
				include(ABSOLUTE_PATH.'includes/404.php');
				return FALSE;
			}
			$this->diafan->cat = $this->diafan->configmodules("cat") ? $row["cat_id"] : 0;
			$row["cat_id"]     = $this->diafan->configmodules("cat") ? $row["cat_id"] : 0;

			$row = $this->get_price($row);
			foreach($row as $id => $value)
			{
				$this->result[$id] = $value;
			}
			$this->result["currency"]    = $this->diafan->configmodules("currency", "shop");

			$this->result["img"]         = Image::get($row["id"], $row["name"], 'shop');
			$this->result["param"]       = $this->get_param($row["id"], $this->diafan->cid);
			$this->result["titlemodule"] = $row["name"];

			$previous = DB::fetch_array(DB::query("SELECT [name], id, cat_id FROM {shop} WHERE [act]='1' AND trash='0' AND site_id=%d"
							      .($this->diafan->configmodules("cat") ? " AND cat_id='".$this->diafan->cat."'" : '')
							      ." AND (sort<%d OR sort=%d AND id<%d) ORDER BY sort DESC, id DESC LIMIT 1",
							      $this->diafan->cid, $row["sort"], $row["sort"], $this->diafan->show));
			if($previous)
			{
				$this->result["previous"]["text"] = $previous["name"];
				if(! empty($this->diafan->rewrites["shop"]["element".$previous["id"]]))
				{
					$this->result["previous"]["link"] = $this->diafan->rewrites["shop"]["element".$previous["id"]]["rewrite"].'/';
				}
				elseif($previous["cat_id"] && $this->diafan->configmodules("cat") && ! empty($this->diafan->rewrites["shop"]["cat".$previous["cat_id"]]))
				{
					$this->result["previous"]["link"] = $this->diafan->rewrites["shop"]["cat".$previous["cat_id"]]["rewrite"].'/'
									    .'show'.$previous["id"].'/';
				}
				else
				{
					$this->result["previous"]["link"] = ($this->diafan->rewrite ? $this->diafan->rewrite.'/' : '')
					                                    .($previous["cat_id"] && $this->diafan->configmodules("cat")
									      ? 'cat'.$previous["cat_id"].'/' : '')
									    .'show'.$previous["id"].'/';
				}
			}
			$next = DB::fetch_array(DB::query("SELECT [name], id, cat_id FROM {shop} WHERE [act]='1' AND trash='0' AND site_id=%d"
							  .($this->diafan->configmodules("cat") ? " AND cat_id='".$this->diafan->cat."'" : '')
							  ." AND (sort>%d OR sort=%d AND id>%d) ORDER BY sort ASC, id ASC LIMIT 1",
							  $this->diafan->cid, $row["sort"], $row["sort"], $this->diafan->show));
			if($next)
			{
				$this->result["next"]["text"] = $next["name"];
				if(! empty($this->diafan->rewrites["shop"]["element".$next["id"]]))
				{
					$this->result["next"]["link"] = $this->diafan->rewrites["shop"]["element".$next["id"]]["rewrite"].'/';
				}
				elseif($next["cat_id"] && $this->diafan->configmodules("cat") && ! empty($this->diafan->rewrites["shop"]["cat".$next["cat_id"]]))
				{
					$this->result["next"]["link"] = $this->diafan->rewrites["shop"]["cat".$next["cat_id"]]["rewrite"].'/'
					                                .'show'.$next["id"].'/';
				}
				else
				{
					$this->result["next"]["link"] = ($this->diafan->rewrite ? $this->diafan->rewrite.'/' : '')
					                                .($next["cat_id"] && $this->diafan->configmodules("cat")
									  ? 'cat'.$next["cat_id"].'/' : '')
									.'show'.$next["id"].'/';
				}
			}

			$this->result["path"]        = $this->get_path('shop');

			//сохранение кеша
			$this->diafan->_cache->save($this->result, $cache_meta, $this->diafan->module);
		}
		$this->diafan->cat = $this->result["cat_id"];

		$row = DB::fetch_array(DB::query("SELECT [text], [anons] FROM {shop} WHERE id=%d", $this->diafan->show));

		$this->result["text"]  = $row["text"];
		$this->result["anons"] = $row["anons"];
		$this->result["buy"]   = ! $this->diafan->configmodules('security_user', "shop") || $this->diafan->_user->id ? TRUE : FALSE;
		$this->result["error"] = $this->get_error("shop".$this->result["id"]);
		$this->result          = $this->select_price($this->result);

		if($this->diafan->configmodules("rating"))
		{
			include_once(ABSOLUTE_PATH.'modules/rating/rating.inc.php');
			$rating = new Rating_inc($this->diafan);
			$this->result["show-rating"] = $rating->get();
			$this->result["add-rating"] = $rating->add();			
		}

		if($this->diafan->configmodules("comments"))
		{
			$this->result["comments"] = $this->diafan->_comments->get();
		}

		if($this->diafan->configmodules("tags"))
		{
			include_once(ABSOLUTE_PATH."modules/tags/tags.inc.php");
			$tags = new Tags_inc($this->diafan);
			$this->result["tags"] = $tags->get();
		}

		return $this->result;
	}

	/**
	 * Генерирует данные для шаблонной функции: блок товаров
	 * 
	 * @param integer $count количество файлов
	 * @param integer $cat_id категория
	 * @param integer $site_id страница сайта
	 * @param string $sort сортировка date - по дате, rand - случайно
	 * @param integer $photo количество фотографий
	 * @return array
	 */
	public function show_block($count, $cat_id, $site_id, $sort, $photo)
	{
		//кеширование
		$cache_meta = array(
				    "name"     => "block",
				    "cat_id"   => $cat_id,
				    "count"    => $count,
				    "language" => _LANG,
				    "site_id"  => $site_id,
				    "photo"    => $photo,
				    "sort"     => $sort
				    );

		if($sort == "rand" || ! $this->result = $this->diafan->_cache->get($cache_meta, "shop"))
		{
			if(! $site_id)
			{
				$site_id = DB::query_result("SELECT id FROM {site} WHERE addmodule='shop' AND trash='0' AND act='1'");
			}
			if($cat_id && $this->diafan->configmodules("cat", "shop"))
			{
				if($this->diafan->configmodules("childrens_elements", "shop"))
				{ 
					$children = $this->diafan->get_children($cat_id, "shop_category");
				}
			}

			if($sort == "rand")
			{
				$max_count = DB::query_result("SELECT COUNT(DISTINCT e.id) FROM {shop} as e"
							      .($cat_id && $this->diafan->configmodules("cat","shop")
							       ? " INNER JOIN {category} as c ON c.module_name='shop' AND c.element_id=e.id"
							         .(! empty($children) ? ' AND c.cat_id IN ('.implode(',', $children).')' : ' AND c.cat_id='.$cat_id)
							       : ''
							       )
							      ." WHERE e.[act]='1' AND e.trash='0'"
							      .($site_id
								? " AND e.site_id=".$site_id
								: ''
								)
							      );
				$rands = array();
				for($i = 1; $i <= min($max_count, $count); $i++)
				{
					do
					{
						$rand = mt_rand(0, $max_count - 1);
					}
					while(in_array($rand, $rands));
					$rands[] = $rand;
				}
			}
			else
			{
				$rands[0] = 1;
			}
			$this->result["rows"] = array();

			foreach($rands as $rand)
			{
				$result = DB::query_range("SELECT e.id, e.[name], e.[anons], e.cat_id, e.timeedit, e.site_id FROM {shop} AS e"
							  .($sort == "price" ? " INNER JOIN {shop_price} AS p ON p.shop_id=e.id" : '')
							  .($cat_id && $this->diafan->configmodules("cat", "shop")
							    ? " INNER JOIN {category} as c ON c.module_name='shop' AND c.element_id=e.id"
							      .(! empty($children) ? ' AND c.cat_id in ('.implode(',', $children).')':' AND c.cat_id='.$cat_id)
							    : ''
							    )
							  ." WHERE e.[act]='1' AND e.trash='0'"
							  .($site_id
							    ? " AND e.site_id=".$site_id
							    : ''
							    )
							  ." GROUP BY e.id"
							  .($sort == "price" ? ' ORDER BY p.price ASC' : ($sort == "date" ? ' ORDER BY e.id DESC' : '')),
							  $sort == "rand"    ? $rand : 0,
							  $sort == "rand"    ? 1     : $count);
				$this->result["rows"] = array_merge($this->result["rows"], $this->get_elements($result, $photo, "block"));
			}

			if(! empty($this->result["rows"]) && $cat_id && $this->diafan->configmodules("cat", "shop"))
			{
				$cat = DB::fetch_array(DB::query("SELECT [name], site_id, id FROM {shop_category} WHERE id=%d", $cat_id));
				
				$this->result["name"] = $cat["name"];
				$this->result["allshop"]["link"] = $this->get_link_category($cat, 'shop');
				$this->result["allshop"]["text"] = _LANG_ALL_GOODS.' '._LANG_IN_CATEGORY.' '._LANG_LQUOT.$cat["name"]._LANG_RQUOT;
			}
			elseif(! empty($this->result["rows"]) && $site_id)
			{
				$this->result["name"] = DB::query_result("SELECT [name] FROM {site} WHERE id=%d", $site_id);
				$this->result["allshop"]["link"] = ! empty($this->diafan->rewrites[$site_id]["rewrite"]) ? $this->diafan->rewrites[$site_id]["rewrite"].'/' : '';
				$this->result["allshop"]["text"] = _LANG_ALL_GOODS;
			}
			if($site_id)
			{
				$this->result["action"] = ! empty($this->diafan->rewrites[$site_id]["rewrite"]) ? $this->diafan->rewrites[$site_id]["rewrite"].'/' : '';
			}
			elseif($this->diafan->module == "shop")
			{
				$this->result["action"] = $this->diafan->rewrite ? $this->diafan->rewrite.'/' : '';
			}
			else
			{
				$shop_site_id = DB::query_result("SELECT id FROM {site} WHERE addmodule='shop' AND trash='0' AND act='1'");
				$this->result["action"] = ! empty($this->diafan->rewrites[$shop_site_id]["rewrite"]) ? $this->diafan->rewrites[$shop_site_id]["rewrite"].'/' : '';
			}
			//сохранение кеша
			if($sort != "rand")
			{
				$this->diafan->_cache->save($this->result, $cache_meta, "shop");
			}
		}

		if($this->result["rows"])
		{
			for($i = 0; $i < count($this->result["rows"]); $i++)
			{
				$this->result["rows"][$i]["error"] = $this->get_error("shop".$this->result["rows"][$i]["id"]);
				$this->result["rows"][$i] = $this->select_price($this->result["rows"][$i]);
			}
		}
		$this->result["buy"] = ! $this->diafan->configmodules('security_user', "shop") || $this->diafan->_user->id ? TRUE : FALSE;
		return $this->result;
	}

	/**
	 * Генерирует контент для шаблонной функции: форма поиска по товарам
	 *
	 * @param integer|string $cat_id номер категории, в которой осуществляется поиск
	 * @param integer $site_id страница сайта, к которой прикреплен модуль Магазин
	 * @return array
	 */
	public function show_search($cat_id, $site_id)
	{
		$this->result = array();
		if((! $site_id || $site_id == $this->diafan->cid) && $this->diafan->module == "shop")
		{
			$rewrite = $this->diafan->rewrite;
		}
		else
		{
			if(! $site_id)
			{
				$site_id = DB::query_result("SELECT id FROM {site} WHERE addmodule='shop' AND trash='0' AND act='1'");
			}
			$rewrite = ! empty($this->diafan->rewrites[$site_id]["rewrite"]) ? $this->diafan->rewrites[$site_id]["rewrite"] : '';
		}

		if($cat_id === 'current')
		{
			if($this->diafan->cat && $this->diafan->module == "shop" && (! $site_id || $site_id == $this->diafan->cid))
			{
				$cat_id = $this->diafan->cat;
			}
			elseif(! empty($_GET["catid"]) && $this->diafan->module=="shop" && (! $site_id || $site_id == $this->diafan->cid))
			{	
				$this->diafan->cat = $this->diafan->get_param($_GET, "catid", '', 2);
				$cat_id = $this->diafan->cat;
			}
			else
			{
				$cat_id = 0;
			}
		}
		$this->result["path"]   = $rewrite ? $rewrite.'/' : '';
		$this->result["cat_id"] = $cat_id;
		
		if($this->diafan->configmodules("search_name", "shop"))
		{
			$this->result["name"] = array(
						    "name"  => 1,
						    "value" => ! empty($_GET["n"]) ? trim(htmlspecialchars(stripslashes($_GET["n"]))) : ''
						    );
		}

		if($this->diafan->configmodules("search_text", "shop"))
		{
			$this->result["text"] = array(
						    "name"  => 1,
						    "value" => ! empty($_GET["d"]) ? trim(htmlspecialchars(stripslashes($_GET["d"]))) : ''
						    );
		}

		if($this->diafan->configmodules("search_price", "shop"))
		{
			$this->result["price"] = array(
						    "name"   => 1,
						    "value1" => $this->diafan->get_param($_GET, "pr1", '', 2),
						    "value2" => $this->diafan->get_param($_GET, "pr2", '', 2)
						    );
		}

		$result = DB::query("SELECT DISTINCT(p.id), p.type, p.[name] FROM {param} as p "
							." INNER JOIN {category} AS c ON p.id=c.element_id AND c.module_name='shopparam' AND c.cat_id"
							.($cat_id ? " IN (".$cat_id.", 0) " : '=0')
							." WHERE p.search='1' AND p.trash='0' AND p.module_name='shop'");
		while($row = DB::fetch_array($result))
		{
			if($row["type"] == 'text' || $row["type"] == 'textarea' || $row["type"] == 'editor')
			{
				$row["value"] = ! empty($_GET["p".$row["id"]]) ? trim(htmlspecialchars(stripslashes($_GET["p".$row["id"]]))) : '';
		  	}
			elseif($row["type"] == 'date')
			{
		  		$row["d1"] = $this->diafan->get_param($_GET, "p".$row["id"]."d1", '', 2);
				$row["d1"] = $row["d1"] ? $row["d1"] : "dd";

		  		$row["m1"] = $this->diafan->get_param($_GET, "p".$row["id"]."m1", '', 2);
				$row["m1"] = $row["m1"] ? $row["m1"] : "mm";

		  		$row["y1"] = $this->diafan->get_param($_GET, "p".$row["id"]."y1", '', 2);
				$row["y1"] = $row["y1"] ? $row["y1"] : "yyyy";

		  		$row["d2"] = $this->diafan->get_param($_GET, "p".$row["id"]."d2", '', 2);
				$row["d2"] = $row["d2"] ? $row["d2"] : "dd";

		  		$row["m2"] = $this->diafan->get_param($_GET, "p".$row["id"]."m2", '', 2);
				$row["m2"] = $row["m2"] ? $row["m2"] : "mm";

		  		$row["y2"] = $this->diafan->get_param($_GET, "p".$row["id"]."y2", '', 2);
				$row["y2"] = $row["y2"] ? $row["y2"] : "yyyy";
		 	}
			elseif($row["type"] == 'numtext')
			{
				$row["value1"] = $this->diafan->get_param($_GET, "p".$row["id"]."_1", '', 2);
				$row["value2"] = $this->diafan->get_param($_GET, "p".$row["id"]."_2", '', 2);
		  	}
			elseif($row["type"] == 'checkbox')
			{
				$row["value"] = $this->diafan->get_param($_GET, "p".$row["id"], '', 2);
		  	}
			elseif($row["type"] == 'select' || $row["type"] == 'multiple')
			{
				$result_select = DB::query("SELECT [name], value FROM {param_select} WHERE param_id=%d ORDER BY value ASC", $row["id"]);
				while($row_select = DB::fetch_array($result_select))
				{
					$row["select_array"][$row_select["value"]] = $row_select["name"];
				}
				if($row["type"] == 'select')
				{
					$row["value"] = $this->diafan->get_param($_GET, "p".$row["id"], '', 2);
				}
				elseif(! empty($_GET["p".$row["id"]]) && is_array($_GET["p".$row["id"]]))
				{
					foreach($_GET["p".$row["id"]] as $val)
					{
						$row["value"][] = intval($val);
					}
				}
				else
				{
					$row["value"] = array();
				}
		  	}
			unset($row["param"]);
			$this->result["rows"][] = $row;
		}
		return $this->result;
	}

	/**
	 * Форматирует данные о товаре для списка товаров
	 *
	 * @param resource $result результат выполнения SQL-запроса
	 * @param string $photo показывать фотографии
	 * @param string $function функция, для которой генерируется список товаров
	 * @return array
	 */
	private function get_elements($result, $photo = 'list', $function = 'list')
	{	
		if(empty($this->result["timeedit"]))
		{
			$this->result["timeedit"] = '';
		}
		$rows = array();
		while ($row = DB::fetch_array($result))
		{
			if($row["timeedit"] < $this->result["timeedit"])
			{
				$this->result["timeedit"] = $row["timeedit"];
			}
			unset($row["timeedit"]);

			if(! empty($this->diafan->rewrites["shop"]["element".$row["id"]]))
			{
				$row["link"] = $this->diafan->rewrites["shop"]["element".$row["id"]]["rewrite"].'/';
			}
			elseif(! empty($row["cat_id"]) && $this->diafan->configmodules("cat", "shop", '', $row["site_id"]) && ! empty($this->diafan->rewrites["shop"]["cat".$row["cat_id"]]))
			{
				$row["link"] = $this->diafan->rewrites["shop"]["cat".$row["cat_id"]]["rewrite"].'/show'.$row["id"].'/';
			}
			else
			{
				$row["link"] = (! empty($this->diafan->rewrites[$row["site_id"]]["rewrite"]) ? $this->diafan->rewrites[$row["site_id"]]["rewrite"].'/' : '')
				               .(! empty($row["cat_id"]) && $this->diafan->configmodules("cat", "shop", '', $row["site_id"])? 'cat'.$row["cat_id"].'/' : '')
					           .'show'.$row["id"].'/';
			}
			unset($row["cat_id"]);

			if($photo === 'list' && $this->diafan->configmodules("list_img", "shop"))
			{
				$row["img"]  = Image::get($row["id"], $row["name"], 'shop',
							   $this->diafan->configmodules("list_img") == 1 ? 1 : 0,
							   $row["link"]);
			}
			elseif($photo != 'list' && $photo)
			{
				$row["img"] = Image::get($row["id"], $row["name"], 'shop', $photo, $row["link"]);	
			}
			$row = $this->get_price($row);
			$row["param"] = $this->get_param($row["id"], $row["site_id"], $function);

			$rows[] = $row;	
		}
		$this->result["currency"] = $this->diafan->configmodules("currency", "shop");
		return $rows;
	}

	/**
	 * Формирует данные о цене товара
	 *
	 * @param array $row данные о товаре
	 * @return array
	 */
	private function get_price($row)
	{
		$row["param_select_buy"] = array();
		$params = array();

		$result = DB::query("SELECT e.param_id, e.value FROM {param_element} as e"
							." INNER JOIN {param} as p ON e.param_id=p.id"
							." WHERE e.element_id=%d AND e.module_name='shop'"
							." AND p.`type`='multiple' AND p.required='1'"
							." GROUP BY e.id ORDER BY p.sort, LENGTH(e.value), e.value ASC", $row["id"]);
		while ($row_param = DB::fetch_array($result))
		{
			$param["value"] = $row_param["value"];

			$param["selected"] = FALSE;
			$row["param_select_buy"][$row_param["param_id"]][] = $param;

			if(empty($this->result["depends"][$row_param["param_id"]]))
			{
				$this->result["depends"][$row_param["param_id"]]["name"] = DB::query_result("SELECT [name] FROM {param} WHERE id=%d", $row_param["param_id"]);
				$result_select = DB::query("SELECT [name], value FROM {param_select} WHERE param_id=%d", $row_param["param_id"]);
				while ($row_select = DB::fetch_array($result_select))
				{
					$this->result["depends"][$row_param["param_id"]]["values"][$row_select["value"]] = $row_select["name"];
				}
			}
		}
		foreach($row["param_select_buy"] as $id => $rows)
		{
			if(count($row["param_select_buy"][$id]) < 2)
			{
				unset($row["param_select_buy"][$id]);
			}
		}

		$row["depend"] = array();
		$row["price"] = 0;
		$pr1 = $this->diafan->get_param($_GET, "pr1", "", 2);
		$pr2 = $this->diafan->get_param($_GET, "pr2", "", 2);
		$result = DB::query("SELECT price, param_id, param_value FROM {shop_price} WHERE shop_id=%d"
							.($pr1 ? " AND price>=".$pr1 : '').($pr2 ? " AND price<=".$pr2 : '')
							." ORDER BY param_value ASC", $row["id"]);
		while ($row_param = DB::fetch_array($result))
		{
			// если задана цена без зависимых параметров, то цена, считается независимой
			if(! $row_param["param_id"])
			{
				$row["price"]  = $this->diafan->format_price($row_param["price"]);
				$row["depend"] = array();
				return $row;
			}
			if(! empty($row["param_select_buy"][$row_param["param_id"]]))
			{
				unset($row["param_select_buy"][$row_param["param_id"]]);
			}
			$depend["price"]    = $this->diafan->format_price($row_param["price"]);
			$depend["value"]    = $row_param["param_value"];
			$row["depend"]["param_id"] = $row_param["param_id"];

			$depend["selected"] = FALSE;
			$row["depend"]["rows"][] = $depend;
			if(empty($this->result["depends"][$row_param["param_id"]]))
			{
				$this->result["depends"][$row_param["param_id"]]["name"] = DB::query_result("SELECT [name] FROM {param} WHERE id=%d", $row_param["param_id"]);
				$result_select = DB::query("SELECT [name], value FROM {param_select} WHERE param_id=%d", $row_param["param_id"]);
				while ($row_select = DB::fetch_array($result_select))
				{
					$this->result["depends"][$row_param["param_id"]]["values"][$row_select["value"]] = $row_select["name"];
				}
			}
		}

		return $row;
	}

	/**
	 * Задает выделение параметров, учитываемый при покупке товара
	 *
	 * @param array $row данные о товаре
	 * @return array
	 */
	private function select_price($row)
	{
		foreach($row["param_select_buy"] as $id => $params)
		{
			$new_params = array();
			foreach($params as $param)
			{
				if(! empty($_GET["p".$id]) && $_GET["p".$id] == $param["value"])
				{
					$param["selected"] = TRUE;
				}
				$new_params[] = $param;
			}
			$row["param_select_buy"][$id] = $new_params;
		}
		if(! empty($row["depend"]["rows"]))
		{
			$new_params = array();
			foreach($row["depend"]["rows"] as $param)
			{
				if(! empty($_GET["p".$row["depend"]["param_id"]]) && $_GET["p".$row["depend"]["param_id"]] == $param["value"])
				{
					$param["selected"] = TRUE;
					$row["price"] = $param["price"];
				}
				$new_params[] = $param;
			}
			$row["depend"]["rows"] = $new_params;
		}

		return $row;
	}

	/**
	 * Получает дополнительные характеристики товара
	 * 
	 * @param integer $id номер товара
	 * @param integer $site_id номер страницы, к которой прикреплен товар
	 * @param string $function функция, для которой выбираются параметры
	 * @return array
	 */
	private function get_param($id, $site_id, $function = "id")
	{
		global $param_select;
		$values    = array();
		$rvalues   = array();
		$result_el = DB::query("SELECT value as rv, [value], param_id FROM {param_element} WHERE element_id=%d AND module_name='shop' ORDER BY LENGTH(value) ASC, value ASC", $id);
		while ($row_el = DB::fetch_array($result_el))
		{
			$values[$row_el["param_id"]][]  = $row_el["value"];
			$rvalues[$row_el["param_id"]][] = $row_el["rv"];
		}
		$result = DB::query("SELECT p.id, p.[name], p.type FROM {param} as p "
				    .($this->diafan->configmodules("cat", "shop", '', $site_id)
					  ? " INNER JOIN {category} as c ON c.module_name='shop' AND c.element_id=%d"
					  : "")
				    ." INNER JOIN {category} as cp ON cp.module_name='shopparam' AND ("
					  .($this->diafan->configmodules("cat", "shop", '', $site_id) ? "cp.cat_id=c.cat_id OR " : "")
					  ."cp.cat_id=0) AND cp.element_id=p.id"
				    ." WHERE p.trash='0' AND p.module_name='shop'"
				    .($function == "block" ? " AND p.block='1'" : '')
				    .($function == "list"  ? " AND p.list='1'"  : '')
				    ." ORDER BY p.sort ASC", $id
				    );

		$param = array();
		while($row = DB::fetch_array($result))
		{
			unset($value);
			switch ($row["type"])
			{
				case "text":
				case "textarea":
				case "editor":
					$value = ! empty($values[$row["id"]][0]) ? $values[$row["id"]][0] : '';
					break;
				case "date":
					$value = ! empty($rvalues[$row["id"]][0]) ? date("d.m.Y", $rvalues[$row["id"]][0]) : '';
					break;
				case "select":
					if(! empty($rvalues[$row["id"]][0]) && empty($param_select[$row["id"]][$rvalues[$row["id"]][0]]))
					{
						$param_select[$row["id"]][$rvalues[$row["id"]][0]] = DB::query_result("SELECT [name] FROM {param_select} WHERE value=%d AND param_id=%d", $values[$row["id"]][0], $row["id"]);
					}
					$value = ! empty($rvalues[$row["id"]][0]) ? $param_select[$row["id"]][$rvalues[$row["id"]][0]] : '';
					break;
				case "multiple":
					$value = array();
					if(! empty($rvalues[$row["id"]]))
					{
						foreach($rvalues[$row["id"]] as $val)
						{
							if(empty($param_select[$row["id"]][$val]))
							{
								$param_select[$row["id"]][$val] = DB::query_result("SELECT [name] FROM {param_select}"
																				   ." WHERE value=%d AND param_id=%d", $val, $row["id"]);
							}
							$value[] = $param_select[$row["id"]][$val];
						}
					}
					break;
				case "checkbox":
					$value = ! empty($rvalues[$row["id"]][0]) ? 1 : 0;
					if(! isset($param_select[$row["id"]][$value]))
					{
						$param_select[$row["id"]][$value] = DB::query_result("SELECT [name] FROM {param_select} WHERE value=%d"
																			 ." AND param_id=%d", $value, $row["id"]);
					}
					if(! $param_select[$row["id"]][$value] && $value == 1)
					{
						$value = '';
						$param[$row["name"]] = '';
					}
					else
					{
						$value = $param_select[$row["id"]][$value];
					}
					break;
				default:
					$value = ! empty($rvalues[$row["id"]][0]) ? $rvalues[$row["id"]][0] : '';
					break;
			}
			if(! empty($value))
			{
				$param[$row["name"]] = $value;
			}
		}
		return $param;
	}

	/**
	 * Формирует SQL-запрос при поиске по товарам
	 * 
	 * @return array
	 */
	private function get_where()
	{
		$where_param  = '';
		$where        = ' AND s.site_id=%d';
		$values       = array();
		$values_param = array();
		$getnav       = '';

		if(isset($_GET["catid"]))
		{
			$this->diafan->cat = $this->diafan->get_param($_GET,  "catid", '', 2);
			$catarr = array(0);
			$getnav = '?catid='.$this->diafan->cat;
			if($this->diafan->cat)
			{
				if($this->diafan->configmodules("childrens_elements"))
				{
					$children = $this->diafan->get_children($this->diafan->cat, "shop_category");
				}
				$where_param .= " INNER JOIN {category} AS c ON c.module_name='shop' AND s.id=c.element_id AND "
				                .($this->diafan->configmodules("childrens_elements")
								  ? 'c.cat_id in ('.implode(',', $children).')'
								  : 'c.cat_id='.$this->diafan->cat);
			}
			
			if(! empty($_GET["n"]) && $this->diafan->configmodules("search_name"))
			{
				$where   .= " AND s.name"._LANG." LIKE '%%%h%%'";
				$values[] = trim($this->diafan->get_param($_GET,  "n", '', 1));
				$getnav  .= '&n='.trim($this->diafan->get_param($_GET,  "n", '', 1));
			}
			
			if(! empty($_GET["d"]) && $this->diafan->configmodules("search_text"))
			{
				$where   .= " AND s.text"._LANG." LIKE '%%%h%%'";
				$values[] = trim($this->diafan->get_param($_GET,  "d", '', 1));
				$getnav  .= '&d='.trim($this->diafan->get_param($_GET,  "d", '', 1));
			}

			if(! empty($_GET["pr1"]) || ! empty($_GET["pr2"]))
			{
				$where_param .= " INNER JOIN {shop_price} AS pr ON pr.shop_id=s.id"
				                .($_GET["pr1"] ? " AND pr.price>=%d" : '')
								.($_GET["pr2"] ? " AND pr.price<=%d" : '');
				if($_GET["pr1"])
				{
					$pr1 = $this->diafan->get_param($_GET,  "pr1", '', 2);
					$values_param[] = $pr1;
					$getnav .= '&pr1='.$pr1;
				}
				if($_GET["pr2"])
				{
					$pr2 = $this->diafan->get_param($_GET,  "pr2", '', 2);
					$values_param[] = $pr2;
					$getnav .= '&pr2='.$pr2;
				}
			}

			$result = DB::query("SELECT DISTINCT(p.id), p.type FROM {param} as p "
								." INNER JOIN {category} AS c ON p.id=c.element_id AND c.module_name='shopparam' AND c.cat_id"
								.($this->diafan->cat ? " IN (".$this->diafan->cat.", 0) " : '=0')
								." WHERE p.search='1' AND p.trash='0' AND p.module_name='shop'");
			while($row = DB::fetch_array($result))
			{
				if (($row["type"] == 'text' || $row["type"] == 'textarea' || $row["type"] == 'editor') && ! empty($_GET["p".$row["id"]]))
				{
					$val = trim($this->diafan->get_param($_GET, "p".$row["id"], '', 1));
					$where_param .= "
					INNER JOIN {param_element} AS pe".$row["id"]." ON pe".$row["id"].".element_id=s.id AND pe".$row["id"].".param_id='%d'"
					." AND pe".$row["id"].".trash='0' AND pe".$row["id"].".module_name='shop'"
					." AND pe".$row["id"].".value"._LANG." LIKE '%%%h%%'";
					$values_param[] = $row["id"];
					$values_param[] = $val;
					$getnav .= '&p'.$row["id"].'='.$val;
				}
				elseif($row["type"] == 'date' && (! empty($_GET["p".$row["id"]."d1"]) && ! empty($_GET["p".$row["id"]."m1"]) && ! empty($_GET["p".$row["id"]."y1"]) || ! empty($_GET["p".$row["id"]."d2"]) && ! empty($_GET["p".$row["id"]."m2"]) && ! empty($_GET["p".$row["id"]."y2"])))
				{
					$valm1 = $this->diafan->get_param($_GET, "p".$row["id"]."m1", '', 2);
					$vald1 = $this->diafan->get_param($_GET, "p".$row["id"]."d1", '', 2);
					$valy1 = $this->diafan->get_param($_GET, "p".$row["id"]."y1", '', 2);

					$valm2 = $this->diafan->get_param($_GET, "p".$row["id"]."m2", '', 2);
					$vald2 = $this->diafan->get_param($_GET, "p".$row["id"]."d2", '', 2);
					$valy2 = $this->diafan->get_param($_GET, "p".$row["id"]."y2", '', 2);

					$where_param .= "
					INNER JOIN {param_element} AS pe".$row["id"]." ON pe".$row["id"].".element_id=s.id AND pe".$row["id"].".param_id='%d'"
					." AND pe".$row["id"].".trash='0' AND pe".$row["id"].".module_name='shop'";
					$values_param[] = $row["id"];
					if($valm1 && $vald1 && $valy1)
					{
						$where_param .= " AND pe".$row["id"].".value>=%d";
						$values_param[] = mktime(0, 0, 0, $valm1, $vald1, $valy1);
						$getnav .= '&p'.$row["id"].'d1='.$vald1.'&p'.$row["id"].'m1='.$vald1.'&p'.$row["id"].'y1='.$valy1;
					}
					if($valm2 && $vald2 && $valy2)
					{
						$where_param .= " AND pe".$row["id"].".value<=%d";
						$values_param[] = mktime(0, 0, 0, $valm2, $vald2, $valy2);
						$getnav .= '&p'.$row["id"].'d2='.$vald2.'&p'.$row["id"].'m2='.$vald2.'&p'.$row["id"].'y2='.$valy2;
					}
				}
				elseif($row["type"] == 'numtext' && (! empty($_GET["p".$row["id"]."_2"]) ||  ! empty($_GET["p".$row["id"]."_1"])))
				{
					$val1 = $this->diafan->get_param($_GET, "p".$row["id"]."_1", '', 2);
					$val2 = $this->diafan->get_param($_GET, "p".$row["id"]."_2", '', 2);
					$where_param .= "
					INNER JOIN {param_element} AS pe".$row["id"]." ON pe".$row["id"].".element_id=s.id AND pe".$row["id"].".param_id='%d'"
					." AND pe".$row["id"].".trash='0' AND pe".$row["id"].".module_name='shop'"
					.($val1 ? " AND pe".$row["id"].".value>=%d" : '')
					.($val2 ? " AND pe".$row["id"].".value<=%d" : '')
					;
					$values_param[] = $row["id"];
					if($val1)
					{
						$values_param[] = $val1;
						$getnav .= '&p'.$row["id"].'_1='.$val1;
					}
					if($val2)
					{
						$values_param[] = $val2;
						$getnav .= '&p'.$row["id"].'_2='.$val2;
					}
				}
				elseif($row["type"] == 'checkbox' && ! empty($_GET["p".$row["id"]]))
				{
					$where_param .= "
					INNER JOIN {param_element} AS pe".$row["id"]." ON pe".$row["id"].".element_id=s.id AND pe".$row["id"].".param_id='%d'"
					." AND pe".$row["id"].".trash='0' AND pe".$row["id"].".module_name='shop'"
					." AND pe".$row["id"].".value='1'";
					$values_param[] = $row["id"];
					$getnav .= '&p'.$row["id"].'=1';
				}
				elseif($row["type"] == 'select' && ! empty($_GET["p".$row["id"]]))
				{
					$val = $this->diafan->get_param($_GET, "p".$row["id"], '', 2);
					$where_param .= "
					INNER JOIN {param_element} AS pe".$row["id"]." ON pe".$row["id"].".element_id=s.id AND pe".$row["id"].".param_id='%d'"
					." AND pe".$row["id"].".trash='0' AND pe".$row["id"].".module_name='shop'"
					." AND pe".$row["id"].".value='%d'";
					$values_param[] = $row["id"];
					$values_param[] = $val;
					$getnav .= '&p'.$row["id"].'='.$val;
				}
				elseif($row["type"] == 'multiple' && ! empty($_GET["p".$row["id"]]) && is_array($_GET["p".$row["id"]]))
				{
					$vals= array();
					foreach($_GET["p".$row["id"]] as $val)
					{
						if($val)
						{
							$val = intval($val);
							$vals[] = $val;
							$getnav .= '&p'.$row["id"].'[]='.$val;
						}
					}
					if(! empty($vals))
					{
						$where_param .= "
						INNER JOIN {param_element} AS pe".$row["id"]." ON pe".$row["id"].".element_id=s.id AND pe".$row["id"].".param_id='%d'"
						." AND pe".$row["id"].".trash='0' AND pe".$row["id"].".module_name='shop'"
						." AND pe".$row["id"].".value IN (".implode(", ", $vals).")";
						$values_param[] = $row["id"];
					}
				}
			}
		}
		$values = array_merge($values_param, array($this->diafan->cid), $values);
		return array($where, $where_param, $values, $getnav);
	}
}