<?php
/**
 * Diafan.CMS
 *
 * @author     diafan.ru
 * @version    4.3
 * @license http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2011 OOO "Диафан". (http://diafan.ru)
 */

if(! defined('DIAFAN'))
{
	include(dirname(dirname(dirname(__FILE__))).'/includes/404.php');
}

/**
 * Shop
 *
 * Контроллер модуля "Магазин"
 */
class Shop extends Frame
{
	/**
	 * Конструктор класса
	 * 
	 * @return void
	 */
	public function __construct(&$diafan)
	{
		$this->diafan = &$diafan;
		$this->rewrite_variable_names = array('page', 'show', 'cat', 'sort');
		if($this->diafan->configmodules("comments") || $this->diafan->configmodules("comments_cat") && ! $this->diafan->show && $this->diafan->cat)
		{
			$this->rewrite_variable_names[] = 'comments';
		}
		$this->diafan->rewrite_variable_names = $this->rewrite_variable_names;

		include_once(ABSOLUTE_PATH.'modules/shop/shop.ajax.php');
		$ajax = new Shop_ajax($diafan);
		if($ajax->ajax_request())
		{
			exit;
		}

		$model = new Shop_model($diafan);

		if($this->diafan->show)
		{
			$this->result = $model->id();
		}
		elseif(isset($_GET["catid"]))
		{
			$this->result = $model->list_search();
		}
		elseif(! $this->diafan->configmodules("cat"))
		{
			$this->result = $model->list_();
		}
		elseif(! $this->diafan->cat)
		{
			$this->result = $model->first_page();
		}
		else
		{
			$this->result = $model->list_category();
		}
		$this->get_global_variables();
	}

	/**
	 * Выводит контент модуля
	 * 
	 * @return boolean true
	 */
	public function show_module()
	{
		if(! empty($this->result["err"]))
		{
			echo $this->result["err"];
			return FALSE;
		}
		
		if($this->diafan->show)
		{
			Shop_view::id($this->result);
		}
		elseif($this->diafan->configmodules("cat") && ! $this->diafan->cat && ! isset($_GET["catid"]))
		{
			Shop_view::first_page($this->result);
		}
		else
		{
			if(! empty($this->result["level"]) && is_callable(array('Shop_view', 'list_level_'.$this->result["level"])))
			{
				$func = 'list_level_'.$this->result["level"];
				Shop_view::$func($this->result);
			}
			else
			{
				Shop_view::list_($this->result);
			}
		}

		return TRUE;
	}

	/**
	 * Шаблонная функция: категории товаров
	 * 
	 * @param object $th основной объект
	 * @param array $attributes атрибуты шаблонного тэга
	 * @return boolean
	 */
	function show_catalog($th, $attributes)
	{
		if(! $th->diafan->configmodules("cat", "shop"))
		{
			return FALSE;
		}
		$attributes = $this->get_attributes($attributes, 'tag_start_1', 'tag_end_1', 'tag_active_start_1', 'tag_active_end_1', 'tag_level_start_1',
						    'tag_level_end_1', 'tag_active_child_start_1', 'tag_active_child_end_1', 'hide_parent_link',
						    'separator_1', array('koren', 0), 'count_level', 'show_all_level', 'level', 'site_id');

		$attributes["show_all_level"] = $attributes["show_all_level"] ? TRUE : FALSE;
		if(! $attributes["site_id"])
		{
			$attributes["site_id"] = DB::query_result("SELECT id FROM {site} WHERE addmodule='shop' AND trash='0' AND act='1'");
		}

		include_once(ABSOLUTE_PATH.'includes/menu.php');
		$menu = new Menu($th->diafan, $attributes);
		$menu->table = "shop_category";
		$menu->cid = $this->diafan->module == "shop" ? $this->diafan->cat : '';
		$menu_data = $menu->get_menu();
		$menu->draw_menu($menu_data);
		return TRUE;
	}

	/**
	 * Шаблонная функция: блок товаров
	 * 
	 * @param object $th основной объект
	 * @param array $attributes атрибуты шаблонного тэга
	 * @return boolean
	 */
	function show_block($th, $attributes)
	{
		if($th->diafan->rewrite == "shop")
			return FALSE;

		$attributes = $this->get_attributes($attributes, 'count', 'cat_id', 'site_id', 'photo', 'sort', 'template');

		$count   = $attributes["count"] ? intval($attributes["count"]) : 3;
		$cat_id  = intval($attributes["cat_id"]);
		$site_id = intval($attributes["site_id"]);
		$photo   = intval($attributes["photo"]);
		$sort    = $attributes["sort"] == "date" || $attributes["sort"] == "rand" || $attributes["sort"] == "price" ? $attributes["sort"] : "date";

		include_once(ABSOLUTE_PATH.'modules/shop/shop.model.php');
		$model = new Shop_model($th->diafan);
		$result = $model->show_block($count, $cat_id, $site_id, $sort, $photo);

		include_once(ABSOLUTE_PATH."modules/shop/shop.view.php");
		$func = 'show_block';
		if($attributes["template"])
		{
			$func .= '_'.preg_replace('/[^a-z_]+/','', $attributes["template"]);
			
		}
		Shop_view::$func($result);

		return TRUE;
	}

	/**
	 * Шаблонная функция: форма поиска по товарам
	 * 
	 * @param object $th основной объект
	 * @param array $attributes атрибуты шаблонного тэга
	 * @return boolean
	 */
	function show_search($th, $attributes)
	{
		$attributes = $this->get_attributes($attributes, 'only_shop', 'cat_id', 'site_id', 'template');

		$only_shop = $attributes["only_shop"];
		$cat_id    = $attributes["cat_id"] === 'current' ? 'current' : intval($attributes["cat_id"]);
		$site_id   = intval($attributes["site_id"]);

		if($only_shop && ($th->diafan->module != "shop" || $site_id && $th->diafan->cid != $site_id))
			return FALSE;

		include_once(ABSOLUTE_PATH.'modules/shop/shop.model.php');
		$model = new Shop_model($th->diafan);
		$result = $model->show_search($cat_id, $site_id);

		include_once(ABSOLUTE_PATH."modules/shop/shop.view.php");
		$func = 'show_search';
		if($attributes["template"])
		{
			$func .= '_'.preg_replace('/[^a-z_]+/','', $attributes["template"]);
			
		}
		Shop_view::$func($result);

		return TRUE;
	}
}