<?php
/**
 * Diafan.CMS
 *
 * @author     diafan.ru
 * @version    4.3
 * @license http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2011 OOO "Диафан". (http://diafan.ru)
 */

if(! defined('DIAFAN'))
{
	include(dirname(dirname(dirname(__FILE__))).'/includes/404.php');
}

/**
 * Tags_model
 *
 * Модель модуля "Тэги"
 */
class Tags_model
{
	/**
	 * @var array сгенерированные данные, передаваемые в шаблон
	 */
	private $result;

	/**
	 * @var integer количество элементов, показанных на странице
	 */
	private $nastr = 10;

	/**
	 * Конструктор класса
	 * 
	 * @return void
	 */
	public function __construct($diafan)
	{
		$this->diafan = $diafan;
	}

	/**
	 * Геренирует список элементов, к которым прикреплен тэг
	 * 
	 * @return array
	 */
	public function list_module()
	{
		$time = mktime(23, 59, 0, date("m"), date("d"), date("Y"));
		//кеширование
		$cache_meta = array(
				    "name"     => "list",
				    "show"     => $this->diafan->show,
				    "language" => _LANG,
				    "time"     => $time,
				    "page"     => $this->diafan->page > 1 ? $this->diafan->page : 1
				    );
		if($this->result = $this->diafan->_cache->get($cache_meta, $this->diafan->module))
		{
			include_once(ABSOLUTE_PATH.'includes/paginator/paginator.view.php');
			return $this->result;
		}
		$row = DB::fetch_array(DB::query("SELECT [name] FROM {tags_name} WHERE id=%d AND trash='0'", $this->diafan->show));
		if(! $row)
		{
			include(ABSOLUTE_PATH.'includes/404.php');
		}
		//заголовок
		$this->result["title"] = $this->diafan->name.': '.$row["name"];
		
		$print_union = 0;
		$k = 0;
		$sql_count = '';
		$sql = '';

		//Страницы сайта
		if($this->diafan->configmodules("tags", "site"))
		{
			$sql_count = "SELECT COUNT(t.id) FROM {tags} AS t"
				." INNER JOIN {site} AS s ON t.element_id=s.id "
				." AND s.[act]='1' AND s.trash='0'"
				.(! $this->diafan->_user->id ? " AND s.access='0'" : "")
				." WHERE t.tags_name_id=%d AND t.trash='0' AND t.module_name='site'"
			;

			$sql = "SELECT s.id, s.[name], othurl, s.[text], '' AS site_id, '' AS cat_id, 'site' AS module"
				." FROM {tags} AS t"
				." INNER JOIN {site} AS s ON t.element_id=s.id "
				." AND s.[act]='1' AND s.trash='0'"
				.(! $this->diafan->_user->id ? " AND s.access='0'" : "")
				." WHERE t.tags_name_id=%d AND t.trash='0' AND t.module_name='site'"
			;
			$k = 1;
			$print_union = 1;
		}
			
		//Товары в интернет-магазине
		if($this->diafan->configmodules("tags", "shop"))
		{
			$sql_count .= ($print_union ? " UNION " : "")
				."SELECT COUNT(t.id) FROM {tags} as t"
				." INNER JOIN {shop} as s ON t.element_id=s.id "
				." AND s.[act]='1' AND s.trash='0'"
				." WHERE t.tags_name_id=%d AND t.trash='0' AND t.module_name='shop'";

			$sql .= ($print_union ? " UNION " : "")
				."SELECT s.id, s.[name], '' AS othurl, s.[text], s.site_id, s.cat_id, 'shop' AS module"
				." FROM {tags} as t"
				." INNER JOIN {shop} AS s ON t.element_id=s.id "
				." AND s.[act]='1' AND s.trash='0'"
				." WHERE t.tags_name_id=%d AND t.trash='0' AND t.module_name='shop'";
			$k++;
			$print_union = 1;
		}

		//Фотографии
		if($this->diafan->configmodules("tags", "photo"))
		{
			$sql_count .= ($print_union ? " UNION " : "")
				."SELECT COUNT(t.id) FROM {tags} as t"
				." INNER JOIN  {photo} as s ON t.element_id=s.id "
				." AND s.[act]='1' AND s.trash='0'"
				." WHERE t.tags_name_id=%d AND t.trash='0' AND t.module_name='photo'";

			$sql .= ($print_union ? " UNION " : "")
				."SELECT s.id,s.[name],'' AS othurl, '' AS text, s.site_id, s.cat_id, 'photo' AS module"
				." FROM {tags} as t"
				." INNER JOIN  {photo} as s ON t.element_id=s.id "
				." AND s.[act]='1' AND s.trash='0'"
				." WHERE t.tags_name_id=%d AND t.trash='0' AND t.module_name='photo'";
			$k++;
			$print_union = 1;
		}

		//Файлы
		if($this->diafan->configmodules("tags", "files"))
		{
			$sql_count .= ($print_union ? " UNION " : "")
				."SELECT COUNT(s.id) FROM {tags} as t"
				." INNER JOIN {files} as s ON t.element_id=s.id"
				." AND s.[act]='1' AND s.trash='0'"
				." WHERE t.tags_name_id=%d AND t.trash='0' AND t.module_name='files'";

			$sql .= ($print_union ? " UNION " : "")
				."SELECT s.id, s.[name], '' AS othurl, s.[text], s.site_id, s.cat_id, 'files' AS module"
				." FROM {tags} as t"
				." INNER JOIN {files} as s ON t.element_id=s.id"
				." AND s.[act]='1' AND s.trash='0'"
				." WHERE t.tags_name_id=%d AND t.trash='0' AND t.module_name='files'";
			$k++;
			$print_union = 1;
		}
			
		//Вопрос-Ответ
		if($this->diafan->configmodules("tags", "faq"))
		{
			$sql_count .= ($print_union ? " UNION " : "")
				."SELECT COUNT(s.id) FROM {tags} as t"
				." INNER JOIN {faq} as s ON t.element_id=s.id"
				." AND s.[act]='1' AND s.trash='0'"
				." WHERE t.tags_name_id=%d AND t.trash='0' AND t.module_name='faq'";

			$sql .= ($print_union ? " UNION " : "")
				."SELECT s.id, s.anons"._LANG." AS name, '' AS othurl, s.[text], s.site_id, s.cat_id, 'faq' AS module"
				." FROM {tags} as t"
				." INNER JOIN {faq} as s ON t.element_id=s.id"
				." AND s.[act]='1' AND s.trash='0'"
				." WHERE t.tags_name_id=%d AND t.trash='0' AND t.module_name='faq'";
			$k++;
			$print_union = 1;
		}

		//Новости
		if($this->diafan->configmodules("tags", "news"))
		{
			$sql_count .= ($print_union ? " UNION " : "")
				."SELECT COUNT(s.id) FROM {tags} as t"
				." INNER JOIN {news} as s ON t.element_id=s.id"
				." AND s.[act]='1' AND s.trash='0'"
				." WHERE t.tags_name_id=%d AND t.trash='0' AND t.module_name='news'";

			$sql .= ($print_union ? " UNION " : "")
				."SELECT s.id, s.[name], '' AS othurl, s.[text], s.site_id, s.cat_id, 'news' AS module"
				." FROM {tags} as t"
				." INNER JOIN {news} as s ON t.element_id=s.id"
				." AND s.[act]='1' AND s.trash='0'"
				." WHERE t.tags_name_id=%d AND t.trash='0' AND t.module_name='news'";
			$k++;
			$print_union = 1;
		}

		//Статьи
		if($this->diafan->configmodules("tags", "clauses"))
		{
			$sql_count .= ($print_union ? " UNION " : "")
				."SELECT COUNT(s.id) FROM {tags} as t"
				." INNER JOIN {clauses} as s ON t.element_id=s.id"
				." AND s.[act]='1' AND s.trash='0'"
				." WHERE t.tags_name_id=%d AND t.trash='0' AND t.module_name='clauses'";

			$sql .= ($print_union ? " UNION " : "")
				."SELECT s.id, s.[name], '' AS othurl, s.[text], s.site_id, s.cat_id, 'clauses' AS module"
				." FROM {tags} as t"
				." INNER JOIN {clauses} as s ON t.element_id=s.id"
				." AND s.[act]='1' AND s.trash='0'"
				." WHERE t.tags_name_id=%d AND t.trash='0' AND t.module_name='clauses'";
			$k++;
			$print_union = 1;
		}

		if($print_union)
		{
			include_once(ABSOLUTE_PATH."modules/tags/tags.inc.php");
			$tags = new Tags_inc($this->diafan);

			for($i = 0; $i < $k; $i++)
			{
				$values[] = $this->diafan->show;
			}
			
			$nen = 0;
			$result = DB::query($sql_count, $values);
			while ($row = DB::fetch_array($result))
			{
				$nen += $row[0];
			}

			if(! $nen)
			{
				include(ABSOLUTE_PATH.'includes/404.php');
			}

			////navigation//
			include_once(ABSOLUTE_PATH.'includes/paginator/paginator.inc.php');
			$paginator = new Paginator_inc();
			if($this->nastr)
			{
				$paginator->nastr = $this->nastr;
			}
			$paginator->page    = $this->diafan->page;
			$paginator->navlink = $this->diafan->path_module_variable("page");
			$paginator->nen = $nen;
			$links = $paginator->get();
			////navigation///

			$this->result["paginator"] = $links;

			$result = DB::query_range($sql, $values, $paginator->polog, $paginator->nastr);
			while ($row = DB::fetch_array($result))
			{
				$row["i"] = $k++;
				$row["cat"]  = array();
				$row["site"] = array();
				$row["link"] = '';

				if($row["module"] == "site")
				{
					if(! empty($this->diafan->rewrites[$row["id"]]))
					{
						$row["link"] = $this->diafan->rewrites[$row["id"]]["rewrite"]
						        ? $this->diafan->rewrites[$row["id"]]["rewrite"].'/' : '';
					}
				}
				else
				{
					if(! $this->diafan->configmodules("cat", $row["module"], _LANG, $row["site_id"]))
					{
						$row["cat_id"] = 0;
					}
					if($row["cat_id"])
					{
						if(! isset($cats[$row["module"]][$row["cat_id"]]))
						{
							$cats[$row["module"]][$row["cat_id"]] = DB::query_result("SELECT [name] FROM {".$row["module"]."_category} WHERE id=%d AND act='1' AND trash='0'", $row["cat_id"]);
						}
						if($cats[$row["module"]][$row["cat_id"]])
						{
							$link = '';
							if(! empty($this->diafan->rewrites[$row["module"]]["cat".$row["cat_id"]]))
							{
								$link = $this->diafan->rewrites[$row["module"]]["cat".$row["cat_id"]]["rewrite"].'/';
							}
							elseif(! empty($this->diafan->rewrites[$row["site_id"]]))
							{
								$link = ($this->diafan->rewrites[$row["site_id"]]["rewrite"]
								         ? $this->diafan->rewrites[$row["site_id"]]["rewrite"].'/' : '')
								        .'cat'.$row["cat_id"].'/';
							}
							$row["cat"] = array(
								     'link' => $link,
								     'name' => $cats[$row["module"]][$row["cat_id"]]
								     );
						}
					}
					if(! isset($cats[$row["module"]][$row["cat_id"]]))
					{
						$cats[$row["module"]][$row["cat_id"]] = DB::query_result("SELECT [name] FROM {".$row["module"]."_category} WHERE id=%d AND act='1' AND trash='0'", $row["cat_id"]);
					}
					if(empty($site_arr_link[$row["site_id"]]))
					{
						$site_arr_link[$row["site_id"]] = DB::query_result("SELECT [name] FROM {site} WHERE id=%d AND act='1' AND trash='0'", $row["site_id"]);
					}

					$link = '';
					if(! empty($this->diafan->rewrites[$row["site_id"]]))
					{
						$link = $this->diafan->rewrites[$row["site_id"]]["rewrite"]
						        ? $this->diafan->rewrites[$row["site_id"]]["rewrite"].'/' : '';
					}
					
					$row["site"] = array(
						     'link' => $link,
						     'name' => $site_arr_link[$row["site_id"]]
						     );
					if(! empty($this->diafan->rewrites[$row["module"]]["element".$row["id"]]))
					{
						$row["link"] = $this->diafan->rewrites[$row["module"]]["element".$row["id"]]["rewrite"].'/';
					}
					elseif($row["cat_id"] && ! empty($this->diafan->rewrites[$row["module"]]["cat".$row["cat_id"]]))
					{
						$row["link"] = $this->diafan->rewrites[$row["module"]]["cat".$row["cat_id"]]["rewrite"].'/show'.$row["id"].'/';
					}
					elseif(! empty($this->diafan->rewrites[$row["site_id"]]))
					{
						$row["link"] = ($this->diafan->rewrites[$row["site_id"]]["rewrite"]
						         ? $this->diafan->rewrites[$row["site_id"]]["rewrite"].'/' : '')
						        .($row["cat_id"] ? 'cat'.$row["cat_id"].'/' : '')
							.'show'.$row["id"].'/';
					}
				}
				$row["name"] = $this->diafan->short_text($row["name"], 20);

				$row["snippet"] = $this->diafan->short_text($row["text"], 100);
				$row["tags"] = $tags->get($row["id"], $row["module"]);
				$this->result["rows"][] = $row;
			}
		}

		$this->diafan->_cache->save($this->result, $cache_meta, $this->diafan->module);
		return $this->result;
	}

	/**
	 * Геренирует данные для облака тэгов
	 * 
	 * @return array
	 */
	function show_block()
	{
		//кеширование
		$cache_meta = array(
				    "name"     => "block",
				    "language" => _LANG
				    );
		if(! $result = $this->diafan->_cache->get($cache_meta, "tags"))
		{
			$site_id = DB::query_result("SELECT id FROM {site} WHERE addmodule='tags' AND trash='0' AND act='1'");
			if(! $site_id)
			{
				return FALSE;
			}
	
			//максимальный и минимальный размеры текста в em
			$max = 3;
			$min = 0.9;
	
			$maxr = 0;
			$minr = 10;
			$result["rows"] = array();
	
			$res = DB::query("SELECT id, [name] FROM {tags_name} WHERE trash='0' ORDER BY [name] ASC");
			while($row = DB::fetch_array($res))
			{
				$row["size"] = DB::query_result("SELECT COUNT(*) FROM {tags} WHERE tags_name_id='%d' and trash='0'", $row["id"]);
				if(! $row["size"])
				{
					continue;
				}
				$maxr = $maxr < $row["size"] ? $row["size"] : $maxr;
				$minr = $minr > $row["size"] ? $row["size"] : $minr;
	
				$row["link"] = (! empty($this->diafan->rewrites["tags"]["element".$row["id"]]["rewrite"])
						? $this->diafan->rewrites["tags"]["element".$row["id"]]["rewrite"].'/'
						: (! empty($this->diafan->rewrites[$site_id]["rewrite"])
						   ? $this->diafan->rewrites[$site_id]["rewrite"].'/' : '')
						   .'show'.$row["id"].'/'
					       );
	
				$result["rows"][] = $row;
			}
	
			if($result["rows"])
			{
				for($i = 0; $i < count($result["rows"]); $i++)
				{
					$result["rows"][$i]["size"] = ($maxr - $minr < 1 ? $min
								       : ($max - $min) * ($result["rows"][$i]["size"] - $minr) / ($maxr - $minr) + $min);
					
				}
			}
			//сохранение кеша
			$this->diafan->_cache->save($result, $cache_meta, "tags");
		}
	
		if($result["rows"])
		{
			for($i = 0; $i < count($result["rows"]); $i++)
			{
				$result["rows"][$i]["selected"] = $this->diafan->module == "tags" && $this->diafan->show == $result["rows"][$i]["id"];
				
			}
		}
		return $result;
	}
}