<?php
/**
 * Diafan.CMS
 *
 * @author     diafan.ru
 * @version    4.3
 * @license http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2011 OOO "Диафан". (http://diafan.ru)
 */

if(! defined('DIAFAN'))
{
	include(dirname(dirname(dirname(__FILE__))).'/includes/404.php');
}

/**
 * Trash_admin
 *
 * Корзина с удаленными элементами
 */
class Trash_admin extends Frame_admin
{
	/**
	 * @var string название файла. используется в имени констант, определяющих названия полей
	 */
	public $file	=	'trash';

	/**
	 * @var string таблица в базе данных
	 */
	public $table	=	'trash'; 

	/**
	 * Конструктор класса
	 * 
	 * @return void
	 */
	public function __construct($diafan)
	{
		$this->diafan = $diafan;

		$this->config_module = array(
			'del'		=> 1,	//удалить
			'datetime'	=> 1,	//показывать дату и время в списке
			'parent'	=> 1	//содержит вложенности
		);

		//выводить в списке содержание полей:
		$this->config_other_row = array(
			'element_id',
			'module_name',
			'table_name',
			'user_id'
		);
	}

	/**
	 * Выводит список удаленных в коризун элементов
	 * @return boolean true
	 */
	public function show_module()
	{
		$this->name = "''";
		echo '<table border="0" cellpadding="1" cellspacing="3" class="list">';
			$this->list_row();
		echo '</table>';
		if(! $this->count)
		{
			echo _ADMIN_TRASH_NONE;
		}

		return TRUE;
	}

	/**
	 * Восстанавливает элементы из корзины
	 * @return boolean
	 */
	public function restore_module()
	{
		if($this->diafan->restore)
		{
			$this->restore_module_id($this->diafan->restore);
		}
		elseif(! empty($_GET["ids"]))
		{
			foreach($_GET["ids"] as $id)
			{
				$this->restore_module_id(intval($id));
			}
		}
		$this->diafan->goto_diafan(URL);
	}

	/**
	 * Восстанавливает элемент из корзины
	 * 
	 * @param integer $id номер восстанавливаемого элемента
	 * @return boolean
	 */
	public function restore_module_id($id)
	{
		$row = DB::fetch_array(DB::query("SELECT table_name, element_id, module_name, id FROM {trash} WHERE id = %d", $id));

		if(! $row)
		{
			return FALSE;
		}

		//проверка прав пользователя на удаление
		if($this->diafan->admin_roles('del', $row["module_name"]))
		{
			return FALSE;
		}
		$this->diafan->_cache->delete('', $row["module_name"]);

		$this->restore_elements($row["table_name"], $row["element_id"], $row["id"]);

		return TRUE;
	}

	/**
	 * Восстанавливает связанные элементы из корзины
	 *
	 * @param string $table название таблицы
	 * @param integer $id номер восстанавливаемого элемента
	 * @param integer $trash_id номер записи в корзине
	 * @return boolean true
	 */
	private function restore_elements($table, $id, $trash_id)
	{
		DB::query("UPDATE {".$table."} SET trash='0' WHERE id=%d", $id);
		DB::query("DELETE FROM {trash} WHERE id=%d", $trash_id);

		$result = DB::query("SELECT element_id,table_name,id FROM {trash} WHERE parent_id=%d", $trash_id);

		while($row = DB::fetch_array($result))
		{
			$this->restore_elements($row["table_name"], $row["element_id"], $row["id"]);
		}

		return TRUE;
	}

	/**
	 * Удаляет элемент или элементы из корзины
	 * @return boolean
	 */
	public function del_module()
	{
		if($this->diafan->del)
		{
			$this->del_module_id($this->diafan->del);
		}
		elseif(! empty($_GET["ids"]))
		{
			foreach($_GET["ids"] as $id)
			{
				$this->del_module_id(intval($id));
			}
		}
		$this->diafan->goto_diafan(URL);
	}

	/**
	 * Удаляет элемент из корзины
	 * 
	 * @param integer $id номер восстанавливаемого элемента
	 * @return boolean
	 */
	public function del_module_id($id)
	{
		
		$row = DB::fetch_array(DB::query("SELECT table_name, element_id, id FROM {trash} WHERE id = %d", $id));

		if(! $row)
		{
			return FALSE;
		}
		$this->del_from_trash($row["table_name"], $row["element_id"], $row["id"]);
		return TRUE;
	}

	/**
	 * Выводит название удаленного объекта в списке
	 * @return string
	 */
	protected function other_row_element_id($row)
	{
		if($row["table_name"] == "images")
		{
			$photo_row = DB::fetch_array(DB::query("SELECT id, name, module_name FROM {".$row["table_name"]."} WHERE id='%d'", $row["element_id"]));
			
			if($photo_row && file_exists(ABSOLUTE_PATH.USERFILES."/".$photo_row["module_name"]."/small/".$photo_row["name"]."_".$photo_row["id"].".jpg"))
			{
				return '<img src="http://'.BASE_URL.'/'.USERFILES.'/'.$photo_row["module_name"]."/small/".$photo_row["name"]."_".$photo_row["id"].'.jpg" border="0">';
			}
		}
		if($row["table_name"] == "users" || $row["table_name"] == "users_site")
		{
			$user = DB::fetch_array(DB::query("select fio,name from {".$row["table_name"]."} where id='%d'", $row["element_id"]));
			return ' '.$user["fio"].' ('.$user["name"].')';
		}
		if($row["table_name"] == "comments" || $row["table_name"] == "forum")
		{
			$text = DB::query_result("SELECT [text] FROM {".$row["table_name"]."} WHERE id='%d'", $row["element_id"]);
			return ' '.($text ? $this->diafan->short_text($text) : $row["element_id"]);
		}
		if($row["table_name"] == "tags")
		{
			$text = DB::query_result("SELECT [name] FROM {tags_name} WHERE id IN ("
						 ." SELECT tags_name_id FROM {".$row["table_name"]."} WHERE id='%d')", $row["element_id"]);
			return ' '.($text ? $this->diafan->short_text($text) : $row["element_id"]);
		}
		if($row["table_name"] == "rating")
		{
			$rating = DB::query_result("SELECT rating FROM {".$row["table_name"]."} WHERE id='%d'", $row["element_id"]);
			return ' '.($rating ? $rating : $row["element_id"]);
		}
		if($row["table_name"] == "param_element")
		{
			$param = DB::fetch_array(DB::query("SELECT param_id, value FROM {".$row["table_name"]."} WHERE id='%d'", $row["element_id"]));
			$param_name = DB::query_result("SELECT [name] FROM {param} WHERE id='%d'", $param["param_id"]);
			return ' '.$param_name.': '.$param["value"];
		}
		if($row["table_name"] == "feedback")
		{
			return ' '._ADMIN_FEEDBACK_MESSAGE;
		}
		if($row["table_name"] == "shop_order")
		{
			return ' '._ADMIN_SHOPORDER_NUMBER.' '.$row["element_id"];
		}
		if($row["table_name"] == "rewrite")
		{
			return ' '._ADMIN_REWRITE;
		}
		$name = DB::query_result("SELECT name FROM {".$row["table_name"]."} WHERE id='%d'", $row["element_id"]);
		return ' '.($name ? $name : $row["element_id"]);
	}

	/**
	 * Выводит название модуля удаленного объекта в списке
	 * @return string
	 */
	protected function other_row_module_name($row)
	{
		$const = strtoupper('_LANG_'.$row["module_name"]);
		return '</td><td>'.(defined($const) ? constant($const) : $row["module_name"]);
	}

	/**
	 * Выводит ссылку "Восстановить"
	 * @return string
	 */
	protected function other_row_table_name($row)
	{
		return '</td><td>'.(! $row["parent_id"] ? '<a href="'.URL.'restore'.$row["id"].'/">'._LANG_RESTORE.'</a>':'');
	}

	/**
	 * Выводит имя пользователя, удалившего элемент
	 * @return string
	 */
	protected function other_row_user_id($row)
	{
		return '</td><td>'
		.(! $row["parent_id"] && $row["user_id"] ? '<a href="'.BASE_PATH_HREF.'users/edit'.$row["user_id"].'/">'.DB::title("users", $row["user_id"], "fio").'</a>':'');
	}

	/**
	 * Редактирование поля "Пользователь"
	 * @return boolean true
	 */
	protected function edit_user_id_module()
	{
		if($this->value)
		{
			echo '
			<tr>
				<td align="right">'.$this->variable_names().'</td>
				<td>
					<a href="'.BASE_PATH_HREF.'users/edit'.$this->value.'/">'.DB::title("users", $this->value, "fio").'</a>'
					.$this->show_help_row().'
				</td>
			</tr>';
		}

		return TRUE;
	}
}